<?php
/**
 * The Main file.
 *
 * @since      1.1.0
 * @package    RevivePress
 * @subpackage RevivePress\Core
 * @author     Sayan Datta <iamsayan@protonmail.com>
 */

namespace RevivePress\Core;

use WP_Post;
use RevivePress\Helpers\Hooker;
use RevivePress\Helpers\Scheduler;
use RevivePress\Helpers\HelperFunctions;

defined( 'ABSPATH' ) || exit;

/**
 * Republication class.
 */
class PostRepublish
{
	use HelperFunctions;
    use Hooker;
    use Scheduler;

	/**
	 * Register functions.
	 */
	public function register() {
		$this->action( 'wpar/global_republish_single_post', 'do_republish' );
		$this->action( 'wpar/process_republish_post', 'call_republish' );
		$this->action( 'wpar/as_action_removed', 'remove_meta' );
	}

	/**
	 * Trigger on external API call.
	 * 
	 * @since 1.3.2
	 * @param array   $args   Republish params
	 */
	public function call_republish( array $args ) {
		$method = $args['method'];
		if ( revivepress_fs()->can_use_premium_code__premium_only() ) {
			if ( $method === 'clone' ) {
				$post_id = $this->clone_old_post__premium_only( $args['post_id'], $args['single'], $args['instant'], false, true );
			} elseif ( $action === 'update' ) {
	    		$this->update_old_post( $post_id, $args['single'], $args['instant'], true, true );
	    	}
		}

		if ( 'republish' === $method ) {
			$post_id = $this->update_old_post( $args['post_id'], $args['single'], $args['instant'], false, true );
		}
	}

	/**
	 * Trigger post update process.
	 * 
	 * @since 1.1.7
	 * @param int   $post_id   Post ID
	 */
	public function do_republish( $post_id ) {
		// delete data.
		$this->delete_meta( $post_id, 'wpar_global_republish_status' );
		$this->delete_meta( $post_id, '_wpar_global_republish_datetime' );
		$this->remove_meta( $post_id );

		// Republish.
		$this->handle( (int) $post_id );
	}

	/**
	 * Delete post meta data flags.
	 * 
	 * @since 1.5.1
	 * @param int   $post_id   Post ID
	 */
	public function remove_meta( $post_id ) {
		$this->delete_meta( $post_id, 'wpar_republish_as_action_id' );
		if ( revivepress_fs()->can_use_premium_code__premium_only() ) {
			$this->delete_meta( $post_id, 'wpar_filter_republish_status' );
			$this->delete_meta( $post_id, '_wpar_filter_republish_datetime' );
			$this->delete_meta( $post_id, 'wpar_republish_rule_action' );
		}
	}

	/**
	 * Handle Trigger post update process.
	 *
	 * Override this method to perform any actions required
	 * during the async request.
	 */
	private function handle( int $post_id ) {
		$action = $this->do_filter( 'republish_action', 'repost', false, $post_id );
		if ( $action === 'repost' ) {
			$this->update_old_post( $post_id );
		}

		if ( revivepress_fs()->can_use_premium_code__premium_only() ) {
			if ( $action === 'clone' ) {
	    		$this->clone_old_post__premium_only( $post_id );
	    	} elseif ( $action === 'update' ) {
	    		$this->update_old_post( $post_id, false, false, true );
	    	}
		}
	}

	/**
	 * Run post update process.
	 * 
	 * @param int   $post_id        Post ID
	 * @param bool  $single         Check if it is a single republish event
	 * @param bool  $instant        Check if it is one click republish event
	 * @param bool  $only_update    Check if it is update date/time event
	 * @param bool  $external       Check if it is external custom event
	 * 
	 * @return int $post_id
	 */
	public function update_old_post( int $post_id, bool $single = false, bool $instant = false, bool $only_update = false, bool $external = false ) {
		$post = \get_post( $post_id );
		$new_time = $this->get_publish_time( $post->ID, $single );

		if ( ! $only_update ) {
			$pub_date = $this->get_meta( $post->ID, '_wpar_original_pub_date' );
			if ( ! $pub_date && ( $post->post_status !== 'future' ) ) {
				$this->update_meta( $post->ID, '_wpar_original_pub_date', $post->post_date );
			}
			$this->update_meta( $post->ID, '_wpar_last_pub_date', $post->post_date );
		}

		// remove kses filters
		\kses_remove_filters();

        $args = array(
	    	'post_date'     => $new_time,
	    	'post_date_gmt' => get_gmt_from_date( $new_time ),
	    );

		if ( revivepress_fs()->can_use_premium_code__premium_only() ) {
			if ( $only_update ) {
				$args = array(
					'post_modified'     => $new_time,
					'post_modified_gmt' => get_gmt_from_date( $new_time ),
				);
				$this->update_meta( $post->ID, '_wpar_update_date', $new_time );
			}
		}

		$args = array_merge( array( 'ID' => $post->ID ), $args );
		$args = $this->do_filter( 'update_process_args', $args, $post->ID, $post );
		
		wp_update_post( $args );

		if ( ! revivepress_fs()->can_use_premium_code__premium_only() ) {
		    $this->do_action( 'clear_site_cache' );
		}
		
		if ( revivepress_fs()->can_use_premium_code__premium_only() ) {
			// update reference
			$this->update_meta( $post->ID, 'wpar_republish_meta_query', $new_time );

			$this->do_action( 'insert_log', $post->ID, array(
				'action'    => $only_update ? 'update' : 'republish',
				'timestamp' => $new_time,
				'info'      => __( 'Post Updated.', 'wp-auto-republish' ),
			) );
			$this->do_action( 'old_post_republished', $post->ID, $new_time, 'republish' );
		}

		// reinit kses filters
		\kses_init_filters();

		return $post_id;
	}

	/**
	 * Run post cloning process.
	 * 
	 * @since 1.1.7
	 * @param int   $post_id    Post ID
	 * @param bool  $single     Check if a single republish event
	 * @param bool  $instant    Check if one click republish event
	 * @param bool  $scheduled  Check if scheduled republish event
	 * @param bool  $external   Check if it is external custom event
	 * 
	 * @return int $post_id
	 */
	public function clone_old_post__premium_only( int $post_id, bool $single = false, bool $instant = false, bool $scheduled = false, bool $external = false ) {
		$post = \get_post( $post_id );
		$new_time = $this->get_publish_time( $post->ID, $single, $scheduled );
		$post_name = ( $this->post_exists( $post->post_title ) ) ? $post->post_title . '-' . wp_rand( 200, 99999 ) : $post->post_name;

		// remove kses filters
		\kses_remove_filters();

		// Remove WordPress default filter so a new revision is not created on republish.
		\remove_action( 'post_updated', 'wp_save_post_revision', 10 );

		/**
		 * new post data array
		 */
		$args = array(
			'comment_status' => $post->comment_status,
			'ping_status'    => $post->ping_status,
			'post_author'    => $post->post_author,
			'post_content'   => $post->post_content,
			'post_excerpt'   => $post->post_excerpt,
			'post_name'      => ( $scheduled ) ? $post->post_name : sanitize_title( $this->do_filter( 'clone_post_link', $post_name, $post->ID ) ),
			'post_parent'    => $post->post_parent,
			'post_password'  => $post->post_password,
			'post_status'    => ( $scheduled ) ? 'future' : 'publish',
			'post_title'     => $post->post_title,
			'post_type'      => $post->post_type,
			'to_ping'        => $post->to_ping,
			'menu_order'     => $post->menu_order,
			'post_mime_type' => $post->post_mime_type,
			'post_date'      => $new_time,
	    	'post_date_gmt'  => get_gmt_from_date( $new_time ),
		);

		$args = $this->do_filter( 'clone_process_args', $args, $post->ID, $post );
	
		/**
		 * insert the post by wp_insert_post() function
		 */
		$new_post_id = wp_insert_post( $args, true );
		if ( is_wp_error( $new_post_id ) ) {
			$this->do_action( 'insert_log', $post->ID, array(
				'action' => 'clone',
				'status' => false,
 				'info'   => sprintf( 
					/* translators: error message. */
					__( 'Error Occurred: %s', 'wp-auto-republish' ), 
					$new_post_id->get_error_message() 
				),
			) );
			return $post_id;
		}
	
		/**
		 * get all current post terms and set them to the new post draft
		 */
		$taxonomies = get_object_taxonomies( $post->post_type );
		foreach ( $taxonomies as $taxonomy ) {
			$post_terms = wp_get_object_terms( $post_id, $taxonomy, array( 'fields' => 'slugs' ) );
			wp_set_object_terms( $new_post_id, $post_terms, $taxonomy, false );
		}

		/**
		 * get all current post metas ad set them to the new post
		 */
		$metas = get_post_custom( $post_id );
        foreach ( $metas as $key => $values ) {
			if ( strpos( $key, 'wpar_' ) === false ) {
            	foreach ( $values as $value ) {
            		$this->add_meta( $new_post_id, $key, $value );
		    	}
		    }
		}

		// update reference for admin column
		$this->update_meta( $post->ID, 'wpar_republish_meta_query', $new_time );
		$this->update_meta( $new_post_id, 'wpar_original_post_id', $post->ID );

		if ( $scheduled ) {
			$this->update_meta( $post->ID, 'wpar_rewrite_post_id', $new_post_id );
		} else {
			$this->do_action( 'insert_log', $post->ID, array(
				'action'    => 'clone',
				'timestamp' => $new_time,
				'info'      => sprintf( 
					/* translators: Post Title wrapped with link. */
					__( 'Post Duplicated. New Post: %s', 'wp-auto-republish' ), 
					'<a href="' . esc_url( get_edit_post_link( $new_post_id ) ) . '" target="_blank">' . _draft_or_post_title( $new_post_id ). '</a>' 
				),
			) );
			$this->do_action( 'old_post_republished', $new_post_id, $new_time, 'clone' );
		}

		// Re-enable the creation of a new revision.
		\add_action( 'post_updated', 'wp_save_post_revision', 10, 1 );

		// reinit kses filters
		kses_init_filters();

		return $new_post_id;
	}

	/**
	 * Get new post published time.
	 * 
	 * @since 1.1.7
	 * @param int     $post_id  Post ID
	 * @param bool    $single 
	 * 
	 * @return string
	 */
	protected function get_update_action__premium_only( int $post_id, bool $single = false ) {
		$action = $this->get_data( 'wpar_republish_action', 'repost' );
		if ( $single ) {
	    	$action = $this->get_data( 'wpar_single_republish_action', 'repost' );
	    }

		$get_action = $this->get_meta( $post_id, '_wpar_repost_post_update_action' );
		if ( ! empty( $get_action ) && ( 'default' !== $get_action ) ) {
			$action = $get_action;
		}

		return $action;
	}

	/**
	 * Get new post published time.
	 * 
	 * @since 1.1.7
	 * @param int   $post_id   Post ID
	 * @param bool  $single    Check if a single republish event
	 * @param bool  $scheduled Check if scheduled republish event
	 * 
	 * @return string
	 */
	private function get_publish_time( int $post_id, bool $single = false, bool $scheduled = false ) {
		$post = \get_post( $post_id );
    	$timestamp = $this->current_timestamp();
		$interval = MINUTE_IN_SECONDS * $this->do_filter( 'second_position_interval', wp_rand( 1, 15 ) );

    	$new_time = current_time( 'mysql' );
		if ( $this->get_data( 'wpar_republish_post_position', 'one' ) == 'one' ) {
    		$datetime = $this->get_meta( $post_id, '_wpar_global_republish_datetime' );
			if ( ! empty( $datetime ) && ( $timestamp >= strtotime( $datetime ) ) ) {
			    $new_time = $datetime;
			}
    	} else {
			$args = ( array ) $this->do_filter( 'republish_position_args', array(
    			'post_type'   => $post->post_type,
                'numberposts' => 1,
    			'offset'      => 1,
    			'post_status' => 'publish',
				'order'       => 'DESC',
				'orderby'     => 'date',
				'fields'      => 'ids',
    		), $post );
    		$lastposts = $this->get_posts( $args );

    		if ( ! empty( $lastposts ) ) {
    		    foreach ( $lastposts as $lastpost ) {
					$post_date = get_the_date( 'U', $lastpost );
					$post_date = $post_date + $interval;
    		    	$new_time = date( 'Y-m-d H:i:s', $post_date );
    		    }
    	    }
		}
		
		if ( revivepress_fs()->can_use_premium_code__premium_only() ) {
			if ( $single ) {
				$datetime = $this->get_meta( $post_id, '_wpar_repost_schedule_datetime' );
				if ( ! empty( $datetime ) && ( $timestamp >= strtotime( $datetime ) ) ) {
				    $new_time = $datetime;
				}
			}
			
			if ( $scheduled ) {
				$new_time = date( 'Y-m-d H:i:s', $timestamp + 3600 );
			}
		}

		return $this->do_filter( 'next_scheduled_timestamp', $new_time, $post_id, $single, $scheduled );
	}
}