<?php
/**
 * Admin column.
 *
 * @since      1.1.0
 * @package    RevivePress
 * @subpackage RevivePress\Core\Premium
 * @author     Sayan Datta <iamsayan@protonmail.com>
 */

namespace RevivePress\Core\Premium;

use RevivePress\Helpers\Hooker;
use RevivePress\Helpers\Premium\GetMetaData;

defined( 'ABSPATH' ) || exit;

/**
 * Admin column class.
 */
class AdminColumnView
{
	use GetMetaData;
    use Hooker;

	/**
	 * Register functions.
	 */
	public function register() {
		$this->action( 'init', 'column' );
		$this->action( 'admin_head-edit.php', 'style' );
	}

	/**
	 * Register admin columns.
	 */
	public function column() {
		$post_types = array_unique( array_merge( $this->get_data( 'wpar_post_types', array( 'post' ) ), $this->get_data( 'post_types_list_single', array( 'post' ) ) ) );
		foreach ( $post_types as $post_type ) {
			$this->filter( "manage_edit-{$post_type}_columns", 'register_column', 10, 1 );
			$this->filter( "manage_{$post_type}_posts_columns", 'register_column', 10, 1 );
			$this->action( "manage_{$post_type}_posts_custom_column", 'column_data', 10, 2 );
		}
	}

	/**
	 * Register Column.
	 * 
	 * @param string   $column  Column name
	 * @return string  $column  Filtered column
	 */
	public function register_column( $column ) {
		$column['republish'] = __( 'Republish Info', 'wp-auto-republish' );
		
		return $column;
	}

	/**
	 * Generate column data.
	 * 
	 * @param string   $column   Column name
	 * @param int      $post_id  Post ID
	 * 
	 * @return string  $content
	 */
	public function column_data( $column, $post_id ) {
		switch ( $column ) {
			case 'republish':
				$timestamp = $this->current_timestamp();
				$post = \get_post( $post_id );

				$timestamp = $this->current_timestamp();
				$published_date = $this->get_meta( $post->ID, '_wpar_original_pub_date' );
				$global_pending = $this->get_meta( $post->ID, 'wpar_global_republish_status' );
				$global_schedule = $this->get_meta( $post->ID, '_wpar_global_republish_datetime' );
				$single_pending = $this->get_meta( $post->ID, 'wpar_single_republish_status' );
				$single_schedule = $this->get_meta( $post->ID, '_wpar_repost_schedule_datetime' );

				$format = $this->do_filter( 'admin_column_date_format', 'M j, Y' );
				$full_format = $this->do_filter( 'admin_column_full_datetime_format', get_option( 'date_format' ) . ' @ ' . get_option( 'time_format' ) );
				
				$orig_id = $this->get_meta( $post->ID, 'wpar_original_post_id' );

				$in_queue = false;
				if ( $global_pending && $global_schedule ) {
					$in_queue = $global_schedule;
				}

				if ( $single_pending && $single_schedule ) {
					$in_queue = $single_schedule;
				}

				// Republish Details
				if ( $orig_id ) {
					$in_queue = get_the_time( 'Y-m-d H:i:s', $post );
					$content = sprintf( '<span>%1$s: <strong><a href="%2$s" target="_blank">%3$s</a></strong></span><br />',
						__( 'Original', 'wp-auto-republish' ),
						get_the_permalink( $orig_id ),
						wp_trim_words( get_the_title( $orig_id ), 8 )
					);
				} elseif ( $published_date ) {
						$content = sprintf(
							'<span title="%1$s">%2$s: <strong>%3$s</strong></span><br />
							<span title="%4$s">%5$s: <strong>%6$s</strong></span><br />',
							get_the_date( $full_format, $post ), __( 'Republished', 'wp-auto-republish' ), get_the_date( $format, $post ),
							date_i18n( $full_format, strtotime( $published_date ) ), __( 'First Published', 'wp-auto-republish' ), date_i18n( $format, strtotime( $published_date ) )
						);
					} else {
						$content = sprintf(
							/* translators: Post publish time in human readable format. */
							__( 'Not republished yet.<br />Published %s ago<br />', 'wp-auto-republish' ),
							human_time_diff( get_the_time( 'U', $post ), $this->current_timestamp() )
						);
				}

				// Upcoming republish event details
				if ( $in_queue ) {
					$new_timestamp = strtotime( $in_queue );
					$timediff = $new_timestamp - $timestamp;
					if ( ( $timediff <= 86400 ) && ( $timediff > 0 ) ) {
						$content .= sprintf(
							'<abbr title="%1$s">%2$s %3$s</abbr><br />',
							date_i18n( $full_format, $new_timestamp ),
							__( 'Republication starts in', 'wp-auto-republish' ),
							human_time_diff( $timestamp, $new_timestamp )
						);
					} elseif ( $timediff <= 0 ) {
						$content .= sprintf(
							'<abbr title="%1$s">%2$s</abbr><br />',
							date_i18n( $full_format, $new_timestamp ),
							__( 'Republication in queue', 'wp-auto-republish' )
						);
					} elseif ( $timediff > 86400 ) {
						$content .= sprintf(
							'<abbr title="%1$s">%2$s: <strong>%3$s</strong></abbr><br />',
							date_i18n( $full_format, $new_timestamp ), __( 'Next Republish', 'wp-auto-republish' ), date_i18n( $format, $new_timestamp )
						);
					}
					
					// show republish schedules
					$content .= $this->get_schedule( $post->ID, 'edit', true );
				}

				if ( 'publish' !== $post->post_status ) {
					$content = __( 'Post is not published.', 'wp-auto-republish' );
				}

				echo wp_kses_post( $content );
			    break;
		}
	}

	/**
	 * Column custom CSS.
	 */
	public function style() {
		echo '<style type="text/css">.fixed th.column-republish { width: 200px; }</style>'."\n";
	}
}