<?php
/**
 * Dashboard widget.
 *
 * @since      1.1.0
 * @package    RevivePress
 * @subpackage RevivePress\Core\Premium
 * @author     Sayan Datta <iamsayan@protonmail.com>
 */

namespace RevivePress\Core\Premium;

use WP_Query;
use RevivePress\Helpers\Hooker;
use RevivePress\Helpers\HelperFunctions;

defined( 'ABSPATH' ) || exit;

/**
 * Dashboard widget class.
 */
class DashboardWidget
{
	use HelperFunctions;
    use Hooker;

	/**
	 * Register functions.
	 */
	public function register() {
		$this->action( 'wp_dashboard_setup', 'dashboard_widget' );
		$this->action( 'admin_head-index.php', 'widget_css' );
	}

	/**
	 * Register dashboard widgets.
	 */
	public function dashboard_widget() {
		global $wp_meta_boxes;
		
		wp_add_dashboard_widget(
			'dashboard_republish_posts', // Widget slug.
			__( 'Republish Activity', 'wp-auto-republish' ), // Title.
			array( $this, 'widget_callback' ), // callback.
			array( $this, 'widget_control_callback' ) // control callback.
	    );
	}

	/**
	 * Dashboard widget callback.
	 * 
	 * @param string $widget_id Widget ID
	 */
	public function widget_callback( $widget_id ) {
		$widget_content = '';

        echo '<div id="activity-widget">';

		$widget_content .= $this->activity_block( array(
			'type'        => 'global',
			'block_title' => __( 'Global Republish Scheduled', 'wp-auto-republish' ),
			'query_args'  => array(
				'meta_key'   => '_wpar_global_republish_datetime',
				'meta_query' => array(
					'relation' => 'AND',
					array(
						'key'     => '_wpar_global_republish_datetime',
						'compare' => 'EXISTS',
					),
					array(
						'key'     => 'wpar_global_republish_status',
						'compare' => 'EXISTS',
					),
				),
			),
		) );

		$widget_content .= $this->activity_block( array(
			'type'        => 'single',
			'block_title' => __( 'Single Republish Scheduled', 'wp-auto-republish' ),
			'query_args'  => array(
				'meta_key'   => '_wpar_repost_schedule_datetime',
				'meta_query' => array(
					'relation' => 'AND',
					array(
						'key'     => '_wpar_repost_schedule_datetime',
						'compare' => 'EXISTS',
					),
					array(
						'key'     => 'wpar_single_republish_status',
						'compare' => 'EXISTS',
					),
				),
			),
		) );

		if ( $widget_content ) {
			echo $widget_content; // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
		} else {
			echo '<div class="no-activity">';
			echo '<p>' . esc_html__( 'No activity yet!', 'wp-auto-republish' ) . '</p>';
			echo '</div>';
		}

		echo '</div>';
	}
	
	/**
	 * Dashboard widget control callback.
	 */
	private function activity_block( $args ) {
		// get widget options
		$widget_options = get_option( 'wpar_dashboard_widget_options' );
		$post_count = ! empty( $widget_options['number'] ) ? intval( $widget_options['number'] ) : 5;
		
		$query_args = wp_parse_args( $args['query_args'], array(
			'post_type'        => 'any',
			'post_status'      => 'publish',
			'posts_per_page'   => $post_count,
			'order'            => 'ASC',
			'orderby'          => 'meta_value',
			'no_found_rows'    => true,
			'suppress_filters' => true,
		) );

		$posts = new WP_Query( $this->do_filter( 'dashboard_widget_' . $args['type'] . '_args', $query_args ) );

		$content = '';
	    if ( $posts->have_posts() ) {

			$content .= '<div id="wpar-pending-' . $args['type'] . '-posts" class="activity-block wpar-pending-posts">';
			$content .= '<h3>' . $args['block_title'] . '</h3>';
			$content .= '<ul>';

			$today    = date( 'Y-m-d', $this->current_timestamp() );
		    $tomorrow = date( 'Y-m-d', strtotime( '+1 day', $this->current_timestamp() ) );

            while ( $posts->have_posts() ) {
				$posts->the_post();
	
				$datetime = $this->get_meta( get_the_ID(), $args['query_args']['meta_key'] );
				$time = strtotime( $datetime );
			    if ( date( 'Y-m-d', $time ) == $today ) {
			    	$relative = __( 'Today' ); // phpcs:ignore WordPress.WP.I18n.MissingArgDomain
			    } elseif ( date( 'Y-m-d', $time ) == $tomorrow ) {
			    	$relative = __( 'Tomorrow' ); // phpcs:ignore WordPress.WP.I18n.MissingArgDomain
			    } elseif ( date( 'Y', $time ) !== date( 'Y', $this->current_timestamp() ) ) {
			    	/* translators: Date and time format for recent posts on the dashboard, from a different calendar year, see https://www.php.net/date */
			    	$relative = date_i18n( 'M jS Y', strtotime( $datetime ) );
			    } else {
			    	/* translators: Date and time format for recent posts on the dashboard, see https://www.php.net/date */
			    	$relative = date_i18n( 'M jS', strtotime( $datetime ) );
			    }
				
				// Use the post edit link for those who can edit, the permalink otherwise.
			    $recent_post_link = current_user_can( 'edit_post', get_the_ID() ) ? get_edit_post_link() : get_permalink();
    
			    $draft_or_post_title = _draft_or_post_title();
			    $content .= sprintf(
			    	'<li><span>%1$s</span> <a href="%2$s" aria-label="%3$s">%4$s</a></li>',
			    	/* translators: 1: Relative date, 2: Time. */
			    	sprintf( _x( '%1$s, %2$s', 'dashboard' ), $relative, date_i18n( get_option( 'time_format' ), strtotime( $datetime ) ) ),  // phpcs:ignore WordPress.WP.I18n.MissingArgDomain
			    	$recent_post_link,
			    	/* translators: %s: Post title. */
			    	esc_attr( sprintf( __( 'Edit &#8220;%s&#8221;' ), $draft_or_post_title ) ), // phpcs:ignore WordPress.WP.I18n.MissingArgDomain
					$draft_or_post_title
				);
			}
						
		    $content .= '</ul>';
			$content .= '</div>';
				
		}
		wp_reset_postdata();

	    return $content;
	}

	/**
	 * Dashboard widget control callback.
	 */
	public function widget_control_callback() {
		// Update widget options
		if ( isset( $_SERVER['REQUEST_METHOD'] ) && 'POST' === $_SERVER['REQUEST_METHOD'] && isset( $_POST['wpar_widget_settings'] ) ) {
			check_admin_referer();
			update_option( 'wpar_dashboard_widget_options', array_map( 'intval', wp_unslash( $_POST['wpar_widget_settings'] ) ), false );
		}

		// Get widget options
		$widget_options = get_option( 'wpar_dashboard_widget_options' );
		$value = isset( $widget_options['number'] ) ? $widget_options['number'] : '';

		wp_nonce_field(); ?>
		<p>
			<label for="post-count"><strong><?php esc_html_e( 'No. of Posts to Display on this Widget', 'wp-auto-republish' ); ?>:</strong></label>
			&nbsp;&nbsp;&nbsp;<input class="widefat" id="post-count" name="wpar_widget_settings[number]" type="number" size="15" style="width:15%;vertical-align: middle;" placeholder="5" min="3" max="100" value="<?php echo esc_attr( $value ); ?>" />
		</p>
		<?php
	}

	/**
	 * Locad custom css for dashboard widget.
	 */
	public function widget_css() { ?>
		<style type="text/css">
			#dashboard_republish_posts .no-activity p {
				color: #72777c;
				font-size: 16px;
			}
			#dashboard_republish_posts .no-activity {
				overflow: hidden;
				padding: 12px 0;
				text-align: center;
			}
			#dashboard_republish_posts .inside {
				margin: 0;
				padding-bottom: 0;
			}
			#dashboard_republish_posts .dashboard-widget-control-form {
				padding-top: 10px;
				padding-bottom: 15px;
			}
			.wpar-pending-posts ul span {
                display: inline-block;
                margin-right: 5px;
                min-width: 150px;
                color: #72777c;
            }
			.wpar-open-link {
				font-size: 15px;
				margin-left: 0px;
				min-width: 0px !important;
				display: inline !important;
			}
			#wpar-pending-global-posts li, #wpar-single-global-posts li {
                margin-bottom: 8px;
            }
			#wpar-pending-single-posts ul {
                clear: both;
                margin-bottom: 0;
            }
		</style>
	<?php }
}