<?php
/**
 * Helper functions.
 *
 * @since      1.2.0
 * @package    RevivePress
 * @subpackage RevivePress\Core\Premium
 * @author     Sayan Datta <iamsayan@protonmail.com>
 */

namespace RevivePress\Core\Premium;

use RevivePress\Helpers\Hooker;
use RevivePress\Helpers\Scheduler;
use RevivePress\Helpers\Premium\Extras;
use RevivePress\Helpers\HelperFunctions;

defined( 'ABSPATH' ) || exit;

/**
 * Logger class.
 */
class EventLogger
{
	use Extras;
    use Hooker;
    use HelperFunctions;
    use Scheduler;

	/**
	 * Republish Log post type.
	 *
	 * @var string
	 */
	private $post_type = 'republish_log';
	
	/**
	 * Register functions.
	 */
	public function register() {
		$this->action( 'init', 'register_cpt', 11 );
		$this->action( 'init', 'schedule_cleanup', 15 );
		$this->action( 'admin_init', 'delete_log' );
		$this->filter( 'get_edit_post_link', 'filter_edit_link', 10, 2 );
		$this->filter( 'post_row_actions', 'row_actions', 99, 2 );
		$this->filter( "bulk_actions-edit-$this->post_type", 'bulk_actions', 20 );
		$this->filter( "handle_bulk_actions-edit-$this->post_type", 'bulk_delete', 10, 3 );
		$this->action( 'wpar/insert_log', 'insert_log', 10, 3 );
		$this->action( 'wpar/remove_log_history', 'run_cleanup' );
		$this->action( 'wpar/remove_log_history_task', 'run_cleanup_task' );
		add_filter( "views_edit-$this->post_type", '__return_empty_array' );
	}

	/**
	 * Register Custom post type
	 */
	public function register_cpt() {
		register_extended_post_type( $this->post_type,
		array(
			'supports'            => array( 'title', 'custom-fields', 'author' ),
	    	'hierarchical'        => false,
	    	'public'              => false,
	    	'show_ui'             => true,
	    	'show_in_menu'        => 'revivepress',
	    	'show_in_admin_bar'   => false,
	    	'show_in_nav_menus'   => false,
	    	'can_export'          => false,
	    	'has_archive'         => false,
	    	'exclude_from_search' => true,
	    	'publicly_queryable'  => false,
			'capabilities'        => array_merge( $this->get_capabilities( $this->post_type ), array(
				'create_posts' => 'do_not_allow',
			) ),
			'map_meta_cap'        => true,
			'rewrite'             => false,
			'query_var'           => false,
			'labels'              => array(
				'all_items' => __( 'Event Logs', 'wp-auto-republish' ),
			),
			'admin_cols'          => array(
				'title'         => array(
					'title' => __( 'Post Title', 'wp-auto-republish' ),
				),
				'rvp_post_type' => array(
					'title'    => __( 'Post Type', 'wp-auto-republish' ),
					'meta_key' => 'rvp_post_type',
				),
				'rvp_timestamp' => array(
					'title'    => __( 'Timestamp', 'wp-auto-republish' ),
					'function' => function() {
						global $post;
						$timestamp = $this->get_meta( $post->ID, 'rvp_timestamp' );
						$format = get_option( 'date_format' ) . ' @ ' . get_option( 'time_format' );
						$content = '—';
						if ( ! empty( $timestamp ) ) {
							$content = date_i18n( $format, strtotime( $timestamp ) );
						}
						echo esc_html( $content );
					},
				),
				'rvp_action'    => array(
					'title'    => __( 'Republish Action', 'wp-auto-republish' ),
					'function' => function() {
						global $post;
						$action = $this->get_meta( $post->ID, 'rvp_action' );
						$action_list = $this->action_list();
						if ( isset( $action_list[ $action ] ) ) {
							echo esc_html( $action_list[ $action ] );
						}
					},
				),
				'rvp_status'    => array(
					'title'    => __( 'Status', 'wp-auto-republish' ),
					'function' => function() {
						global $post;
						$status = $this->get_meta( $post->ID, 'rvp_status' );
						$text = __( 'Error', 'wp-auto-republish' );
						$color = 'red';

						if ( $status == 'success' ) {
							$text = __( 'Success', 'wp-auto-republish' );
							$color = 'green';
						}

						echo '<span style="color: ' . esc_attr( $color ) . '">' . esc_html( $text ) . '</span>';
					},
				),
				'rvp_details'   => array(
					'title'    => __( 'Details', 'wp-auto-republish' ),
					'function' => function() {
						global $post;
						$details = $this->get_meta( $post->ID, 'rvp_details' );
						$content = ( ! empty( $details ) ) ? $details : '—';

						echo wp_kses_post( $content );
					},
				),
			),
			'admin_filters'       => array(
				'rvp_post_type' => array(
					'title'    => __( 'Filter Post Type', 'wp-auto-republish' ),
					'meta_key' => 'rvp_post_type',
				),
				'rvp_status'    => array(
					'title'    => __( 'Filter Status', 'wp-auto-republish' ),
					'meta_key' => 'rvp_status',
					'options'  => array(
						'success' => __( 'Success', 'wp-auto-republish' ),
						'error'   => __( 'Error', 'wp-auto-republish' ),
					),
				),
				'rvp_action'    => array(
					'title'    => __( 'Filter Action', 'wp-auto-republish' ),
					'meta_key' => 'rvp_action',
					'options'  => $this->action_list(),
				),
			),
		),
		array(
			'singular' => __( 'Republish Log', 'wp-auto-republish' ),
			'plural'   => __( 'Republish Logs', 'wp-auto-republish' ),
			'slug'     => $this->post_type,
		) );
	}

	/**
	 * Schedule log cleanup task
	 */
	public function schedule_cleanup() {
		$interval = $this->do_filter( 'log_remove_interval', 1 );
		$has_action = $this->has_next_action( 'wpar/remove_log_history' );
		if ( ! $has_action ) {
			$this->schedule_recurring_action( strtotime( '+1 minute' ), DAY_IN_SECONDS * $interval, 'wpar/remove_log_history' );
		}
	}

	/**
	 * Generate log infos.
	 *
	 * @param int    $post_id   Post ID to add in log.
	 * @param mixed  $args      Log Args.
	 */
	public function insert_log( $post_id, $args ) {
		$defaults = array(
			'action'    => 'republish',
            'status'    => true,
            'info'      => '',
			'timestamp' => current_time( 'mysql' ),
        );
		if ( is_array( $args ) ) {
			$args = wp_parse_args( $args, $defaults ); 
		} else {
			$defaults['action'] = $args;
			$args = $defaults;
		}

		$post = \get_post( $post_id );
		$log_args = array(
			'post_status' => 'publish',
			'post_title'  => $post->post_title,
			'post_type'   => $this->post_type,
			'post_date'   => current_time( 'mysql' ),
		);

		$log_id = wp_insert_post( $log_args, true );
		$status = $args['status'] ? 'success' : 'error';

		if ( ! is_wp_error( $log_id ) ) {
			$this->update_meta( $log_id, 'rvp_post_id', $post_id );
			$this->update_meta( $log_id, 'rvp_timestamp', $args['timestamp'] );
			$this->update_meta( $log_id, 'rvp_post_type', get_post_type_object( get_post_type( $post_id ) )->labels->singular_name );
			$this->update_meta( $log_id, 'rvp_action', $args['action'] );
			$this->update_meta( $log_id, 'rvp_status', $status );
			$this->update_meta( $log_id, 'rvp_details', $args['info'] );
			$this->update_meta( $post->ID, 'rvp_log_id', $log_id );
		}
	}

	/**
	 * Filter edit link.
	 *
	 * @param int    $link      Post link.
	 * @param mixed  $post_id   Post ID to add in log.
	 */
	public function filter_edit_link( $link, $post_id ) {
		if ( $this->post_type === get_post_type( $post_id ) ) {
			$republish_post_id = $this->get_meta( $post_id, 'rvp_post_id' );
			$post = get_post( $republish_post_id );
			if ( $post ) {
				\remove_filter( 'get_edit_post_link', array( $this, 'filter_edit_link' ) );
				$link = get_edit_post_link( $post->ID );
				\add_filter( 'get_edit_post_link', array( $this, 'filter_edit_link' ), 15, 2 );
			}
		}
		return $link;
	}

	/**
	 * Register post row action.
	 * 
	 * @param array   $actions  Row actions
	 * @param object  $post     Post Object
	 * 
	 * @return array  $actions  Filtered Row actions
	 */
	public function row_actions( $actions, $post ) {
		if ( $this->post_type === $post->post_type ) {
			$post_type_object = get_post_type_object( $post->post_type );
			if ( ! $post_type_object ) {
				return array();
			}
			
			$delete_link = add_query_arg( 'rvp_action', 'delete', admin_url( sprintf( $post_type_object->_edit_link, $post->ID ) ) );

			$actions = array();
			$actions['delete'] = sprintf(
				'<a href="%s" class="submitdelete" aria-label="%s">%s</a>',
				wp_nonce_url( $delete_link, "delete_republish_log_{$post->ID}" ),
				__( 'Delete Log', 'wp-auto-republish' ),
				__( 'Delete', 'wp-auto-republish' )
			);
		}
		return $actions;
	}

	/**
	 * Handle log item deletion.
	 */
	public function delete_log() {
		if ( ! isset( $_GET['rvp_action'], $_GET['post'] ) || empty( $_GET['post'] ) || empty( $_GET['rvp_action'] ) ) {
			return;
		}

		if ( 'delete' !== $_GET['rvp_action'] ) {
			return;
		}
	
		$post_id = intval( $_GET['post'] );
		
		// check nonce
		check_admin_referer( 'delete_republish_log_' . $post_id );
		
		$post = \get_post( $post_id );
		if ( ! $post ) {
			\wp_die( \esc_html__( 'An error occurred while performing RevivePress Actions.', 'wp-auto-republish' ) );
		}
		wp_delete_post( $post->ID, true );

		$redirect_url = add_query_arg( array(
			'post_type' => $post->post_type,
			'deleted'   => 1,
		), admin_url( 'edit.php') );

		wp_safe_redirect( $redirect_url );
		exit;
	}

	/**
	 * Remove unneeded bulk actions.
	 *
	 * @since 1.3.2
	 * @param  array $actions Actions.
	 * @return array          New actions.
	 */
	public function bulk_actions( $actions ) {
		$actions = array();
		$actions['rvp_delete'] = __( 'Delete', 'wp-auto-republish' );
		return $actions;
	}

	/**
	 * Fires when a custom bulk action should be handled.
	 *
	 * @since 1.3.4
	 *
	 * @param string $redirect The redirect URL.
	 * @param string $doaction The action being taken.
	 * @param array  $post_ids The items to take the action on. Accepts an array of IDs of posts,
	 *                         comments, terms, links, plugins, attachments, or users.
	 */
	public function bulk_delete( $redirect, $doaction, $post_ids ) {
		if ( 'rvp_delete' === trim( $doaction ) ) {
			$deleted = 0;
			foreach ( (array) $post_ids as $post_id ) {
				wp_delete_post( $post_id, true );
				++$deleted;
			}
			$redirect = add_query_arg( 'deleted', $deleted, $redirect );
		}
		return $redirect;
	}

	/**
	 * Republish log cleanup.
	 */
	public function run_cleanup() {
		$log_history = $this->get_data( 'log_history_duration', 'unlimited' );
		if ( 'unlimited' !== $log_history ) {
			$timestamp = $this->current_timestamp();
			$before_date = date( 'Y-m-d H:i:s', strtotime( "-$log_history days", $timestamp ) );

			$args = array(
				'post_type'      => $this->post_type,
				'posts_per_page' => -1,
				'post_status'    => 'any',
				'fields'         => 'ids',
				'date_query'     => array(
					array(
						'before'    => $before_date,
						'inclusive' => true,
					),
				),
			);

			$post_ids = $this->get_posts( $args );
			$this->schedule_batch_actions( $post_ids, 'wpar/remove_log_history_task' );
		}
	}

	/**
	 * Republish log cleanup task.
	 */
	public function run_cleanup_task( array $post_ids ) {
		foreach ( $post_ids as $post_id ) {
			wp_delete_post( $post_id, true );
		}
	}

	/**
	 * List of available log actions.
	 */
	public function action_list() {
		return array(
			'republish'      => __( 'Republish Post', 'wp-auto-republish' ),
			'clone'          => __( 'Duplicate Post', 'wp-auto-republish' ),
			//'update'         => __( 'Update Post', 'wp-auto-republish' ),
			'scheduled'      => __( 'Scheduled Post', 'wp-auto-republish' ),
			'email'          => __( 'Email Sent', 'wp-auto-republish' ),
			'trigger'        => __( 'Trigger Publish', 'wp-auto-republish' ),
			'cache'          => __( 'Cache Cleared', 'wp-auto-republish' ),
			'cron'           => __( 'Global Scheduled Queue', 'wp-auto-republish' ),
			'single_cron'    => __( 'Single Scheduled Queue', 'wp-auto-republish' ),
			'facebook_page'  => __( 'Facebook Page Share', 'wp-auto-republish' ),
			'facebook_group' => __( 'Facebook Group Share', 'wp-auto-republish' ),
			'twitter'        => __( 'Twitter Share', 'wp-auto-republish' ),
			'linkedin'       => __( 'Linkedin Share', 'wp-auto-republish' ),
			'pinterest'      => __( 'Pinterest Share', 'wp-auto-republish' ),
			'tumblr'         => __( 'Tumblr Share', 'wp-auto-republish' ),
		);
	}
}