<?php
/**
 * Filters.
 *
 * @since      1.3.4
 * @package    RevivePress
 * @subpackage RevivePress\Core\Premium
 * @author     Sayan Datta <iamsayan@protonmail.com>
 */

namespace RevivePress\Core\Premium;

use RevivePress\Helpers\Hooker;
use RevivePress\Helpers\Sitepress;
use RevivePress\Helpers\HelperFunctions;

defined( 'ABSPATH' ) || exit;

/**
 * Republication class.
 */
class Filters
{
	use HelperFunctions;
    use Hooker;

	/**
	 * Register functions.
	 */
	public function register() {
		$this->filter( 'wpar/republish_action', 'filter_action', 10, 3 );
		$this->filter( 'wpar/update_process_args', 'filter_title', 10, 2 );
		$this->filter( 'wpar/clone_process_args', 'filter_title', 10, 2 );
		$this->filter( 'wpar/filtered_post_ids', 'set_post_ids', 99 );
		$this->filter( 'wpar/minimum_republish_interval', 'custom_interval', 99 );
		$this->filter( 'wpar/republish_eligibility_age', 'post_ages' );
		$this->filter( 'wpar/random_republish_interval', 'random_interval' );
		$this->filter( 'wpar/republish_orderby_items', 'republish_orderby' );
		$this->filter( 'pll_copy_post_metas', 'copy_post_metas', 10, 3 ); // Polylang support.
	}

	/**
	 * Set post republish type.
	 * 
	 * @since 1.3.4
	 * @param string  $action   Default action
	 * @param bool    $single 
	 * @param int     $post_id  Post ID
	 * 
	 * @return string
	 */
	public function filter_action( $action, $single, $post_id ) {
		$action = $this->get_data( 'wpar_republish_action', $action );
		if ( $single ) {
	    	$action = $this->get_data( 'wpar_single_republish_action', 'repost' );
	    }
		$rule_action = $this->get_meta( $post_id, 'wpar_republish_rule_action' );
		$action = ( $rule_action ) ? $rule_action : $action;

		$get_action = $this->get_meta( $post_id, '_wpar_repost_post_update_action' );
		if ( ! empty( $get_action ) && ( $get_action != 'default' ) ) {
			$action = $get_action;
		}

		return $action;
	}

	/**
	 * Get New Post title and URL if present.
	 * 
	 * @since 1.1.7
	 * @param array  $args    Republish arguments
	 * @param int    $post_id Post ID
	 * 
	 * @return array
	 */
	public function filter_title( $args, $post_id ) {
		$post = \get_post( $post_id );
		$action = $this->get_meta( $post->ID, 'wpar_republish_rule_action' );
        if ( $action ) {
			return $args;
		}

		$get_new_title = $this->get_meta( $post->ID, '_wpar_repost_post_titles' );
    	$new_title = is_array( $get_new_title ) ? $get_new_title : explode( ';;;', $get_new_title );
    	$build_title = array_diff( $new_title, array( $post->post_title ) );
		
		if ( count( $build_title ) >= 1 ) {
            $index = array_rand( $build_title, 1 );
    		$new_title = wp_strip_all_tags( $build_title[ $index ] );

			$remove_title = $this->get_meta( $post->ID, '_wpar_repost_post_remove_title' );
			if ( $remove_title == 'yes' ) {
				$key = array_search( $build_title[ $index ], $new_title );
				if ( $key !== false ) {
					unset( $new_title[ $key ] );
					$this->update_meta( $post->ID, '_wpar_repost_post_remove_title', $new_title );
				}
			}
			
			$title_on = $this->get_meta( $post->ID, '_wpar_repost_post_titles_active' );
			if ( $title_on == 'yes' ) {
				$args['post_title'] = $this->do_filter( 'republished_post_title', $new_title, $post_id );
				
				$update_url = $this->get_meta( $post->ID, '_wpar_repost_post_update_url' );
    			if ( $update_url == 'yes' ) {
					$post_name = ( $this->post_exists( $new_title ) ) ? $new_title . '-' . wp_rand( 200, 99999 ) : $new_title;
					$args['post_name'] = sanitize_title( $this->do_filter( 'republished_post_link', $post_name, $post_id ) );
				}
    		}
		}

		return $args;
	}

	/**
	 * Add custom republish intervals.
	 */
	public function set_post_ids( $ids ) {
		if ( ! $this->is_enabled( 'enable_republish_translated' ) ) {
			return $ids;
		}

		// WPML Support.
		$languages = Sitepress::get()->get_languages();
		if ( ! empty( $languages ) ) {
			foreach ( $languages as $language ) {
				foreach ( $ids as $post_id ) {
					$post_ids[] = \apply_filters( 'wpml_object_id', $post_id, get_post_type( $post_id ), false, $language['code'] );
				}
			}
		}

		// Polylang Support.
		$languages = function_exists( 'pll_languages_list' ) ? pll_languages_list() : array();
		if ( ! empty( $languages ) && function_exists( 'pll_get_post' ) ) {
			foreach ( $languages as $language ) {
				foreach ( $ids as $post_id ) {
					$post_ids[] = \pll_get_post( $post_id, $language );
				}
			}
		}
		
		$ids = array_merge( $ids, array_filter( $post_ids ) );

		return \wp_parse_id_list( $ids );
	}

	/**
	 * Add custom republish intervals.
	 */
	public function custom_interval( $items ) {
		unset( $items['premium'] );
		$items['custom'] = __( 'Custom Interval', 'wp-auto-republish' );

		return $items;
	}

	/**
	 * Add custom post ages.
	 */
	public function post_ages( $items ) {
		unset( $items['premium_1'] );
		unset( $items['premium_2'] );

		$elements = array(
			'none' => __( 'No Age Limit', 'wp-auto-republish' ),
		);

		$items = $elements + $items;
		$items['custom'] = __( 'Custom Age Limit', 'wp-auto-republish' );

		return array_unique( $items );
	}

	/**
	 * Add custom republish intervals.
	 */
	public function random_interval( $items ) {
		for ( $i = 1; $i <= 10; $i++ ) {
			unset( $items[ 'premium_' . $i ] );
		}

		$elements = array(
			'60'    => __( 'No Randomness', 'wp-auto-republish' ),
			'300'   => __( 'Upto 5 Minutes', 'wp-auto-republish' ),
			'600'   => __( 'Upto 10 Minutes', 'wp-auto-republish' ),
			'900'   => __( 'Upto 15 Minutes', 'wp-auto-republish' ),
			'1200'  => __( 'Upto 20 Minutes', 'wp-auto-republish' ),
			'1800'  => __( 'Upto 30 Minutes', 'wp-auto-republish' ),
			'2700'  => __( 'Upto 45 Minutes', 'wp-auto-republish' ),
			'28800' => __( 'Upto 8 hours', 'wp-auto-republish' ),
			'43200' => __( 'Upto 12 hours', 'wp-auto-republish' ),
			'86400' => __( 'Upto 24 hours', 'wp-auto-republish' ),
		);

		$items = $elements + $items;
		ksort( $items );

		return array_unique( $items );
	}

	/**
	 * Add custom republish intervals.
	 */
	public function republish_orderby() {
		$elements = array(
			'date'          => __( 'Post Date', 'wp-auto-republish' ),
			'ID'            => __( 'Post ID', 'wp-auto-republish' ),
			'author'        => __( 'Post Author', 'wp-auto-republish' ),
			'title'         => __( 'Post Title', 'wp-auto-republish' ),
			'name'          => __( 'Post Name', 'wp-auto-republish' ),
			'rand'          => __( 'Random Selection', 'wp-auto-republish' ),
			'comment_count' => __( 'Comment Count', 'wp-auto-republish' ),
			'relevance'     => __( 'Relevance', 'wp-auto-republish' ),
			'menu_order'    => __( 'Menu Order', 'wp-auto-republish' ),
		);

		return $elements;
	}

		/**
	 * Filter Polylang Post meta.
	 */
	public function copy_post_metas( $metas, $sync, $from ) {
		$old_metas = get_post_custom( $from );

		foreach ( $old_metas as $key => $values ) {
			if ( strpos( $key, 'wpar_' ) !== false ) {
				if ( $key != '_wpar_original_pub_date' ) {
					$metas[] = $key;
				}
			}
		}

		return array_unique( $metas );
	}
}