<?php
/**
 * Premium Features.
 *
 * @since      1.2.0
 * @package    RevivePress
 * @subpackage RevivePress\Core\Premium
 * @author     Sayan Datta <iamsayan@protonmail.com>
 */

namespace RevivePress\Core\Premium;

use RevivePress\Core\PostRepublish;

defined( 'ABSPATH' ) || exit;

/**
 * Actions class.
 */
class InstantRepublish extends PostRepublish
{
	/**
	 * Register functions.
	 */
	public function register() {
		$this->action( 'admin_init', 'one_click_republish' );
		$this->action( 'add_meta_boxes', 'register_metabox', 10, 2 );
		$this->filter( 'post_row_actions', 'row_actions', 20, 2 );
		$this->filter( 'page_row_actions', 'row_actions', 20, 2 );
	}

	/**
	 * Register metaboxes.
	 * 
	 * @param object $post The WP_Post object
	 */
	public function register_metabox( $post_type, $post ) {
		// If user can't publish posts, then get out
		if ( ! current_user_can( 'publish_posts' ) || ! $this->is_enabled( 'enable_instant_republishing', true ) ) {
			return;
		}

		$post_status_not_allowed = $this->do_filter( 'not_allowed_post_statuses', array( 'trash', 'auto-draft', 'future' ) );
		if ( in_array( $post->post_status, $post_status_not_allowed ) || ! $this->user_has_cap__premium_only( 'instant_roles' ) ) {
			return;
		}

		$original_post_id = $this->get_meta( $post->ID, 'wpar_original_post_id' );
		$post_screens = $this->get_data( 'show_links_in', array( 'post_list' ) );
		if ( ! in_array( 'edit_screen', $post_screens ) || $original_post_id ) {
			return;
		}

		$post_types = $this->get_data( 'post_types_list_instant', array( 'post' ) );

		add_meta_box( 'wpar_meta_box_instant', __( 'RevivePress Actions', 'wp-auto-republish' ), array( $this, 'metabox_cb' ), $post_types, 'side' );
	}

	/**
	 * Register post row action.
	 * 
	 * @param array   $actions  Row actions
	 * @param object  $post     Post Object
	 * 
	 * @return array  $actions  Filtered Row actions
	 */
	public function row_actions( $actions, $post ) {
		if ( ! $this->is_enabled( 'enable_instant_republishing', true ) ) {
			return $actions;
		}

		$post_types = $this->get_data( 'post_types_list_instant', array( 'post' ) );
		if ( ! $this->user_has_cap__premium_only( 'instant_roles' ) || ! in_array( $post->post_type, $post_types ) ) {
			return $actions;
		}

		$post_status_not_allowed = $this->do_filter( 'not_allowed_post_statuses', array( 'trash', 'draft', 'auto-draft', 'future' ) );
		if ( in_array( $post->post_status, $post_status_not_allowed ) ) {
			return $actions;
		}

		$post_screens = $this->get_data( 'show_links_in', array( 'post_list' ) );
		if ( ! in_array( 'post_list', $post_screens ) ) {
			return $actions;
		}
	
		$enabled_links = $this->get_data( 'allowed_actions', array( 'republish', 'clone' ) );
		$has_rewrited_post = $this->get_meta( $post->ID, 'wpar_rewrite_post_id' );

		// Republish link
		if ( in_array( 'republish', $enabled_links ) && $this->do_filter( 'show_republish_action', true, $post ) ) {
			$actions['rvp_republish'] = '<a href="' . $this->build_url( 'republish', $post ) . '" aria-label="'. esc_attr( 
				sprintf( 
					/* translators: Post title. */
					__( 'Republish %s now', 'wp-auto-republish' ), _draft_or_post_title( $post ) 
				) 
			) . '">' . __( 'Republish', 'wp-auto-republish' ) . '</a>';
		}

		// Clone Link
		if ( in_array( 'clone', $enabled_links ) && $this->do_filter( 'show_clone_action', true, $post ) ) {
			$actions['rvp_clone'] = '<a href="' . $this->build_url( 'clone', $post ) . '" aria-label="'. esc_attr( 
				sprintf( 
					/* translators: Post title. */
					__( 'Duplicate %s now', 'wp-auto-republish' ), _draft_or_post_title( $post ) 
				) 
			) . '">' . __( 'Clone', 'wp-auto-republish' ) . '</a>';
		}

		// Rewrite Link
		if ( in_array( 'rewrite', $enabled_links ) && ! $has_rewrited_post && $this->do_filter( 'show_rewrite_action', true, $post ) ) {
			$actions['rvp_scheduled'] = '<a href="' . $this->build_url( 'rewrite', $post ) . '" aria-label="'. esc_attr( 
				sprintf( 
					/* translators: Post title. */
					__( 'Rewrite & Republish %s', 'wp-auto-republish' ), _draft_or_post_title( $post ) 
				) 
			) . '">' . __( 'Rewrite & Republish', 'wp-auto-republish' ) . '</a>';
		}

		// Social Link
		if ( in_array( 'share', $enabled_links ) && $this->do_filter( 'show_share_action', true, $post ) ) {
			$actions['rvp_share'] = '<a href="#" aria-label="'. esc_attr( 
				sprintf( 
					/* translators: Post title. */
					__( 'Share %s now', 'wp-auto-republish' ), _draft_or_post_title( $post ) 
				) 
			) . '" class="wpar-social-share-link" data-post-id="' . esc_attr( $post->ID ) . '">' . __( 'Share', 'wp-auto-republish' ) . '</a>';
		}

		return $actions;
	}

	/**
	 * Register metaboxes.
	 * 
	 * @param object $post The WP_Post object
	 */
	public function metabox_cb( $post ) {
		$enabled_links = $this->get_data( 'allowed_actions', array( 'republish', 'clone' ) );
		$content = '';
		
		// Republish link
		if ( in_array( 'republish', $enabled_links ) && $this->do_filter( 'show_republish_action', true, $post ) ) {
			$content .= '<a href="' . $this->build_url( 'republish', $post ) . '" aria-label="'. esc_attr( sprintf( __( 'Republish %s now', 'wp-auto-republish' ), _draft_or_post_title( $post ) ) ) . '">' . __( 'Republish', 'wp-auto-republish' ) . '</a>';
		}

		// Clone Link
		if ( in_array( 'clone', $enabled_links ) && $this->do_filter( 'show_clone_action', true, $post ) ) {
			$content .= '<a href="' . $this->build_url( 'clone', $post ) . '" aria-label="'. esc_attr( sprintf( __( 'Duplicate %s now', 'wp-auto-republish' ), _draft_or_post_title( $post ) ) ) . '">' . __( 'Clone', 'wp-auto-republish' ) . '</a>';
		}

		// Rewrite Link
		if ( in_array( 'rewrite', $enabled_links ) && $this->do_filter( 'show_rewrite_action', true, $post ) ) {
			$content .= '<a href="' . $this->build_url( 'rewrite', $post ) . '" aria-label="'. esc_attr( 
				sprintf( 
					/* translators: Post title. */
					__( 'Rewrite %s now', 'wp-auto-republish' ), _draft_or_post_title( $post ) 
				) 
			) . '">' . __( 'Rewrite', 'wp-auto-republish' ) . '</a>';
		}

		// Social Link
		if ( in_array( 'share', $enabled_links ) && $this->do_filter( 'show_share_action', true, $post ) ) {
			$content .= '<a href="#" aria-label="'. esc_attr( 
				sprintf( 
					/* translators: Post title. */
					__( 'Share %s now', 'wp-auto-republish' ), _draft_or_post_title( $post ) 
				) 
			) . '" class="wpar-social-share-link" data-post-id="' . esc_attr( $post->ID ) . '">' . __( 'Share', 'wp-auto-republish' ) . '</a>';
		}

		echo '<div class="wpar-action-links">' . wp_kses_post( $content ) . '</div>';
	}
	
	/**
	 * Generate Nonce URL.
	 * 
	 * @param array   $type  Action type
	 * @param object  $post  Post Object
	 * 
	 * @return string  $url
	 */
	private function build_url( $type, $post ) {
		$build_url = add_query_arg( array(
			'post_type'   => $post->post_type,
			'rvp_action'  => $type,
			'rvp_post_id' => $post->ID,
		) );

		return wp_nonce_url( $build_url, 'rvp_republish_' . $post->ID );
	}

	/**
	 * Trigger One Click republish event.
	 */
	public function one_click_republish() {
		if ( ! isset( $_GET['rvp_action'], $_GET['rvp_post_id'] ) || empty( $_GET['rvp_post_id'] ) || empty( $_GET['rvp_action'] ) ) {
			return;
		}

		// check nonce
		check_admin_referer( 'rvp_republish_' . esc_attr( sanitize_key( $_GET['rvp_post_id'] ) ) );
		$post_id = intval( $_GET['rvp_post_id'] );
		$post = \get_post( $post_id );
		if ( ! $post ) {
			\wp_die( \esc_html__( 'An error occurred while performing RevivePress Actions.', 'wp-auto-republish' ) );
		}

		$action = sanitize_text_field( wp_unslash( $_GET['rvp_action'] ) );
		switch ( $action ) {
			case 'republish':
			default:
				// run republish  process
				$post_id = $this->update_old_post( $post->ID, false, true );
			  	break;
			case 'clone':
				// run republish  process
				$post_id = $this->clone_old_post__premium_only( $post->ID, false, true );
				break;
			case 'rewrite':
				// run republish process
				$new_post_id = $this->clone_old_post__premium_only( $post->ID, false, false, true );
		    		
				$this->update_meta( $new_post_id, 'wpar_filter_republish_status', 'pending' );
				$this->update_meta( $new_post_id, '_wpar_filter_republish_datetime', \get_post( $new_post_id )->post_date );
				
				$redirect_url = get_edit_post_link( $new_post_id, 'edit' );
				$post_id = $new_post_id;
				break;
		}

		$this->do_action( 'do_one_click_process', $action, $post );

		if ( ! isset( $redirect_url ) ) {
			$redirect_url = remove_query_arg( array( 'rvp_action', 'rvp_post_id', '_wpnonce' ) );
		}

		// set temporary transient for admin notice
		set_transient( 'rvp_instant_republish_done', $action );

		wp_safe_redirect( $redirect_url );
		exit;
	}
}