<?php
/**
 * The file for Email Notification.
 *
 * @since      1.1.7
 * @package    RevivePress
 * @subpackage RevivePress\Core\Premium
 * @author     Sayan Datta <iamsayan@protonmail.com>
 */

namespace RevivePress\Core\Premium;

use RevivePress\Helpers\Hooker;
use RevivePress\Helpers\Scheduler;
use RevivePress\Helpers\HelperFunctions;

defined( 'ABSPATH' ) || exit;

/**
 * Email notification class.
 */
class Notification
{
	use Hooker;
    use HelperFunctions;
    use Scheduler;

	/**
	 * Register functions.
	 */
	public function register() {
		$this->action( 'wpar/process_email_notify', 'send_email' );
		$this->filter( 'wpar/email_template_tags', 'generate_template_tags' );
	}

	/**
	 * Send email notification.
	 *
	 * @param int $post_id  Post ID.
	 */
	public function send_email( $post_id ) {
		$post = \get_post( $post_id );
		$post_author = get_user_by( 'id', $post->post_author );
		$disable = $this->get_meta( $post->ID, '_wpar_disable_email' );

		$enable = $this->is_enabled( 'enable_email_notify' );
		$post_types = $this->get_data( 'email_post_types', array( 'post' ) );
		if ( ! is_array( $post_types ) || ! in_array( $post->post_type, $post_types ) ) {
			return;
		}

		$value = $this->get_data( 'email_recipients', array() );
		$recipients = is_array( $value ) ? $value : explode( ',', $value );
		if ( $this->is_enabled( 'enable_post_author_email' ) ) {
			if ( ! in_array( $post_author->data->user_email, $recipients ) ) {
				$recipients[] = $post_author->data->user_email;
			}
		}
		$subject = $this->replace_veriables( $this->get_data( 'email_subject' ), $post );
		$body = $this->replace_veriables( $this->get_data( 'email_message' ), $post, 'body' );
		$headers = array( 'Content-Type: text/html; charset=UTF-8' );
		$headers = $this->do_filter( 'custom_email_headers', $headers );
	
		if ( $enable && ! $disable ) {
			wp_mail( $recipients, $subject, $body, $headers );
			// add eevent to log
			$this->do_action( 'insert_log', $post->ID, 'email' );
		}
	}

	/**
	 * Replace email variables.
	 *
	 * @param string  $text  Input data.
	 * @param object  $post  WP Post object.
	 * @param string  $type  Sting type. Default subject.
	 * 
	 * @return string
	 */
	private function replace_veriables( $text, $post, $type = 'subject' ) {
		$author_name = get_the_author_meta( 'display_name', $post->post_author );
		$post_time = $this->get_meta( $post->ID, '_wpar_original_pub_date' );

		$search = array( '%post_title%', '%post_type%', '%post_link%', '%post_id%', '%author_name%', '%post_time%', '%republish_time%', '%site_name%', '%site_url%', '%admin_email%' );
		$replace = array( $post->post_title, $post->post_type, get_permalink( $post->ID ), $post->ID, $author_name, $post_time, $post->post_date, wp_specialchars_decode( get_bloginfo( 'name' ), ENT_QUOTES ), get_bloginfo( 'url' ), get_bloginfo( 'admin_email' ) );

		$text = stripslashes( $text );
		$text = str_replace( $search, $replace, $text );
		if ( $type === 'subject' ) {
            return stripslashes( wp_strip_all_tags( $text ) );
		}

		return stripslashes( htmlspecialchars_decode( $text ) );
	}

	/**
	 * Generate template tags output.
	 */
	public function generate_template_tags( $type ) {
		ob_start() ?>
		<div class="dynamic-tags dynamic-tags-email-<?php echo esc_attr( $type ); ?>">
			<div class="dynamic-tags-label"><?php esc_html_e( 'Use these tags into email title or/and body. Available Dynamic Tags', 'wp-auto-republish' ); ?>:</div>
			<div class="dynamic-tags-content"><?php echo wp_kses_post( $this->get_available_tags() ); ?></div>
		</div>
		<?php
		return ob_get_clean();
	}

	/**
	 * Get avaialable template tags.
	 * 
	 * @since 1.3.2
	 * @return string
	 */
	private function get_available_tags() {
		$tags = array( '%post_title%', '%post_type%', '%post_link%', '%post_id%', '%author_name%', '%post_time%', '%republish_time%', '%site_name%', '%site_url%', '%admin_email%' );
		$content = array();
		foreach ( $tags as $tag ) {
			$content[] = '<code class="click-to-copy">' . $tag . '</code>';
		}

		return join( ' ', $content );
	}
}