<?php
/**
 * Meta Box.
 *
 * @since      1.1.0
 * @package    RevivePress
 * @subpackage RevivePress\Core\Premium
 * @author     Sayan Datta <iamsayan@protonmail.com>
 */

namespace RevivePress\Core\Premium;

use RevivePress\Helpers\Hooker;
use RevivePress\Helpers\Sitepress;
use RevivePress\Helpers\Premium\GetMetaData;

defined( 'ABSPATH' ) || exit;

/**
 * Post Metabox class.
 */
class PostMetaBox
{
	use GetMetaData;
    use Hooker;

	/**
	 * Settings.
	 */
	private $enable_global = false;

	/**
	 * Settings.
	 */
	private $enable_single = false;

	/**
	 * Settings.
	 */
	private $enable_manual = false;

	/**
	 * Settings.
	 */
	private $can_show = false;

	/**
	 * Register functions.
	 */
	public function register() {
		$this->action( 'add_meta_boxes', 'register_metabox', 10, 2 );
		$this->action( 'save_post', 'save_metadata' );
	}

	/**
	 * Register metaboxes.
	 * 
	 * @param object $post The WP_Post object
	 */
	public function register_metabox( $post_type, $post ) {
		// If user can't publish posts, then get out
		if ( ! current_user_can( 'publish_posts' ) || ! in_array( $post_type, $this->get_post_types( true ) ) || ! $this->do_filter( 'show_republish_meta_boxes', true, $post ) ) {
			return;
		}

		$orig_id = $this->get_meta( $post->ID, 'wpar_original_post_id' );
		if ( 'future' === $post->post_status && $orig_id ) {
			return;
		}

		if ( $this->is_enabled( 'enable_single_metabox' ) && $this->user_has_cap__premium_only( 'wpar_single_roles' ) ) {
		    add_meta_box( 'wpar_meta_box', __( 'RevivePress', 'wp-auto-republish' ), array( $this, 'metabox_cb' ), $post_type, 'normal' );
		}
	}

	/**
	 * Build republish date meta box.
	 *
	 * @param object $post The post object
	 */
	public function metabox_cb( $post ) {
		$timestamp = $this->current_timestamp();

		$post_types_global = $this->get_data( 'wpar_post_types', array( 'post' ) );
		if ( $this->is_enabled( 'enable_plugin', true ) && in_array( $post->post_type, $post_types_global ) ) {
			$this->enable_global = true;
		}

		$post_types_single = $this->get_data( 'post_types_list_single', array( 'post' ) );
		if ( $this->is_enabled( 'enable_single_republishing', true ) && in_array( $post->post_type, $post_types_single ) ) {
			$this->enable_single = true;
		}

		$post_types_manual = $this->get_data( 'post_types_list_instant', array( 'post' ) );
		if ( $this->is_enabled( 'enable_instant_republishing', true ) && in_array( $post->post_type, $post_types_manual ) ) {
			$this->enable_manual = true;
		}

		$this->can_show = $this->enable_global || $this->enable_single || $this->enable_manual;

		// make sure the form request comes from WordPress
		$this->nonce( 'single' ); ?>

		<div id="wpar-metabox-tabs">
			<ul>
				<li><a href="#wpar-tab-details"><?php esc_html_e( 'Details', 'wp-auto-republish' ); ?></a></li>
				<?php if ( $this->enable_single ) { ?>
					<li><a href="#wpar-tab-single"><?php esc_html_e( 'Configure Schedule', 'wp-auto-republish' ); ?></a></li>
				<?php } ?>
				<?php if ( $this->can_show ) { ?>
					<li><a href="#wpar-tab-misc"><?php esc_html_e( 'Republish Options', 'wp-auto-republish' ); ?></a></li>
				<?php } ?>
			</ul>
			<div id="wpar-tab-details">
				<?php $this->info_tab( $post ); ?>
			</div>
			<?php if ( $this->enable_single ) { ?>
				<div id="wpar-tab-single" class="wpar-schedule">
					<?php $this->schedule_tab( $post ); ?>
				</div>
			<?php } ?>
			<?php if ( $this->can_show ) { ?>
				<div id="wpar-tab-misc">
					<?php $this->edit_tab( $post ); ?>
				</div>
			<?php } ?>
		</div>
		<?php 
	}

	/**
	 * Build republish action meta box.
	 *
	 * @param object $post The post object
	 */
	private function info_tab( $post ) {
		$reposted = maybe_unserialize( $this->get_meta( $post->ID, '_wpar_repost_date' ) );
		$excluded = $this->get_meta( $post->ID, '_wpar_exclude_auto_republish' );
		$global_pending = $this->get_meta( $post->ID , 'wpar_global_republish_status' );
		$original_info = $this->get_meta( $post->ID, '_wpar_hide_original_info' );
		$disable_email = $this->get_meta( $post->ID, '_wpar_disable_email' );
		$disable_facebook_share = $this->get_meta( $post->ID, '_wpar_disable_facebook_share' );
		$disable_twitter_share = $this->get_meta( $post->ID, '_wpar_disable_twitter_share' );
		$disable_linkedin_share = $this->get_meta( $post->ID, '_wpar_disable_linkedin_share' );
		$disable_pinterest_share = $this->get_meta( $post->ID, '_wpar_disable_pinterest_share' );
		$disable_tumblr_share = $this->get_meta( $post->ID, '_wpar_disable_tumblr_share' );
		$datetime_format = get_option( 'date_format' ) . ' @ ' . get_option( 'time_format' );
		
		if ( $post->post_status !== 'auto-draft' ) {
	    	if ( ! empty( $reposted ) ) {
	    		$last_reposted = $reposted[ count( $reposted ) - 1 ];
	    		printf(
	    			'<div class="count misc-pub-republished"><span id="republish-count">%s</span></div>',
	    			sprintf( 
						/* translators: Number of republish occurred. */
						wp_kses_post( __( 'Republished %s time(s)', 'wp-auto-republish' ) ), '<strong>' . count( $reposted ) . '</strong>'
					)
	    		);
	    	} else {
	    		printf(
	    			'<div class="count misc-pub-republished"><span id="republish-count">%s</span></div>',
	    			sprintf( esc_html__( 'Not republished yet', 'wp-auto-republish' ) )
	    		);
	    	}
	    }
		
		if ( ! empty( $reposted ) ) {
			$timestamps = '';
			$reposted = array_slice( array_reverse( $reposted ), 1, $this->do_filter( 'show_republish_count', 15 ), true );
			foreach ( $reposted as $times ) {
				$timestamps .= sprintf(
					'<span>%s</span><br />',
					date_i18n( $datetime_format, strtotime( $times ) )
				);
			}
	
			if ( count( $reposted ) > 1 ) {
			    printf(
			    	'<div class="clear" style="margin-top: 10px;"><span id="republish-timestamp">%1$s <b>%2$s</b></span><div id="republish-timestampdiv" class="hide-if-js" style="margin-top: 10px;">%3$s</div></div>',
			    	sprintf(
			    		'<a href="#republish-timestampdiv" class="edit-republish-timestamp hide-if-no-js"><span aria-hidden="true">%s</span></a>',
			    		esc_html__( 'Last Republished:', 'wp-auto-republish' )
			    	),
			    	esc_html( date_i18n( $datetime_format, strtotime( $last_reposted ) ) ),
			    	$timestamps // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
			    );
		    } else {
				printf(
					'<div class="clear" style="margin-top: 10px;"><span id="republish-timestamp">%1$s <b>%2$s</b></span><div id="republish-timestampdiv" class="hide-if-js" style="margin-top: 10px;">%3$s</div></div>',
					esc_html__( 'Last Republished:', 'wp-auto-republish' ),
					esc_html( date_i18n( $datetime_format, strtotime( $last_reposted ) ) ),
					$timestamps // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
				);
			}
		}

		$elements = array(
			array(
				'id'      => 'wpar_exclude_auto_republish',
				'label'   => __( 'Exclude from Auto Republishing', 'wp-auto-republish' ),
				'visible' => $this->is_enabled( 'enable_plugin', true ) && in_array( $post->post_type, $this->get_data( 'wpar_post_types', array( 'post' ) ) ) && ! $global_pending,
				'value'   => $excluded,
			),
			array(
				'id'      => 'wpar_hide_original_info',
				'label'   => __( 'Hide Original Publish Info', 'wp-auto-republish' ),
				'visible' => 'disable' != $this->get_data( 'wpar_republish_position' ) && in_array( $post->post_type, $this->get_data( 'wpar_post_types_display', array( 'post' ) ) ),
				'value'   => $original_info,
			),
			array(
				'id'      => 'wpar_disable_email',
				'label'   => __( 'Disable Email Notification', 'wp-auto-republish' ),
				'visible' => $this->is_enabled( 'enable_email_notify' ) && in_array( $post->post_type, $this->get_data( 'email_post_types', array( 'post' ) ) ),
				'value'   => $disable_email,
			),
			array(
				'id'      => 'wpar_disable_facebook_share',
				'label'   => __( 'Disable Auto Facebook Share', 'wp-auto-republish' ),
				'visible' => $this->is_enabled( 'facebook_enable' ) && in_array( $post->post_type, $this->get_data( 'facebook_post_types_display', array( 'post' ) ) ),
				'value'   => $disable_facebook_share,
			),
			array(
				'id'      => 'wpar_disable_twitter_share',
				'label'   => __( 'Disable Auto Twitter Share', 'wp-auto-republish' ),
				'visible' => $this->is_enabled( 'twitter_enable' ) && in_array( $post->post_type, $this->get_data( 'twitter_post_types_display', array( 'post' ) ) ),
				'value'   => $disable_twitter_share,
			),
			array(
				'id'      => 'wpar_disable_linkedin_share',
				'label'   => __( 'Disable Auto Linkedin Share', 'wp-auto-republish' ),
				'visible' => $this->is_enabled( 'linkedin_enable' ) && in_array( $post->post_type, $this->get_data( 'linkedin_post_types_display', array( 'post' ) ) ),
				'value'   => $disable_linkedin_share,
			),
			array(
				'id'      => 'wpar_disable_pinterest_share',
				'label'   => __( 'Disable Auto Pinterest Share', 'wp-auto-republish' ),
				'visible' => $this->is_enabled( 'pinterest_enable' ) && in_array( $post->post_type, $this->get_data( 'pinterest_post_types_display', array( 'post' ) ) ),
				'value'   => $disable_pinterest_share,
			),
			array(
				'id'      => 'wpar_disable_tumblr_share',
				'label'   => __( 'Disable Auto Tumblr Share', 'wp-auto-republish' ),
				'visible' => $this->is_enabled( 'tumblr_enable' ) && in_array( $post->post_type, $this->get_data( 'tumblr_post_types_display', array( 'post' ) ) ),
				'value'   => $disable_tumblr_share,
			),
		);

		$content = '';
		foreach ( $elements as $element ) {
			$content .= $this->load_checkbox( $element );
		}

		if ( ! empty( $content ) ) {
			echo '<div class="wpar-metabox-share-container">' . $content . '</div>'; // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
		}

		$this->do_action( 'after_metabox_content', $post );
	}

		/**
	 * Build republish date meta box.
	 *
	 * @param object $post The post object
	 */
	private function edit_tab( $post ) {
		$is_active = $this->get_meta( $post->ID, '_wpar_repost_post_titles_active', 'no' );
		$is_update = $this->get_meta( $post->ID, '_wpar_repost_post_update_url', 'no' );
		$action = $this->get_meta( $post->ID, '_wpar_repost_post_update_action', 'default' );
		$get_titles = $this->get_meta( $post->ID, '_wpar_repost_post_titles', array( _draft_or_post_title( $post ) ) );
		$titles = is_array( $get_titles ) ? $get_titles : explode( ';;;', $get_titles );
		$get_titles = implode( ';;;', array_map( 'wp_strip_all_tags', $titles ) ); ?>

		<fieldset class="options wpar-post-edit">
			<div class="row">
				<label for="wpar_repost_post_update_action" class="column">
					<strong><?php esc_html_e( 'Republish Action', 'wp-auto-republish' ); ?>:</strong>
					<span class="fields">
						<select name="wpar_repost_post_update_action" id="wpar_repost_post_update_action">
							<option value="default" <?php selected( 'default', $action ) ?> ><?php esc_html_e( 'Default', 'wp-auto-republish' ); ?></option>
							<option value="repost" <?php selected( 'repost', $action ) ?> ><?php esc_html_e( 'Republish Post', 'wp-auto-republish' ); ?></option>
							<option value="clone" <?php selected( 'clone', $action ) ?> ><?php esc_html_e( 'Duplicate Post', 'wp-auto-republish' ); ?></option>
							<!-- <option value="update" <?php //selected( 'update', $action ) ?> ><?php //esc_html_e( 'Update Post', 'wp-auto-republish' ); ?></option> -->
						</select>
					</span>
				</label>
				<label for="wpar_repost_post_titles_active" class="column">
					<strong><?php esc_html_e( 'Re-Publish Titles', 'wp-auto-republish' ); ?>:</strong>
					<span class="fields">
						<select name="wpar_repost_post_titles_active" id="wpar_repost_post_titles_active">
							<option value="yes" <?php selected( 'yes', $is_active ) ?> ><?php esc_html_e( 'Yes', 'wp-auto-republish' ); ?></option>
							<option value="no" <?php selected( 'no', $is_active ) ?> ><?php esc_html_e( 'No', 'wp-auto-republish' ); ?></option>
						</select>
					</span>
				</label>
			</div>
			<div class="row wpar-republish-title">
				<label for="wpar_repost_post_titles" class="column">
					<strong><?php esc_html_e( 'Enter the New Post Titles', 'wp-auto-republish' ); ?>:</strong>
					<span class="fields">
						<select name="wpar_repost_post_titles[]" id="wpar_repost_post_titles" multiple>
							<?php $get_titles = is_array( $get_titles ) ? $get_titles : explode( ';;;', $get_titles );
							foreach ( $get_titles as $title ) {
								echo '<option value="' . esc_attr( $title ) . '" selected="selected">' . esc_html( $title ) . '</option>';
							} ?>
						</select>
					</span>
				</label>
			</div>
			<div class="row wpar-republish-title">
				<label for="wpar_repost_post_remove_title" class="column">
					<strong><?php esc_html_e( 'Remove Republished Title(s)', 'wp-auto-republish' ); ?>:</strong></span>
					<span class="fields">
						<select name="wpar_repost_post_remove_title" id="wpar_repost_post_remove_title">
							<option value="yes" <?php selected( 'yes', $is_update ) ?> ><?php esc_html_e( 'Yes', 'wp-auto-republish' ); ?></option>
							<option value="no" <?php selected( 'no', $is_update ) ?> ><?php esc_html_e( 'No', 'wp-auto-republish' ); ?></option>
						</select>
					</span>
				</label>
				<label for="wpar_repost_post_update_url" class="column">
					<strong><?php esc_html_e( 'Update Post URL', 'wp-auto-republish' ); ?>:</strong></span>
					<span class="fields">
						<select name="wpar_repost_post_update_url" id="wpar_repost_post_update_url">
							<option value="yes" <?php selected( 'yes', $is_update ) ?> ><?php esc_html_e( 'Yes', 'wp-auto-republish' ); ?></option>
							<option value="no" <?php selected( 'no', $is_update ) ?> ><?php esc_html_e( 'No', 'wp-auto-republish' ); ?></option>
						</select>
					</span>
				</label>
			</div>
		</fieldset>
		<?php 
	}

	/**
	 * Build republish date meta box.
	 *
	 * @param object $post The post object
	 */
	private function schedule_tab( $post ) {
		$timestamp = $this->current_timestamp();
		$start_time = $this->get_data( 'wpar_start_time', '05:00:00' );
		
		// General
		$global_pending = $this->get_meta( $post->ID, 'wpar_global_republish_status' );
		$single_pending = $this->get_meta( $post->ID , 'wpar_single_republish_status' );
		$interval = $this->do_filter( 'random_single_republish_interval', 5 );
		
		// Schedule
		$repost_option = $this->get_meta( $post->ID, '_wpar_repost_option' );
		$repost_after_number = $this->get_meta( $post->ID, '_wpar_repost_after_number' );
		$repost_after_time = $this->get_meta( $post->ID, '_wpar_repost_after_time' );
		$repost_on_date = $this->get_meta( $post->ID, '_wpar_repost_on_date', date( 'm/d/Y', $timestamp ) );
		$repost_repeats = $this->get_meta( $post->ID, '_wpar_repost_repeats' );
		$repost_every = $this->get_meta( $post->ID, '_wpar_repost_every' );
		$repost_monthly_on = $this->get_meta( $post->ID, '_wpar_repost_monthly_on', array(), true );
		$repost_by = $this->get_meta( $post->ID, '_wpar_repost_by' );
		$repost_start = $this->get_meta( $post->ID, '_wpar_repost_start', date( 'm/d/Y', $timestamp ) );
		$repost_include_date = $this->get_meta( $post->ID, '_wpar_include_starts_on' );
		$custom_date_time = $this->get_meta( $post->ID, '_wpar_custom_date_time' );
		$repost_end = $this->get_meta( $post->ID, '_wpar_repost_end' );
		$repost_end_after = $this->get_meta( $post->ID, '_wpar_repost_end_after' );
		$repost_end_on = $this->get_meta( $post->ID, '_wpar_repost_end_on', date( 'm/d/Y', strtotime( '+30 days', $timestamp ) ) );
		$repost_status = $this->get_meta( $post->ID, '_wpar_repost_republish_status', 'default' );
		$repost_status_recurring = $this->get_meta( $post->ID, '_wpar_repost_last_republish_status', 'default' );
		$repost_schedule = $this->get_meta( $post->ID, '_wpar_repost_schedule' );
		$repost_at_time = $this->get_meta( $post->ID, '_wpar_repost_at_specific_time', $start_time );
		$repost_enable = $this->get_meta( $post->ID, '_wpar_repost_done' );
		$repost_datetime = $this->get_meta( $post->ID, '_wpar_repost_schedule_datetime' );
		$repost_start_time = $this->get_meta( $post->ID, '_wpar_repost_start_time' );
		$repost_end_time = $this->get_meta( $post->ID, '_wpar_repost_end_time' );
		
		$hide_checkbox = true;
		$datetime_stamp = strtotime( date( 'H:i:s', strtotime( $repost_datetime ) ) );
		$current_datetime_stamp = strtotime( date( 'H:i:s', strtotime( 'now' ) ) );

		if ( ( strtotime( $repost_datetime ) > strtotime( 'now' ) && $datetime_stamp > $current_datetime_stamp ) || $repost_include_date === 'yes' ) {
			$hide_checkbox = false;
		}
		
		?>

		<div class="wpar-schedule-info">
			<span style="font-weight: 700;"><?php esc_html_e( 'Schedule', 'wp-auto-republish' ); ?>:</span> <?php
			if ( $repost_schedule ) {
				if ( 'disable' !== $repost_option ) {
					printf(
						'<span id="wpar_repost_date">%s</span>',
						$this->get_schedule( $post->ID, 'edit' ) // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
					);
				} else {
					printf(
						'<span id="wpar_repost_date">%s</span>',
						esc_html__( 'Disabled', 'wp-auto-republish' )
					);
				}
			} else {
				printf(
					'<span id="wpar_repost_date">%s</span>',
					esc_html__( 'Not set yet', 'wp-auto-republish' )
				);
			} ?>
			<span class="wpar-time-div"<?php if ( empty( $repost_option ) || in_array( $repost_option, array( 'disable', 'custom' ) ) ) { echo ' style="display: none;"'; } ?>>
				<?php esc_html_e( 'at', 'wp-auto-republish' ); ?> <input type="text" name="wpar_repost_at_specific_time" id="wpar_repost_at_specific_time" class="wpar-republish-timepicker" value="<?php echo esc_attr( $repost_at_time ); ?>" readonly="readonly" style="width: 85px;" />
				<label class="wpar_repost_republish_status" style="<?php echo ( ! in_array( $repost_option, array( 'days', 'date' ) ) ) ? 'display: none;' : ''; ?>">
					<span class="label" style="margin-left: 5px;"><?php esc_html_e( 'Set Post Status', 'wp-auto-republish' ); ?>:</span>
					<span class="fields">
						<select name="wpar_repost_republish_status" id="wpar_repost_republish_status">
							<option value="default" <?php selected( 'default', $repost_status ); ?> ><?php esc_html_e( 'Default', 'wp-auto-republish' ); ?></option>
							<?php 
							$post_statuses = $this->get_post_statuses();
							foreach ( $post_statuses as $post_status => $name ) {
								echo '<option value="' . esc_attr( $post_status ) . '"' . selected( $post_status, $repost_status, false ) .  '>' . esc_html( $name ) . '</option>';
							} ?>
						</select>
					</span>
				</label>
				<span class="tooltip"><span class="dashicons dashicons-editor-help" style="font-size: 18px;"></span>
				<span class="tooltiptext">
					<?php printf( 
						/* translators: Interval in minutes. */
						esc_html__( 'Republish will occur at random time after this time within %s minutes for Recurring Schedules except Minutes and Hourly.', 'wp-auto-republish' ), esc_html( $interval )
					); ?>
				</span>
			</span>
		</div>
		<fieldset class="options wpar-options">
			<div class="option">
				<label>
					<input type="radio" name="wpar_repost_option" id="wpar_repost_option_1" value="days" <?php checked( 'days', $repost_option ); ?> />
					<?php esc_html_e( 'Republish after', 'wp-auto-republish' ); ?>
					<input type="number" name="wpar_repost_after_number" id="wpar_repost_after_number" value="<?php echo esc_attr( $repost_after_number ); ?>" class="small" min="1" max="365" />
					<select name="wpar_repost_after_time" id="wpar_repost_after_time">
						<option value="days" <?php selected( 'days', $repost_after_time ) ?> ><?php esc_html_e( 'Days', 'wp-auto-republish' ); ?></option>
						<option value="weeks" <?php selected( 'weeks', $repost_after_time ) ?> ><?php esc_html_e( 'Weeks', 'wp-auto-republish' ); ?></option>
						<option value="months" <?php selected( 'months', $repost_after_time ) ?> ><?php esc_html_e( 'Months', 'wp-auto-republish' ); ?></option>
						<option value="years" <?php selected( 'years', $repost_after_time ) ?> ><?php esc_html_e( 'Years', 'wp-auto-republish' ); ?></option>
					</select>
				</label>
			</div>
			<div class="option">
				<label>
					<input type="radio" name="wpar_repost_option" id="wpar_repost_option_2" value="date" <?php checked( 'date', $repost_option ); ?> />
					<?php esc_html_e( 'Republish on', 'wp-auto-republish' ); ?>
					<input type="text" id="wpar-static-date" readonly="readonly" value="<?php echo esc_attr( date( 'd/m/Y', strtotime( $repost_on_date ) ) ); ?>" />
					<input type="hidden" name="wpar_repost_on_date" id="wpar_repost_on_date" value="<?php echo esc_attr( $repost_on_date ); ?>" />
				</label>
			</div>
			<div class="option">
				<label>
					<input type="radio" name="wpar_repost_option" id="wpar_repost_option_3" value="repeat" <?php checked( 'repeat', $repost_option ); ?> />
					<?php esc_html_e('Recurring Schedule', 'wp-auto-republish'); ?>
				</label>
				<div class="sub-option" id="repeat_repost_option"<?php echo ( $repost_option != 'repeat' ) ? ' style="display:none;"' : ''; ?>>
					<div class="row">
						<label>
							<span class="label"><?php esc_html_e( 'Repeats', 'wp-auto-republish' ); ?>:</span>
							<span class="fields">
								<select name="wpar_repost_repeats" id="wpar_repost_repeats">
									<option value="minutes" <?php selected( 'minutes', $repost_repeats ) ?> ><?php esc_html_e( 'Minutes', 'wp-auto-republish' ); ?></option>
									<option value="hourly" <?php selected( 'hourly', $repost_repeats ) ?> ><?php esc_html_e( 'Hourly', 'wp-auto-republish' ); ?></option>
									<option value="daily" <?php selected( 'daily', $repost_repeats ) ?> ><?php esc_html_e( 'Daily', 'wp-auto-republish' ); ?></option>
									<option value="weekly" <?php selected( 'weekly', $repost_repeats ) ?> ><?php esc_html_e( 'Weekly', 'wp-auto-republish' ); ?></option>
									<option value="monthly" <?php selected( 'monthly', $repost_repeats ) ?> ><?php esc_html_e( 'Monthly', 'wp-auto-republish' ); ?></option>
									<option value="yearly" <?php selected( 'yearly', $repost_repeats ) ?> ><?php esc_html_e( 'Yearly', 'wp-auto-republish' ); ?></option>
								</select>
							</span>
						</label>
					</div>
					<div class="row">
						<label>
							<span class="label"><?php esc_html_e( 'Repeat Every', 'wp-auto-republish' ); ?>:</span>
							<span class="fields">
								<select name="wpar_repost_every" id="wpar_repost_every"><?php
								for ( $i = 1; $i <= 59; $i++ ) {
									printf( '<option value="%1$s" %2$s>%1$s</option>', esc_attr( $i ), selected( $i, $repost_every, false ) );
								}
								?></select>
								<input type="hidden" id="wpar-minute-limit" value="<?php echo esc_attr( $this->do_filter( 'minute_starts_from', 5 ) ); ?>"/>
							</span>
						</label>
					</div>
					<div class="row" id="wpar_repeat_monthly_on_wrap"<?php echo ( $repost_repeats != 'monthly' ) ? ' style="display:none;"' : ''; ?>>
						<div class="wpar_elements">
							<span class="label" style="margin: 0 10px 0 0;"><?php esc_html_e( 'Repeat On', 'wp-auto-republish' ); ?>:</span>
							<span class="fields offset">
								<label class="wpar_checklabel" ><input type="checkbox" name="wpar_repost_monthly_on[]" id="wpar_repost_monthly_on_1" value="january" <?php checked( 1, in_array( 'january', $repost_monthly_on ) ); ?> class="wpar_repost_monthly_on" data-value="0" data-month="<?php esc_attr_e( 'January', 'wp-auto-republish'); ?>" /> <?php esc_html_e( 'January', 'wp-auto-republish'); ?> </label>
								<label class="wpar_checklabel" ><input type="checkbox" name="wpar_repost_monthly_on[]" id="wpar_repost_monthly_on_2" value="february" <?php checked( 1, in_array( 'february', $repost_monthly_on ) ); ?> class="wpar_repost_monthly_on"  data-value="1" data-month="<?php esc_attr_e( 'February', 'wp-auto-republish'); ?>" /> <?php esc_html_e( 'February', 'wp-auto-republish'); ?> </label>
								<label class="wpar_checklabel" ><input type="checkbox" name="wpar_repost_monthly_on[]" id="wpar_repost_monthly_on_3" value="march" <?php checked( 1, in_array( 'march', $repost_monthly_on ) ); ?> class="wpar_repost_monthly_on"  data-value="2" data-month="<?php esc_attr_e( 'March', 'wp-auto-republish'); ?>" /> <?php esc_html_e( 'March', 'wp-auto-republish'); ?> </label>
								<label class="wpar_checklabel" ><input type="checkbox" name="wpar_repost_monthly_on[]" id="wpar_repost_monthly_on_4" value="april" <?php checked( 1, in_array( 'april', $repost_monthly_on ) ); ?> class="wpar_repost_monthly_on" data-value="3" data-month="<?php esc_attr_e( 'April', 'wp-auto-republish'); ?>" /> <?php esc_html_e( 'April', 'wp-auto-republish'); ?> </label>
								<label class="wpar_checklabel" ><input type="checkbox" name="wpar_repost_monthly_on[]" id="wpar_repost_monthly_on_5" value="may" <?php checked( 1, in_array( 'may', $repost_monthly_on ) ); ?> class="wpar_repost_monthly_on" data-value="4" data-month="<?php esc_attr_e( 'May', 'wp-auto-republish'); ?>" /> <?php esc_html_e( 'May', 'wp-auto-republish'); ?> </label>
								<label class="wpar_checklabel" ><input type="checkbox" name="wpar_repost_monthly_on[]" id="wpar_repost_monthly_on_6" value="june" <?php checked( 1, in_array( 'june', $repost_monthly_on ) ); ?> class="wpar_repost_monthly_on" data-value="5" data-month="<?php esc_attr_e( 'June', 'wp-auto-republish'); ?>" /> <?php esc_html_e( 'June', 'wp-auto-republish'); ?> </label>
								<label class="wpar_checklabel" ><input type="checkbox" name="wpar_repost_monthly_on[]" id="wpar_repost_monthly_on_7" value="july" <?php checked( 1, in_array( 'july', $repost_monthly_on ) ); ?> class="wpar_repost_monthly_on" data-value="6" data-month="<?php esc_attr_e( 'July', 'wp-auto-republish'); ?>" /> <?php esc_html_e( 'July', 'wp-auto-republish'); ?> </label>
								<label class="wpar_checklabel" ><input type="checkbox" name="wpar_repost_monthly_on[]" id="wpar_repost_monthly_on_8" value="august" <?php checked( 1, in_array( 'august', $repost_monthly_on ) ); ?> class="wpar_repost_monthly_on" data-value="7" data-month="<?php esc_attr_e( 'August', 'wp-auto-republish'); ?>" /> <?php esc_html_e( 'August', 'wp-auto-republish'); ?> </label>
								<label class="wpar_checklabel" ><input type="checkbox" name="wpar_repost_monthly_on[]" id="wpar_repost_monthly_on_9" value="september" <?php checked( 1, in_array( 'september', $repost_monthly_on ) ); ?> class="wpar_repost_monthly_on" data-value="8" data-month="<?php esc_attr_e( 'September', 'wp-auto-republish'); ?>" /> <?php esc_html_e( 'September', 'wp-auto-republish'); ?> </label>
								<label class="wpar_checklabel" ><input type="checkbox" name="wpar_repost_monthly_on[]" id="wpar_repost_monthly_on_10" value="october" <?php checked( 1, in_array( 'october', $repost_monthly_on ) ); ?> class="wpar_repost_monthly_on" data-value="9" data-month="<?php esc_attr_e( 'October', 'wp-auto-republish'); ?>" /> <?php esc_html_e( 'October', 'wp-auto-republish'); ?> </label>
								<label class="wpar_checklabel" ><input type="checkbox" name="wpar_repost_monthly_on[]" id="wpar_repost_monthly_on_11" value="november" <?php checked( 1, in_array( 'november', $repost_monthly_on ) ); ?> class="wpar_repost_monthly_on" data-value="10" data-month="<?php esc_attr_e( 'November', 'wp-auto-republish'); ?>" /> <?php esc_html_e( 'November', 'wp-auto-republish'); ?> </label>
								<label class="wpar_checklabel" ><input type="checkbox" name="wpar_repost_monthly_on[]" id="wpar_repost_monthly_on_12" value="december" <?php checked( 1, in_array( 'december', $repost_monthly_on ) ); ?> class="wpar_repost_monthly_on" data-value="11" data-month="<?php esc_attr_e( 'December', 'wp-auto-republish'); ?>" /> <?php esc_html_e( 'December', 'wp-auto-republish'); ?> </label>
							</span>
						</div>
					</div>
					<div class="row" id="wpar_repeat_by_wrap"<?php echo ( $repost_repeats != 'monthly' ) ? ' style="display:none;"' : ''; ?>>
						<span>
							<span class="label"><?php esc_html_e( 'Repeat By', 'wp-auto-republish' ); ?>:</span>
							<span class="fields">
								<label style="padding-right: 10px;"><input type="radio" name="wpar_repost_by" id="wpar_repost_by_month" value="month" <?php checked( 'month', $repost_by ); ?> /><?php esc_html_e( 'Day of the month', 'wp-auto-republish' ); ?></label>
								<label style="padding-right: 10px;"><input type="radio" name="wpar_repost_by" id="wpar_repost_by_week" value="week" <?php checked( 'week', $repost_by ); ?> <?php if ( empty( $repost_by ) ) { echo ' checked="checked"'; } ?> /><?php esc_html_e( 'Day of the week', 'wp-auto-republish' ); ?></label>
							</span>
						</span>
					</div>
					<div class="row">
						<label>
							<span class="label"><?php esc_html_e( 'Starts On', 'wp-auto-republish' ); ?>:</span>
							<span class="fields">
								<input type="text" id="wpar-static-repost-start" readonly="readonly" value="<?php echo esc_attr( date( 'd/m/Y', strtotime( $repost_start ) ) ); ?>" />
								<input type="hidden" name="wpar_repost_start" id="wpar_repost_start" value="<?php echo esc_attr( $repost_start ); ?>" />
								<span class="wpar_include_start" style="<?php echo ( $hide_checkbox ) ? 'display: none;' : ''; ?>">
									<input type="checkbox" id="wpar_include_starts_on" name="wpar_include_starts_on" value="yes" <?php checked( 'yes', $repost_include_date ); ?> /><label for="wpar_include_starts_on"><?php esc_html_e( 'Start Republish from this Date', 'wp-auto-republish' ); ?></label>
								</span>
							</span>
						</label>
					</div>
					<div class="row">
						<span class="wpar-ends-on-options">
							<span class="label"><?php esc_html_e( 'Ends On', 'wp-auto-republish' ); ?>:</span>
							<span class="fields">
								<label>
									<input type="radio" name="wpar_repost_end" id="wpar_repost_end" value="never" <?php checked( 'never', $repost_end ); ?><?php if ( empty( $repost_end ) ) { echo ' checked="checked"'; } ?> /> <?php esc_html_e( 'Never', 'wp-auto-republish' ); ?>
								</label>
								<label>
									<input type="radio" name="wpar_repost_end" id="wpar_repost_end_1" value="after" <?php checked( 'after', $repost_end ); ?> /> <?php esc_html_e( 'After', 'wp-auto-republish' ); ?>
									<input type="number" name="wpar_repost_end_after" id="wpar_repost_end_after" value="<?php echo esc_attr( $repost_end_after ); ?>" class="small" min="1" /> <?php esc_html_e( 'occurences', 'wp-auto-republish' ); ?>
								</label>
								<label>
									<input type="radio" name="wpar_repost_end" id="wpar_repost_end_2" value="on" <?php checked( 'on', $repost_end ); ?> /> <?php esc_html_e( 'Specific Date', 'wp-auto-republish' ); ?>
									<input type="text" id="wpar-repost-end-on" readonly="readonly" value="<?php echo esc_attr( date( 'd/m/Y', strtotime( $repost_end_on ) ) ); ?>" />
									<input type="hidden" name="wpar_repost_end_on" id="wpar_repost_end_on" value="<?php echo esc_attr( $repost_end_on ); ?>" />
								</label>
							</span>
						</span>
					</div>
					<div class="row" id="wpar_last_republish_status" style="<?php echo ( $repost_end == 'never' ) ? 'display: none;' : ''; ?>">
						<label>
							<span class="label"><?php esc_html_e( 'Set Status', 'wp-auto-republish' ); ?>:</span>
							<span class="fields">
								<select name="wpar_repost_last_republish_status" id="wpar_repost_last_republish_status">
									<option value="default" <?php selected( 'default', $repost_status_recurring ); ?> ><?php esc_html_e( 'Default', 'wp-auto-republish' ); ?></option>
									<?php 
									$post_statuses = $this->get_post_statuses();
									foreach ( $post_statuses as $post_status => $name ) {
										echo '<option value="' . esc_attr( $post_status ) . '"' . selected( $post_status, $repost_status_recurring, false ) .  '>' . esc_html( $name ) . '</option>';
									} ?>
								</select><?php esc_html_e( 'at the time of last repeated republish', 'wp-auto-republish' ); ?>
							</span>
						</label>
					</div>
					<div class="row" id="wpar_last_republish_time_range" style="<?php echo ( ! in_array( $repost_repeats, array( 'minutes', 'hourly' ) ) ) ? 'display: none;' : ''; ?>">
						<label>
							<span class="label"><?php esc_html_e( 'Time Range', 'wp-auto-republish' ); ?>:</span>
							<span class="fields">
								<label><?php esc_html_e( 'Starts at', 'wp-auto-republish' ); ?>
									<input type="text" name="wpar_repost_start_time" id="wpar-repost-start-time" class="wpar-timepicker" value="<?php echo esc_attr( $repost_start_time ); ?>" placeholder="<?php esc_attr_e( 'No Restriction', 'wp-auto-republish' ); ?>" readonly="readonly" />
								</label>
								<label><?php esc_html_e( 'Ends at', 'wp-auto-republish' ); ?>
									<input type="text" name="wpar_repost_end_time" id="wpar-repost-end-time" class="wpar-timepicker" value="<?php echo esc_attr( $repost_end_time ); ?>" placeholder="<?php esc_attr_e( 'No Restriction', 'wp-auto-republish' ); ?>" readonly="readonly" />
								</label>
							</span>
						</label>
					</div>
				</div>
			</div>
			<div class="option">
				<label>
					<input type="radio" name="wpar_repost_option" id="wpar_repost_option_4" value="custom" <?php checked( 'custom', $repost_option ); ?> />
					<?php esc_html_e( 'Custom Date & Time(s)', 'wp-auto-republish' ); ?>
				</label>
				<div class="clear"></div>
				<div id="wpar_custom_dates" <?php echo ( $repost_option != 'custom' ) ? ' style="display:none;"' : ''; ?>>
					<textarea id="wpar-custom-date-time" name="wpar_custom_date_time" placeholder="<?php esc_attr_e( 'Format: 2022-05-15 09:30:55 (one per line in ascending order)', 'wp-auto-republish' ); ?>" rows="4" cols="50"><?php echo esc_textarea( $custom_date_time ); ?></textarea>
					<div><?php esc_attr_e( 'Format: 2022-05-15 09:30:55 (one per line in ascending order)', 'wp-auto-republish' ); ?></div>
				</div>
			</div>
			<div class="option">
				<label>
					<input type="radio" name="wpar_repost_option" id="wpar_repost_option_5" value="disable" <?php checked( 'disable', $repost_option ); ?><?php if ( empty( $repost_option ) ) { echo ' checked="checked"'; } ?> />
					<?php esc_html_e( 'Disable Republish', 'wp-auto-republish' ); ?>
				</label>
			</div>
			<input type="hidden" name="wpar_repost_schedule" id="wpar_repost_schedule" value="<?php echo esc_attr( date( 'm/d/Y', strtotime( $repost_schedule ) ) ); ?>" />
			<input type="hidden" name="wpar_repost_done" id="wpar_repost_done" value="<?php echo ( ! empty( $repost_enable ) ) ? esc_attr( $repost_enable ) : 'yes'; ?>" data-last="<?php echo esc_attr( $repost_enable ); ?>" data-republish="<?php echo esc_attr( $repost_datetime ); ?>" />
			<input type="hidden" name="wpar_regenerate_cron" id="wpar_regenerate_cron_event" value="no" />
		</fieldset>
		<?php 
	}

	/**
	 * Build checkboxes.
	 */
	private function load_checkbox( $args ) {
		ob_start();
		if ( isset( $args['visible'] ) && $args['visible'] && $this->can_show ) { ?>
			<div class="checkbox">
				<label for="<?php echo esc_attr( $args['id'] ); ?>" ><input type="checkbox" name="<?php echo esc_attr( $args['id'] ); ?>" id="<?php echo esc_attr( $args['id'] ); ?>" value="yes" <?php checked( $args['value'], 'yes' ); ?> /> <?php echo esc_html( $args['label'] ); ?></label>
			</div><?php
		}
		return ob_get_clean();
	}
	
	/**
	 * Store custom field meta box data.
	 *
	 * @param int $post_id The post ID.
	 */
	public function save_metadata( $post_id ) {
		// return if autosave
		if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) {
			return $post_id;
		}

		// Check the user's permissions.
		if ( ! current_user_can( 'edit_post', $post_id ) ) {
			return $post_id;
		}

		// phpcs:disable WordPress.Security.NonceVerification.Missing, WordPress.Security.ValidatedSanitizedInput.MissingUnslash, WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		if ( ! $this->verify( 'single' ) ) {
			return $post_id;
		}

		$actual_post_id = $post_id;
		$post_ids = array( $post_id );

		if ( $this->is_enabled( 'enable_republish_translated' ) ) {
			// WPML support.
			$languages = Sitepress::get()->get_languages();
			if ( ! empty( $languages ) ) {
				foreach ( $languages as $language ) {
					foreach ( $ids as $post_id ) {
						$post_ids[] = \apply_filters( 'wpml_object_id', $post_id, get_post_type( $post_id ), false, $language['code'] );
					}
				}
			}

			// Polylang support.
			$languages = function_exists( 'pll_languages_list' ) ? pll_languages_list() : array();
			if ( ! empty( $languages ) && function_exists( 'pll_get_post' ) ) {
				foreach ( $languages as $language ) {
					$post_ids[] = \pll_get_post( $post_id, $language );
				}
			}

			$post_ids = \wp_parse_id_list( array_filter( $post_ids ) );
		}

		// store custom fields values
		$metas = array(
			'wpar_repost_option',
			'wpar_repost_after_number',
			'wpar_repost_after_time',
			'wpar_repost_on_date',
			'wpar_repost_repeats',
			'wpar_repost_every',
			'wpar_include_starts_on',
			'wpar_repost_monthly_on',
			'wpar_repost_by',
			'wpar_repost_start',
			'wpar_repost_end',
			'wpar_repost_end_after',
			'wpar_repost_end_on',
			'wpar_repost_republish_status',
			'wpar_repost_last_republish_status',
			'wpar_repost_schedule',
			'wpar_repost_done',
			'wpar_repost_at_specific_time',
			'wpar_repost_start_time',
			'wpar_repost_end_time',
			'wpar_regenerate_cron',
			'wpar_repost_post_titles_active',
			'wpar_repost_post_remove_title',
			'wpar_repost_post_update_url',
			'wpar_repost_post_titles',
			'wpar_repost_post_update_action',
			'wpar_custom_date_time',
			'wpar_exclude_auto_republish',
			'wpar_hide_original_info',
			'wpar_disable_email',
			'wpar_disable_facebook_share',
			'wpar_disable_twitter_share',
			'wpar_disable_linkedin_share',
			'wpar_disable_pinterest_share',
			'wpar_disable_tumblr_share',
		);

		foreach ( $post_ids as $post_id ) {
			// loop start.
			foreach ( $metas as $meta_key ) {
				// check if post meta exists
				if ( ! isset( $_POST[ $meta_key ] ) ) {
					// delete post meta if not exists
					$this->delete_meta( $post_id, '_' . $meta_key );
					continue;
				}

				// get value
				$value = $_POST[ $meta_key ];
				if ( is_array( $value ) ) {
					$value = array_map( 'sanitize_text_field', $value );
				} else {
					$value = ( 'wpar_custom_date_time' === $meta_key ) ? sanitize_textarea_field( $value ) : sanitize_text_field( $value );
				}

				// add or update post meta if not exists
				$this->update_meta( $post_id, '_' . $meta_key, $value );
			}

			$regenerate = $this->get_meta( $post_id, '_wpar_regenerate_cron' );
			if ( $regenerate != 'yes' ) {
				return $actual_post_id;
			}

			// clear count
			$this->delete_meta( $post_id, 'wpar_post_republish_count' );

			$timestamp = $this->current_timestamp();
			$option = $this->get_meta( $post_id, '_wpar_repost_option' );

			if ( in_array( $option, array( 'days', 'date', 'repeat' ) ) ) {
				$schedule = $this->get_meta( $post_id, '_wpar_repost_schedule' );
				$repost_at_time = $this->get_meta( $post_id, '_wpar_repost_at_specific_time' );

				if ( $schedule && $repost_at_time ) {
					$datetime = strtotime( date( 'Y-m-d', strtotime( $schedule ) ) . ' ' . $repost_at_time );
				}
			}

			if ( 'custom' === $option ) {
				$custom_date_time = $this->get_meta( $post_id, '_wpar_custom_date_time' );
				if ( $custom_date_time ) {
					$datetime_list = preg_split( '/\r\n|\r|\n/', $custom_date_time );
					if ( is_array( $datetime_list ) ) {
						$datetime_list = array_map( 'strtotime', array_filter( $datetime_list, array( $this, 'validate_date' ) ) );
						sort( $datetime_list );
						for ( $x = 0; $x < count( $datetime_list ); $x++ ) {
							$datetime = $datetime_list[ $x ];
							if ( $datetime > $timestamp ) {
								break;
							}
						}
						if ( isset( $datetime ) ) {
							$this->update_meta( $post_id, '_wpar_repost_schedule', date( 'm/d/Y', $datetime ) );
							$this->update_meta( $post_id, '_wpar_repost_schedule_custom', $datetime );
						}
					}
				}
			}

			if ( ! isset( $datetime ) || ! is_numeric( $datetime ) ) {
				$datetime = strtotime( '-1 day', $timestamp );
			}

			// flag to indicate that the post is saving.
			$this->update_meta( $post_id, 'wpar_post_is_saving', true );

			// fire cron generate action
			$this->do_action( 'single_post_updated', $post_id, $datetime, true );
		}
		// phpcs:enable

		return $actual_post_id;
	}

	/**
	 * Generate metabox nonce
	 *
	 * @param int $post_id The post ID.
	 */
	private function nonce( $name, $referer = true, $echo = true ) {
		\wp_nonce_field( 'wpar_nonce_' . $name, 'wpar_metabox_' . $name . '_nonce', $referer, $echo );
	}

	/**
	 * Verify metabox nonce
	 *
	 * @param int $post_id The post ID.
	 */
	private function verify( $name ) {
		// phpcs:disable WordPress.Security.ValidatedSanitizedInput.MissingUnslash, WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		if ( ! isset( $_REQUEST[ 'wpar_metabox_' . $name . '_nonce' ] ) || ! \wp_verify_nonce( $_REQUEST[ 'wpar_metabox_' . $name . '_nonce' ], 'wpar_nonce_' . $name ) ) {
			return false;
		}
		// phpcs:enable
		
		return true;
	}
}