<?php
/**
 * The file for Fetch Posts.
 *
 * @since      1.2.5
 * @package    RevivePress
 * @subpackage RevivePress\Core\Premium\Rules
 * @author     Sayan Datta <iamsayan@protonmail.com>
 */

namespace RevivePress\Core\Premium\Rules;

use RevivePress\Core\Premium\SingleRepublish;

defined( 'ABSPATH' ) || exit;

/**
 * Get Posts class.
 */
class GetPosts extends SingleRepublish
{
	/**
	 * Register functions.
	 */
	public function register() {
		$this->action( 'wpar/run_republish_rule_event', 'run_republish_process' );
		$this->action( 'wpar/republish_rule_flat_posts', 'fetch_posts', 10, 2 );
		$this->action( 'wpar/republish_rule_flat_posts_completed', 'generate_action' );
		$this->action( 'wpar/process_rule_flat_batches', 'set_rule_schedule', 10, 2 );
	}

    /**
	 * Run single republish event once.
	 */
    public function run_republish_process( $rule_id ) {
		// prepare actions.
		$this->get_eligible_posts( $rule_id );

		// update schedule
		$this->update_single_schedule( $rule_id, true );
	}

	/**
	 * Get eligible posts.
	 */
	private function get_eligible_posts( $rule_id ) {
		$this->delete_meta( $rule_id, 'wpar_republishable_post_ids' );
		
		$post_types = $this->get_meta( $rule_id, '_wpar_repost_post_types', array( 'post' ), true );
        if ( ! empty( $post_types ) ) {
			$counter = 0;
    		foreach ( $post_types as $post_type ) {
				++$counter;
				$this->schedule_single_action( time() + ( 10 * ( $counter / 2 ) ), 'wpar/republish_rule_flat_posts', array( $post_type, $rule_id ) );
			}

			$this->schedule_single_action( time() + ( 10 * ( ( $counter + 1 ) / 2 ) ), 'wpar/republish_rule_flat_posts_completed', array( $rule_id ) );
    	}
	}

	/**
	 * Post update process Wrapper.
	 * 
	 * @since 1.2.2
	 * @param int   $rule_id  Post ID
	 */
	public function fetch_posts( $post_type, $rule_id ) {
		$timestamp = $this->current_timestamp();
		
		$post_statuses = $this->get_meta( $rule_id, '_wpar_repost_post_statuses', array( 'publish' ), true );
		$eligibility_age = $this->get_meta( $rule_id, '_wpar_repost_eligibility_age', '' );
		$eligibility_age_max = $this->get_meta( $rule_id, '_wpar_repost_eligibility_age_max', '' );
		$eligibility_exclude_before = $this->get_meta( $rule_id, '_wpar_repost_eligibility_exclude_before', '' );

		$post_order = $this->get_meta( $rule_id, '_wpar_repost_post_order', 'old_first' );
		$post_orderby = $this->get_meta( $rule_id, '_wpar_repost_post_orderby', 'date' );
		$thumbnail_status = $this->get_meta( $rule_id, '_wpar_repost_check_thumbnail', 'disable' );

		$args = array(
    		'post_status' => ( array ) $post_statuses,
    		'post_type'   => $post_type,
    		'numberposts' => -1,
			'fields'      => 'ids',
			'orderby'     => $post_orderby,
			'meta_query'  => array(
				'relation' => 'AND',
				array(
					'key'     => 'wpar_global_republish_status',
					'compare' => 'NOT EXISTS',
				),
				array(
					'key'     => 'wpar_single_republish_status',
					'compare' => 'NOT EXISTS',
				),
				array(
					'key'     => 'wpar_exclude_republish_rule',
					'compare' => 'NOT EXISTS',
				),
			),
    	);

		if ( $thumbnail_status == 'enable' ) {
			$args['meta_query'][] = array(
				'key'     => '_thumbnail_id',
				'compare' => 'EXISTS',
			);
		}

		if ( ! empty( $post_order ) ) {
			$args['order'] = ( $post_order == 'new_first' ) ? 'DESC' : 'ASC';
		}

		if ( ! empty( $eligibility_age ) ) {
			$eligibility_age = $this->str_to_second( $eligibility_age );
			$args['date_query'][0]['before'] = $this->do_filter( 'post_before_date_republish_rule', date( 'Y-m-d H:i:s', strtotime( "-$eligibility_age seconds", $timestamp ) ), $timestamp );
		}

		$after_date = '';
		if ( ! empty( $eligibility_age_max ) ) {
			$eligibility_age_max = $this->str_to_second( $eligibility_age_max );
			$after_date = date( 'Y-m-d H:i:s', strtotime( "-$eligibility_age_max seconds", $timestamp ) );
		}

		if ( ! empty( $eligibility_exclude_before ) ) {
			$after_date = date_format( date_create_from_format( 'd/m/Y', $eligibility_exclude_before ), 'Y-m-d H:i:s' );
		}

		if ( ! empty( $after_date ) ) {
			$args['date_query'][0]['after'] = $this->do_filter( 'post_after_date_republish_rule', $after_date, $timestamp );
			$args['date_query'][0]['inclusive'] = true;
		}

		// Filter Taxonomies
		$taxonomies_filter = $this->get_meta( $rule_id, '_wpar_repost_taxonomies_filter', 'include' ); 
		$taxonomies = $this->get_meta( $rule_id, '_wpar_repost_taxonomies', array(), true ); 

    	$cats = $tags = $terms = array();
    	if ( ! empty( $taxonomies ) ) {
    		foreach ( $taxonomies as $taxonomy ) {
				$taxonomy_data = $this->process_taxonomy( $taxonomy );
				$taxonomy_name = $taxonomy_data[0];
    			$term_id = $taxonomy_data[1];

    			if ( is_object_in_taxonomy( $post_type, $taxonomy_name ) ) {
					if ( 'category' === $taxonomy_name ) {
                        $cats[] = $term_id;
    		        } elseif ( 'post_tag' === $taxonomy_name ) {
                        $tags[] = $term_id;
				    } else {
						$terms[ $taxonomy_name ][] = $term_id;
					}
    		    }
			}
			
    	    if ( ! empty( $cats ) ) {
				if ( 'include' === $taxonomies_filter ) {
    		        $args['category__in'] = wp_parse_id_list( $cats );
    		    } else {
					$args['category__not_in'] = wp_parse_id_list( $cats );
				}
    		}

    		if ( ! empty( $tags ) ) {
				if ( 'include' === $taxonomies_filter ) {
					$args['tag__in'] = wp_parse_id_list( $tags );
				} else {
					$args['tag__not_in'] = wp_parse_id_list( $tags );
				}
    		}
			
			if ( ! empty( $terms ) ) {
				$array = array();
		    	if ( count( $terms ) > 1 ) {
		    		$array['relation'] = 'OR';
		    	}

    	    	foreach ( $terms as $taxonomy => $term_ids ) {
    	    		$array[] = array(
    	    			'taxonomy' => $taxonomy,
    	    			'field'    => 'term_id',
    	    			'terms'    => wp_parse_id_list( $term_ids ),
    	    			'operator' => ( 'exclude' === $taxonomies_filter ) ? 'NOT IN' : 'IN',
		    		);
		    	}

    	    	$args['tax_query'] = $array;
		    }
		}

		// Authot filter
		$authors_filter = $this->get_meta( $rule_id, '_wpar_repost_authors_filter', 'include' ); 
		$authors = $this->get_meta( $rule_id, '_wpar_repost_authors', array(), true ); 
		if ( ! empty( $authors ) ) {
			if ( $authors_filter == 'include' ) {
				$args['author__in'] = wp_parse_id_list( $authors );
			} elseif ( $author_filters == 'exclude' ) {
				$args['author__not_in'] = wp_parse_id_list( $authors );
			}
		}

		$args = $this->do_filter( 'query_args_rule', $args );

		// get posts
		$posts = $this->get_posts( $args );

		// store post ids
		$this->store_post_ids( $posts, $post_type, $rule_id );
	}

	/**
	 * Complete
	 */
	public function generate_action( $rule_id ) {
		$timestamp = $this->current_timestamp();
		$post_types = $this->get_meta( $rule_id, '_wpar_repost_post_types', array(), true );
		$post_order = $this->get_meta( $rule_id, '_wpar_repost_post_order', 'old_first' );
		$post_orderby = $this->get_meta( $rule_id, '_wpar_repost_post_orderby', 'date' );
		$number_posts = $this->get_meta( $rule_id, '_wpar_repost_number_posts', 1 );
		$thumbnail_status = $this->get_meta( $rule_id, '_wpar_repost_check_thumbnail' );

		$post_ids = $this->get_meta( $rule_id, 'wpar_republishable_post_ids' );
		if ( ! $post_ids || ! is_array( $post_ids ) ) {
			$post_ids = array();
		}
    
        if ( ! empty( $post_ids ) ) {
			$sticky_posts = get_option( 'sticky_posts' );
			if ( 'enable' === $thumbnail_status && ! empty( $sticky_posts ) && is_array( $sticky_posts ) ) {
				$post_ids = array_diff( $post_ids, $sticky_posts );
			}

    	    $args = array(
				'post_type'   => $post_types,
				'post_status' => 'any',
    	    	'post__in'    => $post_ids,
    	    	'numberposts' => $number_posts,
    	    	'orderby'     => $post_orderby,
				'fields'      => 'ids',
			);

			if ( ! empty( $post_order ) ) {
    	    	$args['order'] = ( $post_order == 'new_first' ) ? 'DESC' : 'ASC';
			}
			
			$args = $this->do_filter( 'post_query_args_rule', $args, $rule_id );
			$post_ids = $this->do_filter( 'filtered_post_ids', $this->get_posts( $args ) );

			$this->schedule_batch_actions( $post_ids, 'wpar/process_rule_flat_batches', 50, 10, array( $rule_id ) );
		}

		// delete reference
		$this->delete_meta( $rule_id, 'wpar_republishable_post_ids' );
	}

	/**
	 * Generate Single cron time.
	 * 
	 * @since 1.3.4
	 * @param array   $post_ids  Post IDs
	 * @param int     $rule_id   Rule ID
	 */
	public function set_rule_schedule( array $post_ids, int $rule_id ) {
		$randomness = $this->get_meta( $rule_id, '_wpar_repost_post_randomness', '' );
		$republish_action = $this->get_meta( $rule_id, '_wpar_repost_republish_action', 'repost' );

		foreach ( $post_ids as $post_id ) {
			// delete previosly scheduled hook if exists any.
			$this->unschedule_all_actions( 'wpar/global_republish_single_post', array( $post_id ) );

			// get the required date time
			$datetime = time();
			if ( $randomness && is_numeric( $randomness ) ) {
				$datetime = $datetime + wp_rand( 0, $randomness );
			}

			// schedule single post republish event
			$action_id = $this->schedule_single_action( $datetime, 'wpar/global_republish_single_post', array( $post_id ) );

			$local_timestamp = get_date_from_gmt( date( 'Y-m-d H:i:s', $datetime ) );
					
			// update required post metas
			$this->update_meta( $post_id, 'wpar_global_republish_status', 'pending' );
			$this->update_meta( $post_id, '_wpar_global_republish_datetime', $local_timestamp );
			$this->update_meta( $post_id, 'wpar_filter_republish_status', 'pending' );
			$this->update_meta( $post_id, '_wpar_filter_republish_datetime', $local_timestamp );
			$this->update_meta( $post_id, 'wpar_republish_rule_action', $republish_action );

			// store action id
			$this->update_meta( $post_id, 'wpar_republish_as_action_id', $action_id );

			// set log action
			$this->do_action( 'insert_log', $post_id, array(
				'timestamp' => $local_timestamp,
				'action'    => 'cron',
			) );
		}
	}

	/**
     * Store Post IDs
     * 
     * @since 1.3.0
     */
    private function store_post_ids( $ids, $post_type, $rule_id ) {
		$post_ids = $this->get_meta( $rule_id, 'wpar_republishable_post_ids' );

		if ( ! $post_ids || ! is_array( $post_ids ) ) {
			$post_ids = array();
		}

		$post_ids = $this->do_filter( 'rules_post_ids_before_store', $post_ids, $post_type, $rule_id );
		$this->update_meta( $rule_id, 'wpar_republishable_post_ids', \wp_parse_id_list( array_merge( $post_ids, $ids ) ) );
	}
}