<?php
/**
 * Clear Single Post Cache.
 *
 * @since      1.2.5
 * @package    RevivePress
 * @subpackage RevivePress\Core\Premium\Rules
 * @author     Sayan Datta <iamsayan@protonmail.com>
 */

namespace RevivePress\Core\Premium\Rules;

use RevivePress\Helpers\Hooker;
use RevivePress\Helpers\Premium\Extras;
use RevivePress\Helpers\Premium\GetMetaData;

defined( 'ABSPATH' ) || exit;

/**
 * Single post cache class.
 */
class PostType
{
	use Extras;
    use GetMetaData;
    use Hooker;

	/**
	 * Republish Rule post type.
	 *
	 * @var string
	 */
	private $post_type = 'republish_rule';

	/**
	 * Register functions.
	 */
	public function register() {
		$this->action( 'init', 'post_type' );
	}

	/**
	 * Register Custom post type
	 */
	public function post_type() {
		register_extended_post_type( $this->post_type,
		array(
			'supports'            => array( 'title' ),
	    	'hierarchical'        => false,
	    	'public'              => false,
	    	'show_ui'             => true,
	    	'show_in_menu'        => 'revivepress',
	    	'show_in_admin_bar'   => true,
	    	'show_in_nav_menus'   => false,
	    	'can_export'          => true,
	    	'has_archive'         => false,
	    	'exclude_from_search' => true,
	    	'publicly_queryable'  => false,
			'capabilities'        => $this->get_capabilities( $this->post_type ),
			'map_meta_cap'        => true,
			'rewrite'             => false,
			'query_var'           => false,
			'labels'              => array(
				'all_items' => __( 'Republish Rules', 'wp-auto-republish' ),
			),
			'quick_edit'          => false,
			'enter_title_here'    => __( 'Enter republish rule name', 'wp-auto-republish' ),
			'admin_cols'          => array(
				'date',
				'rvp_schedule' => array(
					'title'    => __( 'Republish Info', 'wp-auto-republish' ),
					'function' => array( $this, 'column_data' ),
				),
			),
		),
		array(
			'singular' => __( 'Republish Rule', 'wp-auto-republish' ),
			'plural'   => __( 'Republish Rules', 'wp-auto-republish' ),
			'slug'     => $this->post_type,
		) );
	}

	/**
	 * Display column data.
	 */
	public function column_data() {
		global $post;

		$timestamp = $this->current_timestamp();
		$format = $this->do_filter( 'admin_column_date_format', 'M j, Y' );
		$full_format = $this->do_filter( 'admin_column_full_datetime_format', get_option( 'date_format' ) . ' @ ' . get_option( 'time_format' ) );

		$schedule = $this->get_meta( $post->ID, 'wpar_republish_rule_next_timestamp' );
		$info = '';

		if ( $schedule ) {
			$new_timestamp = strtotime( $schedule );
			$timediff = $new_timestamp - $timestamp;
			if ( ( $timediff <= 86400 ) && ( $timediff > 0 ) ) {
				$info .= sprintf(
					'<abbr title="%1$s">%2$s %3$s</abbr><br />',
					date_i18n( $full_format, $new_timestamp ),
					__( 'Republication starts in', 'wp-auto-republish' ),
					human_time_diff( $timestamp, $new_timestamp )
				);
			} elseif ( $timediff <= 0 ) {
				$info .= sprintf(
					'<abbr title="%1$s">%2$s</abbr><br />',
					date_i18n( $full_format, $new_timestamp ),
					__( 'Republication in queue', 'wp-auto-republish' )
				);
			} elseif ( $timediff > 86400 ) {
				$info .= sprintf(
					'<abbr title="%1$s">%2$s: <strong>%3$s</strong></abbr><br />',
					date_i18n( $full_format, $new_timestamp ), __( 'Next Republish', 'wp-auto-republish' ), date_i18n( $format, $new_timestamp )
				);
			}
			
			// show republish schedules
			$info .= $this->get_schedule( $post->ID, 'edit', true );
		} else {
			$info = __( 'Rule is not scheduled.', 'wp-auto-republish' );
		}

		if ( 'publish' !== $post->post_status ) {
			$info = __( 'Rule is not published.', 'wp-auto-republish' );
		}

		echo wp_kses_post( $info );
	}
}