<?php
/**
 * Rule Actions.
 *
 * @since      1.2.5
 * @package    RevivePress
 * @subpackage RevivePress\Core\Premium\Rules
 * @author     Sayan Datta <iamsayan@protonmail.com>
 */

namespace RevivePress\Core\Premium\Rules;

use RevivePress\Helpers\Hooker;
use RevivePress\Helpers\Scheduler;
use RevivePress\Helpers\HelperFunctions;

defined( 'ABSPATH' ) || exit;

/**
 * Post Metabox class.
 */
class RuleActions
{
	use HelperFunctions;
    use Hooker;
    use Scheduler;

	/**
	 * Register functions.
	 */
	public function register() {
		$this->action( 'wpar/republish_rule_updated', 'set_rule_event', 10, 2 );
		$this->action( 'trash_republish_rule', 'remove_schedule' );
		$this->action( 'draft_republish_rule', 'remove_schedule' );
		$this->action( 'pending_republish_rule', 'remove_schedule' );
		$this->action( 'post_submitbox_misc_actions', 'submitbox_edit', 5 );
		$this->action( 'wpar/regenerate_rule_schedule', 'regenerate_schedule' );
	}

	/**
	 * Generate Single Cron if required
	 * 
	 * @param int  $rule_id   Rule ID
	 * @param int  $datetime  Cron schedule
	 */
	public function set_rule_event( $rule_id, $datetime ) {
		$this->unschedule_all_actions( 'wpar/run_republish_rule_event', array( $rule_id ) );
		$this->delete_meta( $rule_id, 'wpar_post_is_saving' );
		$this->delete_meta( $rule_id, 'wpar_republish_rule_next_timestamp' );
		
		$option = $this->get_meta( $rule_id, '_wpar_repost_option' );
		if ( 'disable' === $option ) {
			return;
		}

		$datetime = $this->do_filter( 'republish_rule_timestamp', $datetime, $rule_id );

		$formatted_datetime = date( 'Y-m-d H:i:s', $datetime );
		$timestamp = time();
		$new_datetime = get_gmt_from_date( $formatted_datetime, 'U' );

		if ( $new_datetime > $timestamp ) {
            // schedule single post republish event
		    $action_id = $this->schedule_single_action( $new_datetime, 'wpar/run_republish_rule_event', array( $rule_id ) );

			// update required post metas
			$this->update_meta( $rule_id, 'wpar_republish_rule_next_timestamp', $formatted_datetime );
			$this->update_meta( $rule_id, '_wpar_repost_schedule_datetime', $formatted_datetime );
			$this->update_meta( $rule_id, '_wpar_filter_republish_datetime', $formatted_datetime );

			// store action id
			$this->update_meta( $rule_id, 'wpar_republish_as_action_id', $action_id );
		}
	}

	/**
	 * Remove Tasks
	 * 
	 * @param int  $rule_id   Post ID
	 */
	public function remove_schedule( $rule_id ) {
		// clear cron event if any
        $this->unschedule_all_actions( 'wpar/run_republish_rule_event', array( $rule_id ) );
	}

	/**
     * Process regenerate schedule
     */
	public function regenerate_schedule() {
		$args = array(
			'post_type'   => 'republish_rule',
			'numberposts' => -1,
			'post_status' => 'publish',
			'fields'      => 'ids',
			'meta_query'  => array(
				array(
				    'key'     => '_wpar_filter_republish_datetime',
    			    'compare' => 'EXISTS',
			    ),
			),
		);
		$post_ids = $this->get_posts( $args );
		foreach ( $post_ids as $post_id ) {
			$datetime = $this->get_meta( $post_id, '_wpar_filter_republish_datetime' );
			$utc_datetime = get_gmt_from_date( $datetime, 'U' );
			$timestamp = time();

			if ( $utc_datetime > $timestamp ) {
				$this->set_rule_event( $post_id, $datetime );
			} else {
				$this->do_action( 'update_schedule', $post_id, true );
			}
		}
	}

	/**
	 * Post Submit Box HTML output.
	 * 
	 * @param string $post WP Post
	 */
	public function submitbox_edit( $post ) {
		if ( 'republish_rule' === $post->post_type ) {
		    // hides everything except the 'publish' button in the 'publish'-metabox
		    echo '<style type="text/css"> #duplicate-action, #minor-publishing-actions, #misc-publishing-actions, #preview-action { display: none !important; } </style>';
		}
	}
}