<?php
/**
 * Premium Features.
 *
 * @since      1.3.2
 * @package    RevivePress
 * @subpackage RevivePress\Core\Premium
 * @author     Sayan Datta <iamsayan@protonmail.com>
 */

namespace RevivePress\Core\Premium;

use WP_Post;
use RevivePress\Helpers\Hooker;
use RevivePress\Helpers\HelperFunctions;

defined( 'ABSPATH' ) || exit;

/**
 * Actions class.
 */
class ScheduledEvent
{
	use HelperFunctions;
    use Hooker;

	/**
	 * Register functions.
	 */
	public function register() {
		$this->action( 'publish_future_post', 'republish_post' );
		$this->action( 'before_delete_post', 'remove_trace' );
		$this->filter( 'wp_insert_post_data', 'lock_status', 99, 2 );
		$this->filter( 'display_post_states', 'post_states', 20, 2 );
		$this->action( 'admin_notices', 'show_notice' );
		$this->action( 'admin_footer-post.php', 'block_editor_notice', 999 );
	}

	/**
	 * Trigger on future post publish event.
	 * 
	 * @param int   $post_id  WP Post ID
	 */
	public function republish_post( $post_id ) {
		$original_post_id = $this->get_meta( $post_id, 'wpar_original_post_id' );

		// break early if given post is not an actual scheduled post created by this plugin.
		if ( ! $original_post_id ) {
			return $post_id;
		}

		$original_post = \get_post( $original_post_id );
		if ( ! $original_post ) {
			// delete old original post
			\wp_delete_post( $original_post->ID, false );
		}

		$post = \get_post( $post_id );
		
		$post->post_name = $original_post->post_name;
		$post->guid = $original_post->guid;
		$post->post_parent = $original_post->post_parent;
		$post->comment_count = $original_post->comment_count;
		$post->post_status = 'publish';
		$post->post_date = current_time( 'mysql' );
		$post->post_date_gmt = current_time( 'mysql', true );

		$this->delete_meta( $post->ID, 'wpar_original_post_id' );
		$this->delete_meta( $post->ID, 'wpar_filter_republish_status' );

		// remove kses filters
		\kses_remove_filters();

		// Remove WordPress default filter so a new revision is not created on republish.
		\remove_action( 'post_updated', 'wp_save_post_revision', 10 );

		/**
		 * Yoast SEO and other plugins prevent from accidentally updating another post's
		 * data (e.g. the Yoast SEO metadata by checking the $_POST data ID with the post object ID.
		 * We need to overwrite the $_POST data ID to allow updating the original post.
		 */
		$_POST['ID'] = $original_post->ID;

		// Update post
		wp_update_post( $post );

		// Re-enable the creation of a new revision.
		\add_action( 'post_updated', 'wp_save_post_revision', 10, 1 );

		// reinit kses filters
		\kses_init_filters();
		
		// delete old original post
		\wp_delete_post( $original_post->ID, true );

		$this->do_action( 'insert_log', $post->ID, 'scheduled' );
		$this->do_action( 'old_post_republished', $post->ID, $post->post_date, 'scheduled' );

		return $post_id;
	}

	/**
	 * Trigger on future post publish event.
	 * 
	 * @param int   $post_id  WP Post ID
	 */
	public function remove_trace( $post_id ) {
		$original_post_id = $this->get_meta( $post_id, 'wpar_original_post_id' );
		
		// break early if given post is not an actual scheduled post created by this plugin.
		if ( ! $original_post_id ) {
			return;
		}

		$this->delete_meta( $original_post_id, 'wpar_rewrite_post_id' );
	}

		/**
	 * Lock post info on update.
	 * 
	 * @param object   $data     Old Data
	 * @param object   $postarr  Current Data
	 * 
	 * @return object  $data
	 */
	public function lock_status( $data, $postarr ) {
		if ( ! \array_key_exists( 'ID', $postarr ) || empty( $postarr['ID'] ) ) {
			return $data;
		}

		$post = \get_post( $postarr['ID'] );
		if ( ! $post ) {
			return $data;
		}

		// allow only trash post status
		if ( 'trash' === $postarr['post_status'] ) {
			return $data;
		}

		$timestamp = $this->current_timestamp();
		$new_time = date( 'Y-m-d H:i:s', $timestamp + 86400 ); // add extra 1 day

        // check if is a actual scheduled republishing
		$scheduled_pending = $this->get_meta( $postarr['ID'], 'wpar_original_post_id' );
    	if ( isset( $postarr['post_date'] ) && $scheduled_pending ) {
		    $data['post_status'] = 'future';
			// prevent backwards timestamp
			if ( $timestamp >= strtotime( $postarr['post_date'] ) ) {
				$data['post_date'] = $new_time;
				$data['post_date_gmt'] = get_gmt_from_date( $new_time );
				$data['post_modified'] = $new_time;
				$data['post_modified_gmt'] = get_gmt_from_date( $new_time );
			}

			// update required post meta
			$this->update_meta( $postarr['ID'], '_wpar_filter_republish_datetime', $postarr['post_date'] );
		}

		return $data;
	}

	
	/**
	 * Filter admin states to show scheduled republish nag.
	 * 
	 * @param array   $states   Post States
	 * @param object  $post     Post Object
	 */
	public function post_states( $states, $post ) {
		// If post status is not future, return default states
		if ( ! in_array( $post->post_status, array( 'future', 'trash' ) ) ) {
			return $states;
		}

		$scheduled_pending = $this->get_meta( $post->ID, 'wpar_original_post_id' );
    	if ( $scheduled_pending ) {
		    $states['rvp_scheduled'] = sprintf( 
				/* translators: Original Post Id. */
				__( 'Republish Scheduled (#%s)', 'wp-auto-republish' ), $scheduled_pending 
			); // Adds the label of the current post status
		}

		return $states;
	}

	/**
	 * Show internal admin notices.
	 */
	public function show_notice() {
		if ( ! $this->is_classic_editor() ) {
			return;
		}

		$post = \get_post();

		if ( ! $post instanceof WP_Post ) {
			return;
		}

		$content = $this->get_notice_text( $post );
		if ( ! empty( $content ) ) { ?>
			<div class="notice notice-warning is-dismissible"><p><?php echo esc_html( $content ); ?></p></div>
		<?php }
	}

	/**
	 * Show internal admin notices.
	 */
	public function block_editor_notice() {
		if ( ! $this->is_block_editor() ) {
			return;
		}

		$post = \get_post();

		if ( ! $post instanceof WP_Post ) {
			return;
		}

		$content = $this->get_notice_text( $post );
		if ( ! empty( $content ) ) {  ?>
			<script>
				( function( wp ) {
					wp.data.dispatch( 'core/notices' ).createNotice(
						'warning',
						'<?php echo esc_js( $content ); ?>', // Text string to display.
						{
							isDismissible: false, // Whether the user can dismiss the notice.
						}
					);
				} )( window.wp );
			</script><?php
		}
	}

	/**
	 * Generates the translated text for the notice.
	 *
	 * @param WP_Post $post The current post object.
	 *
	 * @return string The translated text for the notice.
	 */
	private function get_notice_text( $post ) {
		$scheduled_copy = $this->get_meta( $post->ID, 'wpar_rewrite_post_id' );
		if ( $scheduled_copy ) {
			$status = get_post_status( $scheduled_copy );
			if ( 'trash' === $status ) {
				return __(
					'RevivePress: You can only make one Rewrite & Republish duplicate at a time, and a duplicate of this post already exists in the trash. Permanently delete it if you want to make a new duplicate.',
					'wp-auto-republish'
				);
			} else {
				return sprintf(
					/* translators: %1$s: scheduled date of the copy, %2$s: scheduled time of the copy. */
					__(
						'RevivePress: A duplicate of this post was made, which is scheduled to replace this post on %1$s at %2$s.  Please note that any changes you make to this post will be replaced when the duplicated version is republished.',
						'wp-auto-republish'
					),
					get_the_time( get_option( 'date_format' ), $scheduled_copy ),
					get_the_time( get_option( 'time_format' ), $scheduled_copy )
				);
			}
		}

		$original_post_id = $this->get_meta( $post->ID, 'wpar_original_post_id' );
		if ( $original_post_id ) {
			return \sprintf(
				/* translators: %1$s: scheduled date of the copy, %2$s: scheduled time of the copy. */
				__(
					'This post is scheduled to replace the post %1$s on %2$s at %3$s.',
					'wp-auto-republish'
				),
				get_the_title( $original_post_id ),
				get_the_time( get_option( 'date_format' ) ),
				get_the_time( get_option( 'time_format' ) )
			);
		}
		return '';
	}
}