<?php
/**
 * The file for Single Republish.
 *
 * @since      1.1.0
 * @package    RevivePress
 * @subpackage RevivePress\Core\Premium
 * @author     Sayan Datta <iamsayan@protonmail.com>
 */

namespace RevivePress\Core\Premium;

use RevivePress\Core\PostRepublish;

defined( 'ABSPATH' ) || exit;

/**
 * Single post republish class.
 */
class SingleRepublish extends PostRepublish
{
	/**
	 * Register functions.
	 */
	public function register() {
		$this->action( 'wpar/single_post_updated', 'set_single_event', 10, 3 );
		$this->action( 'wpar/run_single_republish', 'update_process' );
		$this->action( 'wpar/update_schedule', 'update_single_schedule', 10, 2 );
		$this->action( 'wpar/regenerate_schedule', 'regenerate_schedule' );
	}

	/**
	 * Generate Single Cron if required
	 * 
	 * @param int  $post_id     Post ID
	 * @param int  $datetime    Cron schedule
	 * @param bool $add_random  Add randomness
	 */
	public function set_single_event( $post_id, $datetime, $add_random ) {
		$this->unschedule_all_actions( 'wpar/run_single_republish', array( $post_id ) );
		$this->delete_meta( $post_id, 'wpar_post_is_saving' );
		$this->delete_meta( $post_id, 'wpar_single_republish_status' );

		$global_pending = $this->get_meta( $post_id, 'wpar_global_republish_status' );
		if ( ! $global_pending ) {
		    $this->delete_meta( $post_id, 'wpar_filter_republish_status' );
		}

		if ( ! $this->can_republish( $post_id ) ) {
			return;
		}

		$datetime = is_numeric( $datetime ) ? $datetime : strtotime( $datetime );
		$interval = $this->do_filter( 'random_single_republish_interval', 5 ) * MINUTE_IN_SECONDS;
		$option = $this->get_meta( $post_id, '_wpar_repost_option' );
		$repeats = $this->get_meta( $post_id, '_wpar_repost_repeats' );
		if ( $add_random && 'repeat' === $option && ! in_array( $repeats, array( 'minutes', 'hourly' ) ) ) {
			$datetime = $datetime + wp_rand( 0, $interval );
		}

		$datetime = $this->do_filter( 'single_republish_timestamp', $datetime, $post_id );

		$formatted_datetime = date( 'Y-m-d H:i:s', $datetime );
		$timestamp = time();
		$new_datetime = get_gmt_from_date( $formatted_datetime, 'U' );

		if ( $new_datetime > $timestamp ) {
			// schedule single post republish event
			$action_id = $this->schedule_single_action( $new_datetime, 'wpar/run_single_republish', array( $post_id ) );
		  
			// update required post metas
			$this->update_meta( $post_id, 'wpar_single_republish_status', 'pending' );
			$this->update_meta( $post_id, 'wpar_filter_republish_status', 'pending' );
			$this->update_meta( $post_id, '_wpar_repost_schedule_datetime', $formatted_datetime );
		    $this->update_meta( $post_id, '_wpar_filter_republish_datetime', $formatted_datetime );

			// store action id
			$this->update_meta( $post_id, 'wpar_republish_as_action_id', $action_id );

			$this->do_action( 'insert_log', $post_id, array(
				'action' => 'single_cron',
				'info'   => sprintf( 
					/* translators: Date & time in localized format. */
					__( 'Single Scheduled at: %s', 'wp-auto-republish' ), 
					date_i18n( get_option( 'date_format' ) . ' @ ' . get_option( 'time_format' ), $datetime ) 
				),
			) );
		}
	}

	/**
	 * Post update process Wrapper.
	 * 
	 * @since 1.2.2
	 * @param int   $post_id  Post ID
	 */
	public function update_process( $post_id ) {
		// set occurence
		$this->set_occurence( $post_id );

		// delete metas
		$this->delete_meta( $post_id, 'wpar_single_republish_status' );
		$this->delete_meta( $post_id, '_wpar_repost_schedule_datetime' );
		$this->remove_meta( $post_id );
			
		// Update next post schedule if nescessary
		$this->update_single_schedule( $post_id, false );

		// run republish process
		$this->run_update_process( $post_id );
	}

	/**
	 * Run post update process.
	 * 
	 * @since 1.1.7
	 * @param int   $post_id  Post ID
	 */
	protected function run_update_process( $post_id ) {
		$action = $this->do_filter( 'republish_action', 'repost', true, $post_id );
	    if ( $action === 'repost' ) {
	        $this->update_old_post( $post_id, true );
	    } elseif ( $action === 'clone' ) {
	        $this->clone_old_post__premium_only( $post_id, true );
	    } elseif ( $action === 'update' ) {
			$this->update_old_post( $post_id, true, false, true );
		}

		return $post_id;
	}

	/**
	 * Set single Re-Post next schedule
	 * 
	 * @param int $post_id Post ID
	 */
	public function update_single_schedule( $post_id, $is_rule ) {
		$option = $this->get_meta( $post_id, '_wpar_repost_option' );
		if ( ! $option ) {
			return;
		}

		// Abort.
		if ( in_array( $option, array( 'days', 'date', 'disable' ) ) ) {
			$get_status = $this->get_meta( $post_id, '_wpar_repost_republish_status', 'default' ); 
			if ( 'default' !== $get_status && 'disable' !== $option ) {
				wp_update_post( array(
					'ID'          => $post_id,
					'post_status' => $get_status,
				) );
			}
			$this->update_meta( $post_id, '_wpar_repost_done', 'yes' );
		    return;
		}

		// Get current timestamp.
		$timestamp = $this->current_timestamp();
		
		// Get metas.
		$number = $this->get_meta( $post_id, '_wpar_repost_after_number' );
		$time = $this->get_meta( $post_id, '_wpar_repost_after_time' );
		$repeats = $this->get_meta( $post_id, '_wpar_repost_repeats' );
		$every = intval( $this->get_meta( $post_id, '_wpar_repost_every' ) );
		$weekly = maybe_unserialize( $this->get_meta( $post_id, '_wpar_repost_on' ) );
		if ( empty( $weekly ) ) $weekly = array();
		
		$monthly = $this->get_meta( $post_id, '_wpar_repost_by' );
		$monthly_on = maybe_unserialize( $this->get_meta( $post_id, '_wpar_repost_monthly_on' ) );
		if ( empty( $monthly_on ) ) $monthly_on = array();
	
		$end = $this->get_meta( $post_id, '_wpar_repost_end' );
		$occurence = $this->get_meta( $post_id, '_wpar_repost_end_after' );
		
		$on = strtotime( $this->get_meta( $post_id, '_wpar_repost_on_date' ) );
		$end_on = $this->get_meta( $post_id, '_wpar_repost_end_on' );
		$get_status = $this->get_meta( $post_id, '_wpar_repost_last_republish_status' ); 

		$schedule = strtotime( $this->get_meta( $post_id, '_wpar_repost_schedule' ) );
		$today = date( 'm/d/Y', $this->current_timestamp() );
		
		$repost_at_time = $this->get_meta( $post_id, '_wpar_repost_at_specific_time' );
		$datetime = ( $repost_at_time ) ? date( 'H:i:s', strtotime( $repost_at_time ) ) : '00:00:00';
		$schedule = strtotime( date( 'Y-m-d', $schedule ) . ' ' . $datetime );
		$schedule_generate = true;

		if ( 'repeat' === $option ) {
			switch ( $repeats ) {
				case 'yearly':
					$new_schedule = date( 'm/d/Y', $this->filter_schedule( strtotime( '+'. $every .' years', $schedule ), 'year' ) );
					break;
				
				case 'monthly':
					$new_schedule = date( 'm/d/Y', $this->filter_schedule( strtotime( '+'. $every .' months', $schedule ), 'month' ) );
					if ( $monthly == 'week' ) {
						$new_schedule = date( 'm/d/Y', strtotime( $this->literal_date( date( 'Y-m-d', $schedule ), strtotime( $new_schedule ) ) ) );
					}
					break;
				
				case 'weekly':
					$new_schedule = date( 'm/d/Y', $this->filter_schedule( strtotime( '+'. $every .' weeks', $schedule ), 'week' ) );
					break;
				
				case 'daily':
					$new_schedule = date( 'm/d/Y', $this->filter_schedule( strtotime( '+'. $every .' days', $schedule ), 'day' ) );
					break;

				case 'hourly':
					$full_timestamp = $this->generate_next_time( $post_id, $schedule, $every, 'hours' );
					$new_schedule = date( 'm/d/Y', $full_timestamp );
					$datetime = date( 'H:i:s', $full_timestamp );
					break;

				case 'minutes':
				default:
					$full_timestamp = $this->generate_next_time( $post_id, $schedule, $every, 'minutes' );
					$new_schedule = date( 'm/d/Y', $full_timestamp );
					$datetime = date( 'H:i:s', $full_timestamp );
					break;
			}
			
			if ( $end == 'on' ) {
				if ( strtotime( $new_schedule ) >= strtotime( $end_on ) ) {
					$new_schedule = date( 'm/d/Y', $schedule );
					$schedule_generate = false;
					$this->update_meta( $post_id, '_wpar_repost_done', 'yes' );
					if ( 'default' !== $get_status ) {
						wp_update_post( array(
							'ID'          => $post_id,
							'post_status' => $get_status,
						) );
					}
				}
			} elseif ( $end == 'after' ) {
				$repeat = $this->get_meta( $post_id, 'wpar_post_republish_count' );
				if ( $repeat && ( $repeat >= $occurence ) ) {
					$new_schedule = date( 'm/d/Y', $schedule );
					$schedule_generate = false;
					$this->update_meta( $post_id, '_wpar_repost_done', 'yes' );
					if ( 'default' !== $get_status ) {
						wp_update_post( array(
							'ID'          => $post_id,
							'post_status' => $get_status,
						) );
					}
				}
			}
		
			$this->update_meta( $post_id, '_wpar_repost_at_specific_time', $datetime );
			$datetime = strtotime( date( 'Y-m-d', strtotime( $new_schedule ) ) . ' ' . $datetime );
		}

		if ( 'custom' === $option ) {
			$custom_date_time = $this->get_meta( $post_id, '_wpar_custom_date_time' );
			if ( $custom_date_time ) {
				$datetime_list = array_map( 'strtotime', preg_split( '/\r\n|\r|\n/', $custom_date_time ) );
				sort( $datetime_list );
				for ( $x = 0; $x < count( $datetime_list ); $x++ ) {
					$datetime = $datetime_list[ $x ];
					if ( $datetime > $timestamp ) {
						break;
					}
				}
			}
		}

		if ( ! isset( $datetime ) || ! is_numeric( $datetime ) ) {
			$datetime = strtotime( '-1 day', $timestamp );
		}

		if ( $schedule_generate ) {
			$this->update_meta( $post_id, '_wpar_repost_schedule', date( 'm/d/Y', $datetime ) );
			$this->update_meta( $post_id, '_wpar_repost_schedule_custom', $datetime );

			// flag to indicate that the post is saving.
			$this->update_meta( $post_id, 'wpar_post_is_saving', true );

			if ( $is_rule ) {
				$this->do_action( 'republish_rule_updated', $post_id, $datetime );
			} else {
				$this->do_action( 'single_post_updated', $post_id, $datetime, true );
			}
		}
	}

	/**
	 * Generate timestamp if it is less than current timestamp
	 * 
	 * @since 1.3.4
	 * @param int    $timestamp  Valid Datetime
	 * @param string $interval   Interval
	 * 
	 * @return int   $timestamp  Valid Datetime
	 */
	public function filter_schedule( $timestamp, $interval ) {
		$current_timestamp = $this->current_timestamp();
		
		while ( $timestamp <= $current_timestamp ) {
			$timestamp = strtotime( '+1 ' . $interval, $timestamp );
		}

		return $timestamp;
	}

	/**
	 * Generate Next Cron Time for Hour and Minute type single republishing.
	 * 
	 * @since 1.1.8
	 * 
	 * @param int    $post_id    Post ID
	 * @param int    $datetime   Actual Timestamp
	 * @param int    $every      No. of repeats
	 * @param string $repeats    Repeat type
	 * 
	 * @return int   $timestamp  Valid Datetime
	 */
	public function generate_next_time( $post_id, $datetime, $every, $repeats ) {
		$date = date( 'Y-m-d', $datetime );
		$new_date = $this->filter_schedule( strtotime( '+'. $every . ' ' . $repeats, $datetime ), $repeats );
		$timestamp = $new_date;

		$start_time = $this->get_meta( $post_id, '_wpar_repost_start_time' );
		$end_time = $this->get_meta( $post_id, '_wpar_repost_end_time' );

		if ( ! empty( $start_time ) && ! empty( $end_time ) ) {
			$start_datetime = strtotime( $date . ' ' . $start_time );
			$end_datetime = strtotime( $date . ' ' . $end_time );

			if ( ( $new_date >= $start_datetime ) && ( $new_date <= $end_datetime ) ) {
			    $timestamp = $new_date;
			} else {
				$timestamp = strtotime( date( 'Y-m-d', strtotime( '+1 day', $datetime ) ) . ' ' . $start_time );
			}
		}

		return $timestamp;
	}

	/**
	 * Generate formatted date.
	 * 
	 * @param string $timestamp
	 * 
	 * @return string 
	 */
	public function literal_date( $timestamp, $new_schedule ) {
		$timestamp = is_numeric( $timestamp ) ? $timestamp : strtotime( $timestamp );
		$weekday   = date( 'l', $timestamp );
		$month     = date( 'M', $timestamp );
		$ord       = 0;
	
		while ( date( 'M', ( $timestamp = strtotime( '-1 week', $timestamp ) ) ) == $month ) { // phpcs:ignore WordPress.CodeAnalysis.AssignmentInCondition.FoundInWhileCondition
			++$ord;
		}
	
		$lit = array( 'first', 'second', 'third', 'fourth', 'fifth' );

		return strtolower( $lit[ $ord ] . ' ' . $weekday . ' of ' . date( 'F Y', $new_schedule ) );
	}
	
	/**
	 * Check if republish is not disabled.
	 *
	 * @param int $post_id The post ID.
	 * 
	 * @return bool
	 */
	private function can_republish( int $post_id ) {
		if ( ! $this->is_enabled( 'enable_single_republishing', true ) ) {
			return false;
		}

		$post_types = $this->get_data( 'post_types_list_single', array( 'post' ) );
		$option = $this->get_meta( $post_id, '_wpar_repost_option' );
		
        if ( in_array( get_post_type( $post_id ), $post_types ) ) {
            return 'disable' !== $option;
		}

		return false;
	}

	/**
	 * Set post republish occurence.
	 *
	 * @param int $post_id The post ID.
	 */
    private function set_occurence( $post_id ) {
        $repeat = $this->get_meta( $post_id, 'wpar_post_republish_count' );
    	if ( ! empty( $repeat ) && is_numeric( $repeat ) ) {
    		++$repeat;
    	} else {
    		$repeat = 1;
    	}
		
		$this->update_meta( $post_id, 'wpar_post_republish_count', $repeat );
	}

	/**
     * Process regenerate schedule
     */
	public function regenerate_schedule() {
		$post_types = $this->get_post_types( true );
		$args = array(
			'post_type'   => $post_types,
			'numberposts' => -1,
			'post_status' => 'publish',
			'fields'      => 'ids',
			'meta_query'  => array(
				'relation' => 'AND',
				array(
				    'key'     => 'wpar_single_republish_status',
    			    'compare' => 'EXISTS',
			    ),
				array(
				    'key'     => '_wpar_filter_republish_datetime',
    			    'compare' => 'EXISTS',
			    ),
			),
		);
		$post_ids = $this->get_posts( $args );
		foreach ( $post_ids as $post_id ) {
			$datetime = $this->get_meta( $post_id, '_wpar_filter_republish_datetime' );
			$utc_datetime = get_gmt_from_date( $datetime, 'U' );
			$timestamp = time();

			if ( $utc_datetime > $timestamp ) {
				$this->set_single_event( $post_id, $datetime, false );
			} else {
				$this->do_action( 'update_schedule', $post_id, false );
			}
		}
	}
}