<?php
/**
 * The file for Linkedin Share.
 *
 * @since      1.2.2
 * @package    RevivePress
 * @subpackage RevivePress\Core\Premium\Social\Providers
 * @author     Sayan Datta <iamsayan@protonmail.com>
 */

namespace RevivePress\Core\Premium\Social\Providers;

use RevivePress\Helpers\Ajax;
use RevivePress\Helpers\Hooker;
use RevivePress\Helpers\Scheduler;
use RevivePress\Helpers\Premium\SocialHelpers;
use RevivePress\Dependencies\SayanDatta\LinkedIn;

defined( 'ABSPATH' ) || exit;

/**
 * Linkedin Share class.
 */
class LinkedinShare
{
	use Ajax;
    use Hooker;
    use Scheduler;
    use SocialHelpers;

	/**
	 * Register functions.
	 */
	public function register() {
		$this->filter( 'wpar/social/add_account_linkedin', 'generate_auth_url', 10, 2 );
		$this->action( 'admin_post_wpar_add_linkedin_account', 'get_auth_data' );
		$this->action( 'wpar/process_linkedin_share', 'process_start', 5, 2 );
        $this->filter( 'wpar/display_linkedin_accounts', 'linkedin_accounts_list' );
	}

	/**
	 * Generate Linkedin Authentication URL.
	 */
	public function generate_auth_url( $url, $request ) {
		$app_id = sanitize_text_field( wp_unslash( $request->get_param( 'app_id' ) ) );
		$app_secret = sanitize_text_field( wp_unslash( $request->get_param( 'app_secret' ) ) );

		try {
			$redirect_url = $this->get_auth_url( 'linkedin' );
			$scopes = 'r_emailaddress r_liteprofile w_member_social';
			$state = base64_encode( trailingslashit( admin_url() ) ); // phpcs:ignore WordPress.PHP.DiscouragedPHPFunctions.obfuscation_base64_encode

			$client = $this->connection( $app_id, $app_secret, $redirect_url, $scopes, false, $state );
			$loginUrl = $client->getAuthUrl();

			// save linkedin credentials
			$this->save_credentials( array(
				'app_id'     => $app_id,
				'app_secret' => $app_secret,
			), 'linkedin' );

			return $loginUrl;
		} catch ( \Exception $e ) {
			return __( 'Error: Wrong App ID or App Secret or Check your Callback URL!', 'wp-auto-republish' );
		}
	}

	/**
	 * Fetch and save Linkedin profile data to db.
	 */
	public function get_auth_data() {
		if ( isset( $_GET['auth_code'] ) ) {
			// get temporary credentials
			$credentials = $this->get_credentials( 'linkedin' );
			$redirect_url = $this->get_auth_url( 'linkedin' );
			
			$client = $this->connection( $credentials['app_id'], $credentials['app_secret'], $redirect_url, null, false, null );
			$accessToken = $client->getAccessToken( $_GET['auth_code'] ); // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.MissingUnslash, WordPress.Security.ValidatedSanitizedInput.InputNotSanitized

			$access_token = $accessToken->access_token;
            $profile = $client->getPerson( $access_token );

			// get current user
			$current_user = wp_get_current_user();

			$args = array(
				'id'           => $profile['id'],
				'name'         => $profile['name'],
				'status'       => $this->do_filter( 'linkedin_default_status', true ),
				'valid'        => true,
				'template'     => '',
				'type'         => 'profile',
				'app_id'       => $credentials['app_id'],
				'app_secret'   => $credentials['app_secret'],
				'access_token' => $access_token,
				'timestamp'    => $this->current_timestamp(),
				'added_by'     => $current_user->user_login,
			);

			$this->update_db( $args, 'linkedin' );

			// set temporary transient for admin notice
		    set_transient( 'rvp_social_account_added', 'linkedin' );

			wp_safe_redirect( $this->get_auth_success_url( 'linkedin' ) );
			exit;
		}
	}

	/**
	 * Process API Interaction with Linkedin and Share Post.
	 * 
	 * @param string   $app_id      Linkedin App ID.
	 * @param string   $app_secret  Linkedin App Secret.
	 * @return object
	 */
	private function connection( $app_id = null, $app_secret = '', $callback = null, $scopes = null, $ssl = true, $state = null ) {
		$connection = new LinkedIn( $app_id, $app_secret, $callback, $scopes, $ssl, $state );

		return $connection;
	}

    /**
	 * Get the list of all linkedin accounts.
	 * 
	 * @return string
	 */
	public function linkedin_accounts_list() {
		$accounts = get_option( 'wpar_linkedin_accounts_db' );
		if ( $accounts === false ) {
            return false;
		}

		$accounts = maybe_unserialize( $accounts );
		if ( empty( $accounts ) ) {
            return false;
		}
		
		$html = '<table class="form-table">
			<thead>
				<tr valign="top">
				    <td class="social-td"><strong>' . __( 'Timestamp', 'wp-auto-republish' ) . '</strong></td>
				    <td class="social-td" scope="row"><strong>' . __( 'Account Name', 'wp-auto-republish' ) . '</strong></td>
				    <td class="social-td" scope="row"><strong>' . __( 'Added By', 'wp-auto-republish' ) . '</strong></td>
					<td class="social-td" scope="row"><strong>' . __( 'Status', 'wp-auto-republish' ) . '</strong></td>
					<td class="social-td" scope="row"><strong>' . __( 'Template', 'wp-auto-republish' ) . '</strong></td>
				    <td class="social-td" scope="row"><strong>' . __( 'Action', 'wp-auto-republish' ) . '</strong></td>
				</tr>
			</thead>
			<tbody>';

		foreach ( $accounts as $items => $item ) {
			$enabled = ( $item['status'] ) ? __( 'Enabled', 'wp-auto-republish' ) : __( 'Disabled', 'wp-auto-republish' );
			$css_class = ( $item['status'] ) ? 'enabled' : 'disabled';

			$delete = '<a href="#" class="wpar-delete-social-single" data-account-type="linkedin" data-account-id="' . $item['id'] . '" data-processing="' . __( 'Deleting...', 'wp-auto-republish' ) . '" data-notice="' . sprintf( 
				/* translators: Twitter account name. */
				__( 'It will delete the Linkedin Account: %s from this list. Do you want to continue?', 'wp-auto-republish' ), $item['name'] 
			) . '">' . __( 'Delete', 'wp-auto-republish' ) . '</a>';

			$activate = '<a href="#" class="wpar-social-single-update" data-account-type="linkedin" data-action="activate" data-account-id="' . $item['id'] . '" data-processing="' . __( 'Activating...', 'wp-auto-republish' ) . '" data-next="' . __( 'Enabled', 'wp-auto-republish' ) . '">' . __( 'Activate', 'wp-auto-republish' ) . '</a>';
			$deactivate = '<a href="#" class="wpar-social-single-update" data-account-type="linkedin" data-action="deactivate" data-account-id="' . $item['id'] . '" data-processing="' . __( 'Deactivating...', 'wp-auto-republish' ) . '" data-next="' . __( 'Disabled', 'wp-auto-republish' ) . '">' . __( 'Deactivate', 'wp-auto-republish' ) . '</a>';

			$template = '<a href="#" class="wpar-template-social-single" data-account-type="linkedin" data-account-id="' . $item['id'] . '">' . __( 'View / Edit Template', 'wp-auto-republish' ) . '</a>';

			$button = ( $item['status'] ) ? $deactivate : $activate;
			$button = ( $item['valid'] ) ? $button : '<span class="wpar-social-account-status expired">' . __( 'Token Expired', 'wp-auto-republish' ) . '</span>';

			$html .= '<tr valign="top" id="wpar-linkedin-' . $item['id'] . '" class="wpar-social-accounts">
			    <td class="social-td" scope="row" data-label="' . __( 'Timestamp', 'wp-auto-republish' ) . '">' . date_i18n( 'M j, Y @ ' . get_option( 'time_format' ), $item['timestamp'] ) . '</td>
			    <td class="social-td wpar-linkedin-account-' . $item['id'] . '" scope="row" data-label="' . __( 'Account Name', 'wp-auto-republish' ) . '">' . $item['name'] . '</td>
			    <td class="social-td" scope="row" data-label="' . __( 'Added By', 'wp-auto-republish' ) . '">' . ucwords( $item['added_by'] ) . '</a></td>
				<td class="social-td wpar-linkedin-status-' . $item['id'] . '" scope="row" data-label="' . __( 'Status', 'wp-auto-republish' ) . '"><span class="wpar-social-account-status ' . $css_class . '">' . $enabled . '</span></td>
			    <td class="social-td" scope="row" data-label="' . __( 'Template', 'wp-auto-republish' ) . '">' . $template . '</td>
				<td class="social-td" scope="row" data-label="' . __( 'Action', 'wp-auto-republish' ) . '"><span class="wpar-linkedin-action-btn-' . $item['id'] . '">' . $button . '</span> | ' . $delete . '</td>
		    </tr>';
	    }

		$html .= '</tbody>
		</table>';
		
		return $html;
	}

    /**
	 * Get Auth Tokens & Secret of all activated accounts.
	 * 
	 * @return bool|array
	 */
	private function get_activated_accounts() {
		$accounts = get_option( 'wpar_linkedin_accounts_db' );
		if ( $accounts === false ) {
            return false;
		}

		$accounts = maybe_unserialize( $accounts );
		if ( empty( $accounts ) ) {
            return false;
		}

		$credentials = array();
		foreach ( $accounts as $items => $item ) {
			if ( $item['status'] ) {
				$credentials[ $item['id'] ] = array(
					'app_id'       => $item['app_id'],
				    'app_secret'   => $item['app_secret'],
					'access_token' => $item['access_token'],
					'template'     => $item['template'],
				);
			}
		}

		if ( empty( $credentials ) ) {
            $credentials = false;
		}

		return $credentials;
	}

	/**
	 * Start Linkedin share process.
	 * 
	 * @param int  $post_id  WP Post ID.
	 * @param bool $instant  Check if instant.
	 */
	public function process_start( $post_id, $instant = false ) {
		$enable = $this->is_enabled( 'linkedin_enable' );
		$accounts = $this->get_activated_accounts();
		if ( $enable && $accounts !== false ) {
		    foreach ( $accounts as $account_id => $data ) {
		    	$this->process_share( $data, $account_id, $post_id, $instant );
		    }
		}
	}

	/**
	 * Process API Interaction with Linkedin and Share Post.
	 * 
	 * @param array  $account      Linkedin Data.
	 * @param int    $account_id   Linkedin Account ID.
	 * @param int    $post_id      WP Post ID.
	 * @param bool   $instant      Check if instant.
	 */
	private function process_share( $account, $account_id, $post_id, $instant ) {
		$post = \get_post( $post_id );
		$post_types = $this->get_data( 'linkedin_post_types_display', array( 'post' ) );
		$taxonomies = $this->get_data( 'linkedin_social_taxonomy' );
		$post_as = $this->get_data( 'linkedin_post_as', 'link_status' );
		$template = $this->get_data( 'linkedin_template' );
		$content_source = $this->get_data( 'linkedin_content_source' );
		$disable_linkedin_share = $this->get_meta( $post->ID, '_wpar_disable_linkedin_share' );

		if ( ( ! in_array( $post->post_type, $post_types ) || $disable_linkedin_share == 'yes' ) && ! $instant ) {
			return;
		}

		$account_template = $account['template'];
		if ( ! empty( $account_template ) ) {
			$template = $account_template;
		}

		try {
			$client = $this->connection();
			$access_token = $account['access_token'];
			$type = 'person';

			$permalink = $this->get_permalink( $post_id, 'linkedin' );
			$formated_text = $this->get_social_template( $template, $post->ID, $this->get_hashtags( $taxonomies, $post->ID ), $content_source, 1300, $permalink );
			
			if ( $post_as === 'status' ) {
				$result = $client->linkedInTextPost( $access_token, $type, $account_id, $formated_text );
            } else {
                $post_details = \get_post( $post_id );
                if ( $content_source == 'excerpt' && has_excerpt( $post_details->ID ) ) {
                    $desc = wp_strip_all_tags( strip_shortcodes( $post_details->post_excerpt ) );
                } else {
                    $desc = wp_strip_all_tags( strip_shortcodes( $post_details->post_content ) );
                }

				$title = get_the_title( $post_details->ID );
				$image = '';

				if ( has_post_thumbnail( $post_details->ID ) && $post_as === 'link_status' ) {
					$image_path = wp_get_original_image_path( get_post_thumbnail_id( $post_details->ID ) );
					
					$results = $client->uploadImage( $access_token, $type, $account_id, $image_path );
                	$image = isset( $results['value']['image'] ) ? $results['value']['image'] : '';
				}

				if ( $post_as === 'link_status' ) {
                	$result = $client->linkedInPhotoPost( $access_token, $type, $account_id, $image, $this->filter_little_text( $title ), $formated_text );
				} elseif ( $post_as === 'link' ) {
					$result = $client->linkedInLinkPost( $access_token, $type, $account_id, $formated_text, $permalink, $image, $this->filter_little_text( $title ), $this->filter_little_text( $desc ) );
				}
            }

			$result = json_decode( $result, true );

			if ( isset( $result['id'] ) ) {
			    $this->set_post_metadata( $post_id, array(
			    	'share_id'     => $result['id'] ?? '',
			    	'publish_date' => $this->current_timestamp(),
			    	'account_id'   => $account_id,
			    ), 'linkedin' );

				// insert log entry
			    $this->do_action( 'insert_log', $post_id, 'linkedin' );
			} elseif ( isset( $result['serviceErrorCode'] ) && $result['serviceErrorCode'] != '' ) {
				if ( $result['serviceErrorCode'] == 401 ) {
					$this->expire_token( $account_id, 'linkedin' );
				}
				// insert log entry
			    $this->do_action( 'insert_log', $post_id, array(
					'status' => false,
					'info'   => sprintf( 
						/* translators: Linkedin Api error code with message */
						__( 'Error: %s', 'wp-auto-republish' ), $result['serviceErrorCode'] . ' - ' . $result['message'] 
					),
					'action' => 'linkedin',
				) );
			} elseif ( isset( $result['code'], $result['message'] ) && $result['code'] === 'INVALID_STRING_FORMAT' ) {
				// insert log entry
			    $this->do_action( 'insert_log', $post_id, array(
					'status' => false,
					'info'   => sprintf( 
						/* translators: Linkedin Api error code with message */
						__( 'Error: %s', 'wp-auto-republish' ), $result['code'] . ' - ' . $result['message'] 
					),
					'action' => 'linkedin',
				) );
			}
		} catch ( \Exception $e ) {
			// insert log entry
			$this->do_action( 'insert_log', $post_id, array(
				'status' => false,
				'info'   => $e->getMessage(),
				'action' => 'linkedin',
			) );
		}
	}
}