<?php
/**
 * The file for Pinterest Share.
 *
 * @since      1.4.6
 * @package    RevivePress
 * @subpackage RevivePress\Core\Premium\Social\Providers
 * @author     Sayan Datta <iamsayan@protonmail.com>
 */

namespace RevivePress\Core\Premium\Social\Providers;

use Exception;
use RevivePress\Helpers\Ajax;
use RevivePress\Helpers\Hooker;
use RevivePress\Helpers\Scheduler;
use RevivePress\Helpers\Premium\SocialHelpers;
use RevivePress\Dependencies\DirkGroenen\Pinterest\Pinterest;

defined( 'ABSPATH' ) || exit;

/**
 * Pinterest Share class.
 */
class PinterestShare
{
	use Ajax;
    use Hooker;
    use Scheduler;
    use SocialHelpers;

	/**
	 * Register functions.
	 */
	public function register() {
		$this->filter( 'wpar/social/add_account_pinterest', 'generate_auth_url', 10, 2 );
		$this->action( 'admin_post_wpar_add_pinterest_account', 'get_auth_data' );
		$this->action( 'wpar/process_pinterest_share', 'process_start', 5, 2 );
        $this->filter( 'wpar/display_pinterest_accounts', 'pinterest_accounts_list' );
	}

	/**
	 * Generate Pinterest Authentication URL.
	 */
	public function generate_auth_url( $url, $request ) {
		$app_id = sanitize_text_field( wp_unslash( $request->get_param( 'app_id' ) ) );
		$app_secret = sanitize_text_field( wp_unslash( $request->get_param( 'app_secret' ) ) );

		// @codingStandardsIgnoreStart
		$callback_url = $this->get_auth_url( 'pinterest' );

		try {
			$connection = $this->connection( $app_id, $app_secret );
			$connection->auth->setState( base64_encode( trailingslashit( admin_url() ) ) );

            $loginurl = $connection->auth->getLoginUrl( $callback_url, [ 'boards:read', 'boards:write', 'pins:read', 'pins:write', 'user_accounts:read' ] );

			// save pinterest credentials
			$this->save_credentials( [
				'app_id'     => $app_id,
				'app_secret' => $app_secret,
			], 'pinterest' );
				
			return $loginurl;
		} catch ( \Exception $e ) {
			$this->error( __( 'Error: Wrong App ID or App Secret or Check your Callback URL!', 'wp-auto-republish' ) );
		}
		// @codingStandardsIgnoreEnd
	}

	/**
	 * Fetch and save Pinterest profile data to db.
	 */
	public function get_auth_data() {
		// @codingStandardsIgnoreStart
		if ( ! empty( $_GET['auth_code'] ) ) {
			// get temporary credentials
			$credentials = $this->get_credentials( 'pinterest' );
			$callback_url = $this->get_auth_url( 'pinterest' );

		    // generate access token to fetch profile data
			$connection = $this->connection( $credentials['app_id'], $credentials['app_secret'] );
			
			$token = $connection->auth->getOAuthToken( $_GET['auth_code'], $callback_url );
			$access_token  = $token->access_token;
			$refresh_token = $token->refresh_token;
			$expires_in    = time() + $token->expires_in;
			$rt_expires_in = time() + $token->refresh_token_expires_in;

			$connection->auth->setOAuthToken( $access_token );
			$userinfo = $connection->users->me();

			// get all board list
			$boards_arr = [];
			$page_size  = 1;
			$_boards_arr = [ 'page' => null ];
			do {
				$boards      = $connection->users->getMeBoards( [
					'page_size' => $page_size,
					'bookmark'  => $_boards_arr['page'],
				] );
				$_boards_arr = $boards->toArray();
				$boards_arr  = array_merge( $boards_arr, $_boards_arr['data'] );
			}
			while( ! empty( $_boards_arr['page'] ) );

			foreach( $boards_arr as $key => $board ) {
				$sections = $connection->sections->get( $board['id'], [
					'page_size' => 100,
				] );
				$sections = $sections->toArray();
				$boards_arr[ $board['id'] ] = $board;
				$boards_arr[ $board['id'] ]['sections'] = $sections['data'];
				unset( $boards_arr[ $key ] ); 
			}

			// get current user
			$current_user = wp_get_current_user();

			$args = [
				'id'            	=> $userinfo->username,
				'user_name'     	=> $userinfo->username,
				'website_url'   	=> $userinfo->website_url,
				'account_type'  	=> $userinfo->account_type,
				'thumbnail_url' 	=> $userinfo->profile_image,
				'status'        	=> $this->do_filter( 'pinterest_default_status', true ),
				'valid'         	=> true,
				'template'      	=> '',
				'default_board' 	=> '',
				'default_section' 	=> '',
				'boards' 			=> $boards_arr,
				'callback_url'  	=> $callback_url,
				'access_token'  	=> $access_token,
				'refresh_token' 	=> $refresh_token,
				'expires_in'    	=> $expires_in,
				'rt_expires_in'		=> $rt_expires_in,
				'app_id'          	=> $credentials['app_id'],
				'app_secret'      	=> $credentials['app_secret'],
				'timestamp'       	=> $this->current_timestamp(),
				'added_by'        	=> $current_user->user_login,
			];

			$this->update_db( $args, 'pinterest' );

			// // set temporary transient for admin notice
		    set_transient( 'rvp_social_account_added', 'pinterest' );

			wp_safe_redirect( $this->get_auth_success_url( 'pinterest' ) );
			exit;
		}
		// @codingStandardsIgnoreEnd
	}

	/**
	 * Process API Interaction with Pinterest and Share Post.
	 * 
	 * @param string      $app_id               Pinterest App ID.
	 * @param string      $app_secret           Pinterest App Secret.
	 * @return object
	 */
	private function connection( $app_id = null, $app_secret = null ) {
		$connection = new Pinterest( $app_id, $app_secret );

		return $connection;
	}

    /**
	 * Get the list of all pinterest accounts.
	 * 
	 * @return string
	 */
	public function pinterest_accounts_list() {
		$accounts = get_option( 'wpar_pinterest_accounts_db' );
		if ( $accounts === false ) {
            return false;
		}

		$accounts = maybe_unserialize( $accounts );
		if ( empty( $accounts ) ) {
            return false;
		}
		
		$html = '<table class="form-table">
			<thead>
				<tr valign="top">
				    <td class="social-td"><strong>' . __( 'Timestamp', 'wp-auto-republish' ) . '</strong></td>
				    <td class="social-td" scope="row"><strong>' . __( 'Handle', 'wp-auto-republish' ) . '</strong></td>
				    <td class="social-td" scope="row"><strong>' . __( 'Active Board', 'wp-auto-republish' ) . '</strong></td>
				    <td class="social-td" scope="row"><strong>' . __( 'Active Section', 'wp-auto-republish' ) . '</strong></td>
				    <td class="social-td" scope="row"><strong>' . __( 'Added By', 'wp-auto-republish' ) . '</strong></td>
					<td class="social-td" scope="row"><strong>' . __( 'Status', 'wp-auto-republish' ) . '</strong></td>
					<td class="social-td" scope="row"><strong>' . __( 'Template', 'wp-auto-republish' ) . '</strong></td>
				    <td class="social-td" scope="row"><strong>' . __( 'Action', 'wp-auto-republish' ) . '</strong></td>
				</tr>
			</thead>
			<tbody>';

		foreach ( $accounts as $items => $item ) {
			$board_list = '<select class="wpar-pinterest-board" data-account-id="' . $item['id'] . '" data-type="board" style="width: 100%;">';
			$board_list .= '<option value="">--</option>';
			foreach ( $item['boards'] as $board ) {
				$board_list .= '<option value="' . $board['id'] . '" ' . selected( $board['id'], $item['default_board'], false ) . '>' . $board['name'] . '</option>';
			}
			$board_list .= '</select>';

			$section_list = '<select class="wpar-pinterest-board-section" data-account-id="' . $item['id'] . '" data-type="section" style="width: 100%;">';
			$section_list .= '<option value="">--</option>';
			if ( ! empty( $item['default_board'] ) ) {
				foreach ( $item['boards'][ $item['default_board'] ]['sections'] as $section ) {
					$section_list .= '<option value="' . $section['id'] . '" ' . selected( $section['id'], $item['default_section'], false ) . '>' . $section['name'] . '</option>';
				}
			}
			$section_list .= '</select>';

			$enabled = ( $item['status'] ) ? __( 'Enabled', 'wp-auto-republish' ) : __( 'Disabled', 'wp-auto-republish' );
			$css_class = ( $item['status'] ) ? 'enabled' : 'disabled';

			$delete = '<a href="#" class="wpar-delete-social-single" data-account-type="pinterest" data-account-id="' . $item['id'] . '" data-processing="' . __( 'Deleting...', 'wp-auto-republish' ) . '" data-notice="' . sprintf( 
				/* translators: Pinterest account user name. */
				__( 'It will delete the Pinterest Account: %s from this list. Do you want to continue?', 'wp-auto-republish' ), '@' . $item['user_name'] 
			) . '">' . __( 'Delete', 'wp-auto-republish' ) . '</a>';

			$activate = '<a href="#" class="wpar-social-single-update" data-account-type="pinterest" data-action="activate" data-account-id="' . $item['id'] . '" data-processing="' . __( 'Activating...', 'wp-auto-republish' ) . '" data-next="' . __( 'Enabled', 'wp-auto-republish' ) . '">' . __( 'Activate', 'wp-auto-republish' ) . '</a>';
			$deactivate = '<a href="#" class="wpar-social-single-update" data-account-type="pinterest" data-action="deactivate" data-account-id="' . $item['id'] . '" data-processing="' . __( 'Deactivating...', 'wp-auto-republish' ) . '" data-next="' . __( 'Disabled', 'wp-auto-republish' ) . '">' . __( 'Deactivate', 'wp-auto-republish' ) . '</a>';

			$template = '<a href="#" class="wpar-template-social-single" data-account-type="pinterest" data-account-id="' . $item['id'] . '">' . __( 'View / Edit Template', 'wp-auto-republish' ) . '</a>';

			$button = ( $item['status'] ) ? $deactivate : $activate;
			$button = ( $item['valid'] ) ? $button : '<span class="wpar-social-account-status expired">' . __( 'Token Expired', 'wp-auto-republish' ) . '</span>';
			$button = '<span class="wpar-pinterest-action-btn-' . $item['id'] . '">' . $button . '</span> | ' . $delete;

			$button = ( ! empty( $item['default_board'] ) ) ? $button : $delete;

			$html .= '<tr valign="top" id="wpar-pinterest-' . $item['id'] . '" class="wpar-social-accounts">
			    <td class="social-td" scope="row" data-label="' . __( 'Timestamp', 'wp-auto-republish' ) . '">' . date_i18n( 'M j, Y @ ' . get_option( 'time_format' ), $item['timestamp'] ) . '</td>
			    <td class="social-td wpar-pinterest-account-' . $item['id'] . '" scope="row" data-label="' . __( 'Handle', 'wp-auto-republish' ) . '"><a href="https://pinterest.com/' . $item['user_name'] . '" target="_blank">@' . $item['user_name'] . '</a></td>
			    <td class="social-td" scope="row" data-label="' . __( 'Active Board', 'wp-auto-republish' ) . '">' . $board_list . '</td>
			    <td class="social-td wpar-pinterest-section-' . $item['id'] . '" scope="row" data-label="' . __( 'Active Section', 'wp-auto-republish' ) . '">' . $section_list . '</td>
				<td class="social-td" scope="row" data-label="' . __( 'Added By', 'wp-auto-republish' ) . '">' . ucwords( $item['added_by'] ) . '</a></td>
				<td class="social-td wpar-pinterest-status-' . $item['id'] . '" scope="row" data-label="' . __( 'Status', 'wp-auto-republish' ) . '"><span class="wpar-social-account-status ' . $css_class . '">' . $enabled . '</span></td>
			    <td class="social-td" scope="row" data-label="' . __( 'Template', 'wp-auto-republish' ) . '">' . $template . '</td>
				<td class="social-td" scope="row" data-label="' . __( 'Action', 'wp-auto-republish' ) . '">' . $button . '</td>
		    </tr>';
	    }

		$html .= '</tbody>
		</table>';
		
		return $html;
	}

    /**
	 * Get Auth Tokens & Secret of all activated accounts.
	 * 
	 * @return bool|array
	 */
	private function get_activated_accounts() {
		$accounts = get_option( 'wpar_pinterest_accounts_db' );
		if ( $accounts === false ) {
            return false;
		}

		$accounts = maybe_unserialize( $accounts );
		if ( empty( $accounts ) ) {
            return false;
		}

		$credentials = array();
		foreach ( $accounts as $items => $item ) {
			if ( $item['status'] ) {
				$credentials[ $item['id'] ] = array(
					'app_id'          => $item['app_id'],
				    'app_secret'      => $item['app_secret'],
					'access_token'    => $item['access_token'],
					'default_board'   => $item['default_board'],
					'default_section' => $item['default_section'],
					'template'        => $item['template'],
				);
			}
		}

		if ( empty( $credentials ) ) {
            $credentials = false;
		}

		return $credentials;
	}

	/**
	 * Start Pinterest share process.
	 * 
	 * @param int  $post_id  WP Post ID.
	 * @param bool $instant  Check if instant.
	 */
	public function process_start( $post_id, $instant = false ) {
		$enable = $this->is_enabled( 'pinterest_enable' );
		$accounts = $this->get_activated_accounts();
		if ( $enable && $accounts !== false ) {
		    foreach ( $accounts as $account_id => $data ) {
		    	$this->process_share( $data, $account_id, $post_id, $instant );
		    }
		}
	}

	/**
	 * Process API Interaction with Pinterest and Share Post.
	 * 
	 * @param array  $account     Pinterest Auth Data.
	 * @param int    $account_id  Pinterest Account ID.
	 * @param int    $post_id     WP Post ID.
	 * @param bool   $instant     Check if instant.
	 */
	private function process_share( $account, $account_id, $post_id, $instant ) {
		$post = \get_post( $post_id );
		$post_types = $this->get_data( 'pinterest_post_types_display', array( 'post' ) );
		$taxonomies = $this->get_data( 'pinterest_social_taxonomy' );
		$template = $this->get_data( 'pinterest_template' );
		$content_source = $this->get_data( 'pinterest_content_source' );
		$disable_pinterest_share = $this->get_meta( $post->ID, '_wpar_disable_pinterest_share' );

		if ( ( ! in_array( $post->post_type, $post_types ) || $disable_pinterest_share == 'yes' ) && ! $instant ) {
			return;
		}

		if ( $this->is_localhost() || ! has_post_thumbnail( $post->ID ) ) {
			// insert log entry
			$this->do_action( 'insert_log', $post_id, array(
				'status' => false,
				'info'   => $this->is_localhost() ? __( 'Localhost not allowed!', 'wp-auto-republish' ) : __( 'Post Thumbnail missing!', 'wp-auto-republish' ),
				'action' => 'pinterest',
			) );
			return;
		}

		$account_template = $account['template'];
		if ( ! empty( $account_template ) ) {
			$template = $account_template;
		}

		$permalink = $this->get_permalink( $post->ID, 'pinterest' );
		$pin_args = array(
			'title'        => wp_strip_all_tags( $post->post_title ),
            'description'  => $this->get_social_template( $template, $post->ID, $this->get_hashtags( $taxonomies, $post->ID ), $content_source, 500, $permalink ),
			'link'         => $permalink,
			'media_source' => array(
				'source_type' => 'image_url',
				'url'         => get_the_post_thumbnail_url( $post->ID, 'full' ),
			),
			'board_id'     => ( string ) $account['default_board'],
		);

		if ( ! empty( $account['default_section'] ) ) {
			$pin_args['board_section_id'] = ( string ) $account['default_section'];
		}

		try {
		    // generate access token to fetch profile data
			$connection = $this->connection();
			$connection->auth->setOAuthToken( $account['access_token'] );
			$results = $connection->pins->create( $pin_args );

			if ( $results != '' ) {
				$this->set_post_metadata( $post_id, array(
					'share_id'     => $results->id,
					'publish_date' => $this->current_timestamp(),
					'account_id'   => $account_id,
				), 'pinterest' );

				// insert log entry
				$this->do_action( 'insert_log', $post_id, 'pinterest' );
		    } else {
				// insert log entry
				$this->do_action( 'insert_log', $post_id, array(
					'status' => false,
					'action' => 'pinterest',
				) );
            }
		} catch ( Exception $e ) {
			// insert log entry
			$this->do_action( 'insert_log', $post_id, array(
				'status' => false,
				'info'   => $e->getMessage(),
				'action' => 'pinterest',
			) );
		}
	}
}