<?php
/**
 * The file for Tumblr Share.
 *
 * @since      1.4.0
 * @package    RevivePress
 * @subpackage RevivePress\Core\Premium\Social\Providers
 * @author     Sayan Datta <iamsayan@protonmail.com>
 */

namespace RevivePress\Core\Premium\Social\Providers;

use Exception;
use RevivePress\Helpers\Ajax;
use RevivePress\Helpers\Hooker;
use RevivePress\Helpers\Scheduler;
use RevivePress\Helpers\Premium\SocialHelpers;
use RevivePress\Dependencies\Tumblr\API\Client as TumblrClient;

defined( 'ABSPATH' ) || exit;

/**
 * Tumblr Share class.
 */
class TumblrShare
{
	use Ajax;
    use Hooker;
    use Scheduler;
    use SocialHelpers;

	/**
	 * Register functions.
	 */
	public function register() {
		$this->filter( 'wpar/social/add_account_tumblr', 'generate_auth_url', 10, 2 );
		$this->action( 'admin_post_wpar_add_tumblr_account', 'get_auth_data' );
		$this->action( 'wpar/process_tumblr_share', 'process_start', 5, 2 );
        $this->filter( 'wpar/display_tumblr_accounts', 'tumblr_accounts_list' );
	}

	/**
	 * Generate Tumblr Authentication URL.
	 */
	public function generate_auth_url( $url, $request ) {
		$app_id = sanitize_text_field( wp_unslash( $request->get_param( 'app_id' ) ) );
		$app_secret = sanitize_text_field( wp_unslash( $request->get_param( 'app_secret' ) ) );

		// @codingStandardsIgnoreStart
		$callback_url = add_query_arg( 'state', base64_encode( trailingslashit( admin_url() ) ), $this->get_auth_url( 'tumblr' ) );
		
		try {
			$connection = $this->connection( $app_id, $app_secret );
			$request_handler = $connection->getRequestHandler();
			$request_handler->setBaseUrl( 'https://www.tumblr.com/' );
			$response = $request_handler->request( 'POST', 'oauth/request_token', [ 'oauth_callback' => esc_url( $callback_url ) ] );
			$out = $response->body;
			$data = [];
			parse_str( $out, $data );

			if ( isset( $data['oauth_token'], $data['oauth_token_secret'] ) ) {
				// save tumblr credentials
				$this->save_credentials( [
					'app_id'     => $app_id,
					'app_secret' => $app_secret,
				], 'tumblr' );

				set_transient( 'tumblr_temp_auth_token', $data['oauth_token_secret'] );
				
				$url = add_query_arg( [
					'oauth_token' => $data['oauth_token'],
				], 'https://www.tumblr.com/oauth/authorize' );

				return $url;
			} else {
				return __( 'Error: Auth Token or Auth Token Secret has error!', 'wp-auto-republish' );
			}
		} catch ( \Exception $e ) {
			return __( 'Error: Wrong App ID or App Secret or Check your Callback URL!', 'wp-auto-republish' );
		}
		// @codingStandardsIgnoreEnd
	}

	/**
	 * Fetch and save Tumblr profile data to db.
	 */
	public function get_auth_data() {
		// @codingStandardsIgnoreStart
		if ( isset( $_GET['oauth_token'], $_GET['oauth_verifier'] ) ) {
			// get temporary credentials
			$credentials = $this->get_credentials( 'tumblr' );

		    // generate access token to fetch profile data
			$connection = $this->connection( $credentials['app_id'], $credentials['app_secret'] );
			$connection->setToken( $_GET['oauth_token'], get_transient( 'tumblr_temp_auth_token' ) );
			
			$request_handler = $connection->getRequestHandler();
			$request_handler->setBaseUrl( 'https://www.tumblr.com/' );
			$response = $request_handler->request( 'GET', 'oauth/access_token', [ 'oauth_verifier' => $_GET['oauth_verifier'] ] );

			$access_token = [];
			parse_str( $response->body, $access_token );

			// get current user
			$current_user = wp_get_current_user();

			// get user data using access tokens
			$connection = $this->connection( $credentials['app_id'], $credentials['app_secret'], $access_token['oauth_token'], $access_token['oauth_token_secret'] );
			$content = $connection->getUserInfo();

			foreach ( $content->user->blogs as $blog ) {
				$args = [
					'id'                 => str_replace( 't:', '', $blog->uuid ),
					'name'               => $blog->title,
					'user_name'          => $blog->name,
					'status'             => $this->do_filter( 'tumblr_default_status', true ),
					'valid'              => true,
					'template'           => '',
					'app_id'             => $credentials['app_id'],
					'app_secret'         => $credentials['app_secret'],
					'oauth_token'        => $access_token['oauth_token'],
					'oauth_token_secret' => $access_token['oauth_token_secret'],
					'timestamp'          => $this->current_timestamp(),
					'added_by'           => $current_user->user_login,
				];

				$this->update_db( $args, 'tumblr' );
			}

			// set temporary transient for admin notice
		    set_transient( 'rvp_social_account_added', 'tumblr' );

			wp_safe_redirect( $this->get_auth_success_url( 'tumblr' ) );
			exit;
		}
		// @codingStandardsIgnoreEnd
	}

	/**
	 * Process API Interaction with Tumblr and Share Post.
	 * 
	 * @param string      $app_id               Tumblr App ID.
	 * @param string      $app_secret           Tumblr App Secret.
	 * @param string|null $oauth_token Tumblr  Tumblr Auth Token.
	 * @param string      $oauth_token_secret   Tumblr Auth Token Secret.
	 * @return object
	 */
	private function connection( $app_id, $app_secret, $oauth_token = null, $oauth_token_secret = null ) {
		$connection = new TumblrClient( $app_id, $app_secret, $oauth_token, $oauth_token_secret );

		return $connection;
	}

    /**
	 * Get the list of all tumblr accounts.
	 * 
	 * @return string
	 */
	public function tumblr_accounts_list() {
		$accounts = get_option( 'wpar_tumblr_accounts_db' );
		if ( $accounts === false ) {
            return false;
		}

		$accounts = maybe_unserialize( $accounts );
		if ( empty( $accounts ) ) {
            return false;
		}
		
		$html = '<table class="form-table">
			<thead>
				<tr valign="top">
				    <td class="social-td"><strong>' . __( 'Timestamp', 'wp-auto-republish' ) . '</strong></td>
				    <td class="social-td" scope="row"><strong>' . __( 'Name', 'wp-auto-republish' ) . '</strong></td>
				    <td class="social-td" scope="row"><strong>' . __( 'Handle', 'wp-auto-republish' ) . '</strong></td>
				    <td class="social-td" scope="row"><strong>' . __( 'Added By', 'wp-auto-republish' ) . '</strong></td>
					<td class="social-td" scope="row"><strong>' . __( 'Status', 'wp-auto-republish' ) . '</strong></td>
					<td class="social-td" scope="row"><strong>' . __( 'Template', 'wp-auto-republish' ) . '</strong></td>
				    <td class="social-td" scope="row"><strong>' . __( 'Action', 'wp-auto-republish' ) . '</strong></td>
				</tr>
			</thead>
			<tbody>';

		foreach ( $accounts as $items => $item ) {
			$enabled = ( $item['status'] ) ? __( 'Enabled', 'wp-auto-republish' ) : __( 'Disabled', 'wp-auto-republish' );
			$css_class = ( $item['status'] ) ? 'enabled' : 'disabled';

			$delete = '<a href="#" class="wpar-delete-social-single" data-account-type="tumblr" data-account-id="' . $item['id'] . '" data-processing="' . __( 'Deleting...', 'wp-auto-republish' ) . '" data-notice="' . sprintf( 
				/* translators: Tumblr account name with user name. */
				__( 'It will delete the Tumblr Account: %s from this list. Do you want to continue?', 'wp-auto-republish' ), $item['name'] . ' (@' . $item['user_name'] . ')' 
			) . '">' . __( 'Delete', 'wp-auto-republish' ) . '</a>';

			$activate = '<a href="#" class="wpar-social-single-update" data-account-type="tumblr" data-action="activate" data-account-id="' . $item['id'] . '" data-processing="' . __( 'Activating...', 'wp-auto-republish' ) . '" data-next="' . __( 'Enabled', 'wp-auto-republish' ) . '">' . __( 'Activate', 'wp-auto-republish' ) . '</a>';
			$deactivate = '<a href="#" class="wpar-social-single-update" data-account-type="tumblr" data-action="deactivate" data-account-id="' . $item['id'] . '" data-processing="' . __( 'Deactivating...', 'wp-auto-republish' ) . '" data-next="' . __( 'Disabled', 'wp-auto-republish' ) . '">' . __( 'Deactivate', 'wp-auto-republish' ) . '</a>';

			$template = '<a href="#" class="wpar-template-social-single" data-account-type="tumblr" data-account-id="' . $item['id'] . '">' . __( 'View / Edit Template', 'wp-auto-republish' ) . '</a>';

			$button = ( $item['status'] ) ? $deactivate : $activate;
			$button = ( $item['valid'] ) ? $button : '<span class="wpar-social-account-status expired">' . __( 'Token Expired', 'wp-auto-republish' ) . '</span>';

			$html .= '<tr valign="top" id="wpar-tumblr-' . $item['id'] . '" class="wpar-social-accounts">
			    <td class="social-td" scope="row" data-label="' . __( 'Timestamp', 'wp-auto-republish' ) . '">' . date_i18n( 'M j, Y @ ' . get_option( 'time_format' ), $item['timestamp'] ) . '</td>
			    <td class="social-td wpar-tumblr-account-' . $item['id'] . '" scope="row" data-label="' . __( 'Name', 'wp-auto-republish' ) . '">' . $item['name'] . '</td>
			    <td class="social-td" scope="row" data-label="' . __( 'Handle', 'wp-auto-republish' ) . '"><a href="https://' . $item['user_name'] . '.tumblr.com" target="_blank">@' . $item['user_name'] . '</a></td>
				<td class="social-td" scope="row" data-label="' . __( 'Added By', 'wp-auto-republish' ) . '">' . ucwords( $item['added_by'] ) . '</a></td>
				<td class="social-td wpar-tumblr-status-' . $item['id'] . '" scope="row" data-label="' . __( 'Status', 'wp-auto-republish' ) . '"><span class="wpar-social-account-status ' . $css_class . '">' . $enabled . '</span></td>
			    <td class="social-td" scope="row" data-label="' . __( 'Template', 'wp-auto-republish' ) . '">' . $template . '</td>
				<td class="social-td" scope="row" data-label="' . __( 'Action', 'wp-auto-republish' ) . '"><span class="wpar-tumblr-action-btn-' . $item['id'] . '">' . $button . '</span> | ' . $delete . '</td>
		    </tr>';
	    }

		$html .= '</tbody>
		</table>';
		
		return $html;
	}

    /**
	 * Get Auth Tokens & Secret of all activated accounts.
	 * 
	 * @return bool|array
	 */
	private function get_activated_accounts() {
		$accounts = get_option( 'wpar_tumblr_accounts_db' );
		if ( $accounts === false ) {
            return false;
		}

		$accounts = maybe_unserialize( $accounts );
		if ( empty( $accounts ) ) {
            return false;
		}

		$credentials = array();
		foreach ( $accounts as $items => $item ) {
			if ( $item['status'] ) {
				$credentials[ $item['id'] ] = array(
					'app_id'            => $item['app_id'],
				    'app_secret'        => $item['app_secret'],
					'auth_token'        => $item['oauth_token'], 
					'auth_token_secret' => $item['oauth_token_secret'],
					'template'          => $item['template'],
					'blog_name'         => $item['user_name'],
				);
			}
		}

		if ( empty( $credentials ) ) {
            $credentials = false;
		}

		return $credentials;
	}

	/**
	 * Start Tumblr share process.
	 * 
	 * @param int  $post_id  WP Post ID.
	 * @param bool $instant  Check if instant.
	 */
	public function process_start( $post_id, $instant = false ) {
		$enable = $this->is_enabled( 'tumblr_enable' );
		$accounts = $this->get_activated_accounts();
		if ( $enable && $accounts !== false ) {
		    foreach ( $accounts as $account_id => $data ) {
		    	$this->process_share( $data, $account_id, $post_id, $instant );
		    }
		}
	}

	/**
	 * Process API Interaction with Tumblr and Share Post.
	 * 
	 * @param array  $account     Tumblr Account Data.
	 * @param int    $account_id  Tumblr Account ID.
	 * @param int    $post_id     WP Post ID.
	 * @param bool   $instant     Check if instant.
	 */
	private function process_share( $account, $account_id, $post_id, $instant ) {
		$post = \get_post( $post_id );
		$post_types = $this->get_data( 'tumblr_post_types_display', array( 'post' ) );
		$taxonomies = $this->get_data( 'tumblr_social_taxonomy' );
		$add_thumbnail = $this->get_data( 'tumblr_thumbnail', 'yes' );
		$template = $this->get_data( 'tumblr_template' );
		$content_source = $this->get_data( 'tumblr_content_source' );
		$posting_type = $this->get_data( 'tumblr_posting_type', 'link' );
		$disable_tumblr_share = $this->get_meta( $post->ID, '_wpar_disable_tumblr_share' );

		if ( ( ! in_array( $post->post_type, $post_types ) || $disable_tumblr_share == 'yes' ) && ! $instant ) {
			return;
		}

		$account_template = $account['template'];
		if ( ! empty( $account_template ) ) {
			$template = $account_template;
		}

		$permalink = $this->get_permalink( $post->ID, 'tumblr' );
		$hashtags = $this->get_hashtags( $taxonomies, $post->ID );
		$content = $this->get_social_template( $template, $post->ID, '', $content_source, 280, $permalink );
		$data = array(
			'type'  => $posting_type,
			'title' => wp_strip_all_tags( $post->post_title ),
		);

		if ( $posting_type == 'text' ) {
			$data['body'] = $content;
		} elseif ( $posting_type == 'link' ) {
			$data['url'] = $permalink;
			$data['description'] = esc_html( $content );

			if ( has_post_thumbnail( $post->ID ) && ! $this->is_localhost() ) {
				$data['thumbnail'] = get_the_post_thumbnail_url( $post->ID, 'full' );
			}

			$userdata = \get_userdata( $post->post_author );
			$first_name = $userdata->first_name;
			$last_name = $userdata->last_name;
			$data['author'] = esc_html( $first_name . ' ' . $last_name );

			$data['excerpt'] = ( has_excerpt( $post->ID ) ) ? wp_strip_all_tags( strip_shortcodes( $post->post_excerpt ) ) : esc_html( wp_trim_words( $content, 10 ) );
		}

		if ( ! empty( $hashtags ) && is_array( $hashtags ) && count( $hashtags ) > 0 ) {
			shuffle( $hashtags );
			$data['tags'] = join( ',', $hashtags );
		}
		
		$data = $this->do_filter( 'tumblr_share_args', $data, $post );

		try {
		    // generate access token to fetch profile data
			$connection = $this->connection( $account['app_id'], $account['app_secret'], $account['auth_token'], $account['auth_token_secret'] );

			$result = $connection->createPost( $account['blog_name'], $data );

			$this->set_post_metadata( $post_id, array(
				'share_id'     => $result->id,
				'publish_date' => $this->current_timestamp(),
				'account_id'   => $account_id,
			), 'tumblr' );

			// insert log entry
			$this->do_action( 'insert_log', $post_id, 'tumblr' );
		} catch ( Exception $e ) {
			// insert log entry
			$this->do_action( 'insert_log', $post_id, array(
				'status' => false,
				'info'   => $e->getMessage(),
				'action' => 'tumblr',
			) );
		}
	}
}