<?php
/**
 * The file for Twitter Share.
 *
 * @since      1.2.2
 * @package    RevivePress
 * @subpackage RevivePress\Core\Premium\Social\Providers
 * @author     Sayan Datta <iamsayan@protonmail.com>
 */

namespace RevivePress\Core\Premium\Social\Providers;

use Exception;
use RevivePress\Helpers\Ajax;
use RevivePress\Helpers\Hooker;
use RevivePress\Helpers\Scheduler;
use RevivePress\Helpers\Premium\SocialHelpers;
use RevivePress\Dependencies\Abraham\TwitterOAuth\TwitterOAuth;

defined( 'ABSPATH' ) || exit;

/**
 * Twitter Share class.
 */
class TwitterShare
{
	use Ajax;
    use Hooker;
    use Scheduler;
    use SocialHelpers;

	/**
	 * Register functions.
	 */
	public function register() {
		$this->filter( 'wpar/social/add_account_twitter', 'generate_auth_url', 10, 2 );
		$this->action( 'admin_post_wpar_add_twitter_account', 'get_auth_data' );
		$this->action( 'wpar/process_twitter_share', 'process_start', 5, 2 );
        $this->filter( 'wpar/display_twitter_accounts', 'twitter_accounts_list' );
	}

	/**
	 * Generate Twitter Authentication URL.
	 */
	public function generate_auth_url( $url, $request ) {
		$app_id = sanitize_text_field( wp_unslash( $request->get_param( 'app_id' ) ) );
		$app_secret = sanitize_text_field( wp_unslash( $request->get_param( 'app_secret' ) ) );

		// @codingStandardsIgnoreStart
		$callback_url = add_query_arg( 'state', base64_encode( trailingslashit( admin_url() ) ), $this->get_auth_url( 'twitter' ) );

		try {
			$connection = $this->connection( $app_id, $app_secret );
			$request_token = $connection->oauth( 'oauth/request_token', [ 'oauth_callback' => esc_url( $callback_url ) ] );
	
			if ( isset( $request_token['oauth_token'], $request_token['oauth_token_secret'] ) ) {
				// save twitter credentials
				$this->save_credentials( [
					'app_id'     => $app_id,
					'app_secret' => $app_secret,
				], 'twitter' );
				
				// get Auth URL
				$url = $connection->url( 'oauth/authorize', [ 'oauth_token' => $request_token['oauth_token'] ] );
				
				return $url;
			} else {
				return __( 'Error: Auth Token or Auth Token Secret has error!', 'wp-auto-republish' );
			}
		} catch ( \Exception $e ) {
			$this->error( __( 'Error: Wrong App ID or App Secret or Check your Callback URL!', 'wp-auto-republish' ) );
		}
		// @codingStandardsIgnoreEnd
	}

	/**
	 * Fetch and save Twitter profile data to db.
	 */
	public function get_auth_data() {
		// @codingStandardsIgnoreStart
		if ( isset( $_GET['oauth_token'], $_GET['oauth_verifier'] ) ) {
			// get temporary credentials
			$credentials = $this->get_credentials( 'twitter' );

		    // generate access token to fetch profile data
			$access_token = $this->connection( $credentials['app_id'], $credentials['app_secret'], $_GET['oauth_token'], $_GET['oauth_verifier'] )->oauth( 'oauth/access_token', [ 'oauth_verifier' => $_GET['oauth_verifier'] ] );
			
			// get user data using access tokens
			$content = $this->connection( $credentials['app_id'], $credentials['app_secret'], $access_token['oauth_token'], $access_token['oauth_token_secret'] )->get( 'account/verify_credentials' );
			
			// get current user
			$current_user = wp_get_current_user();

			$args = [
				'id'                 => $content->id,
				'name'               => $content->name,
				'user_name'          => $content->screen_name,
				'status'             => $this->do_filter( 'twitter_default_status', true ),
				'valid'              => true,
				'template'           => '',
				'app_id'             => $credentials['app_id'],
				'app_secret'         => $credentials['app_secret'],
				'oauth_token'        => $access_token['oauth_token'],
				'oauth_token_secret' => $access_token['oauth_token_secret'],
				'timestamp'          => $this->current_timestamp(),
				'added_by'           => $current_user->user_login,
			];

			$this->update_db( $args, 'twitter' );

			// set temporary transient for admin notice
		    set_transient( 'rvp_social_account_added', 'twitter' );

			wp_safe_redirect( $this->get_auth_success_url( 'twitter' ) );
			exit;
		}
		// @codingStandardsIgnoreEnd
	}

	/**
	 * Process API Interaction with Twitter and Share Post.
	 * 
	 * @param string      $app_id               Twitter App ID.
	 * @param string      $app_secret           Twitter App Secret.
	 * @param string|null $oauth_token Twitter  Twitter Auth Token.
	 * @param string      $oauth_token_secret   Twitter Auth Token Secret.
	 * @return object
	 */
	private function connection( $app_id, $app_secret, $oauth_token = null, $oauth_token_secret = null ) {
		$connection = new TwitterOAuth( $app_id, $app_secret, $oauth_token, $oauth_token_secret );

		return $connection;
	}

    /**
	 * Get the list of all twitter accounts.
	 * 
	 * @return string
	 */
	public function twitter_accounts_list() {
		$accounts = get_option( 'wpar_twitter_accounts_db' );
		if ( $accounts === false ) {
            return false;
		}

		$accounts = maybe_unserialize( $accounts );
		if ( empty( $accounts ) ) {
            return false;
		}
		
		$html = '<table class="form-table">
			<thead>
				<tr valign="top">
				    <td class="social-td"><strong>' . __( 'Timestamp', 'wp-auto-republish' ) . '</strong></td>
				    <td class="social-td" scope="row"><strong>' . __( 'Name', 'wp-auto-republish' ) . '</strong></td>
				    <td class="social-td" scope="row"><strong>' . __( 'Handle', 'wp-auto-republish' ) . '</strong></td>
				    <td class="social-td" scope="row"><strong>' . __( 'Added By', 'wp-auto-republish' ) . '</strong></td>
					<td class="social-td" scope="row"><strong>' . __( 'Status', 'wp-auto-republish' ) . '</strong></td>
					<td class="social-td" scope="row"><strong>' . __( 'Template', 'wp-auto-republish' ) . '</strong></td>
				    <td class="social-td" scope="row"><strong>' . __( 'Action', 'wp-auto-republish' ) . '</strong></td>
				</tr>
			</thead>
			<tbody>';

		foreach ( $accounts as $items => $item ) {
			$enabled = ( $item['status'] ) ? __( 'Enabled', 'wp-auto-republish' ) : __( 'Disabled', 'wp-auto-republish' );
			$css_class = ( $item['status'] ) ? 'enabled' : 'disabled';

			$delete = '<a href="#" class="wpar-delete-social-single" data-account-type="twitter" data-account-id="' . $item['id'] . '" data-processing="' . __( 'Deleting...', 'wp-auto-republish' ) . '" data-notice="' . sprintf( 
				/* translators: Twitter account name with user name. */
				__( 'It will delete the Twitter Account: %s from this list. Do you want to continue?', 'wp-auto-republish' ), $item['name'] . ' (@' . $item['user_name'] . ')' 
			) . '">' . __( 'Delete', 'wp-auto-republish' ) . '</a>';

			$activate = '<a href="#" class="wpar-social-single-update" data-account-type="twitter" data-action="activate" data-account-id="' . $item['id'] . '" data-processing="' . __( 'Activating...', 'wp-auto-republish' ) . '" data-next="' . __( 'Enabled', 'wp-auto-republish' ) . '">' . __( 'Activate', 'wp-auto-republish' ) . '</a>';
			$deactivate = '<a href="#" class="wpar-social-single-update" data-account-type="twitter" data-action="deactivate" data-account-id="' . $item['id'] . '" data-processing="' . __( 'Deactivating...', 'wp-auto-republish' ) . '" data-next="' . __( 'Disabled', 'wp-auto-republish' ) . '">' . __( 'Deactivate', 'wp-auto-republish' ) . '</a>';

			$template = '<a href="#" class="wpar-template-social-single" data-account-type="twitter" data-account-id="' . $item['id'] . '">' . __( 'View / Edit Template', 'wp-auto-republish' ) . '</a>';

			$button = ( $item['status'] ) ? $deactivate : $activate;
			$button = ( $item['valid'] ) ? $button : '<span class="wpar-social-account-status expired">' . __( 'Token Expired', 'wp-auto-republish' ) . '</span>';

			$html .= '<tr valign="top" id="wpar-twitter-' . $item['id'] . '" class="wpar-social-accounts">
			    <td class="social-td" scope="row" data-label="' . __( 'Timestamp', 'wp-auto-republish' ) . '">' . date_i18n( 'M j, Y @ ' . get_option( 'time_format' ), $item['timestamp'] ) . '</td>
			    <td class="social-td wpar-twitter-account-' . $item['id'] . '" scope="row" data-label="' . __( 'Name', 'wp-auto-republish' ) . '">' . $item['name'] . '</td>
			    <td class="social-td" scope="row" data-label="' . __( 'Handle', 'wp-auto-republish' ) . '"><a href="https://twitter.com/' . $item['user_name'] . '" target="_blank">@' . $item['user_name'] . '</a></td>
				<td class="social-td" scope="row" data-label="' . __( 'Added By', 'wp-auto-republish' ) . '">' . ucwords( $item['added_by'] ) . '</a></td>
				<td class="social-td wpar-twitter-status-' . $item['id'] . '" scope="row" data-label="' . __( 'Status', 'wp-auto-republish' ) . '"><span class="wpar-social-account-status ' . $css_class . '">' . $enabled . '</span></td>
			    <td class="social-td" scope="row" data-label="' . __( 'Template', 'wp-auto-republish' ) . '">' . $template . '</td>
				<td class="social-td" scope="row" data-label="' . __( 'Action', 'wp-auto-republish' ) . '"><span class="wpar-twitter-action-btn-' . $item['id'] . '">' . $button . '</span> | ' . $delete . '</td>
		    </tr>';
	    }

		$html .= '</tbody>
		</table>';
		
		return $html;
	}

    /**
	 * Get Auth Tokens & Secret of all activated accounts.
	 * 
	 * @return bool|array
	 */
	private function get_activated_accounts() {
		$accounts = get_option( 'wpar_twitter_accounts_db' );
		if ( $accounts === false ) {
            return false;
		}

		$accounts = maybe_unserialize( $accounts );
		if ( empty( $accounts ) ) {
            return false;
		}

		$credentials = array();
		foreach ( $accounts as $items => $item ) {
			if ( $item['status'] ) {
				$credentials[ $item['id'] ] = array(
					'app_id'            => $item['app_id'],
				    'app_secret'        => $item['app_secret'],
					'auth_token'        => $item['oauth_token'], 
					'auth_token_secret' => $item['oauth_token_secret'],
					'template'          => $item['template'],
				);
			}
		}

		if ( empty( $credentials ) ) {
            $credentials = false;
		}

		return $credentials;
	}

	/**
	 * Start Twitter share process.
	 * 
	 * @param int  $post_id  WP Post ID.
	 * @param bool $instant  Check if instant.
	 */
	public function process_start( $post_id, $instant = false ) {
		$enable = $this->is_enabled( 'twitter_enable' );
		$accounts = $this->get_activated_accounts();
		if ( $enable && $accounts !== false ) {
		    foreach ( $accounts as $account_id => $data ) {
		    	$this->process_share( $data, $account_id, $post_id, $instant );
		    }
		}
	}

	/**
	 * Process API Interaction with Twitter and Share Post.
	 * 
	 * @param array  $account     Twitter Auth Data.
	 * @param int    $account_id  Twitter Account ID.
	 * @param int    $post_id     WP Post ID.
	 * @param bool   $instant     Check if instant.
	 */
	private function process_share( $account, $account_id, $post_id, $instant ) {
		$post = \get_post( $post_id );
		$post_types = $this->get_data( 'twitter_post_types_display', array( 'post' ) );
		$taxonomies = $this->get_data( 'twitter_social_taxonomy' );
		$add_thumbnail = $this->get_data( 'twitter_thumbnail', 'yes' );
		$template = $this->get_data( 'twitter_template' );
		$content_source = $this->get_data( 'twitter_content_source' );
		$disable_twitter_share = $this->get_meta( $post->ID, '_wpar_disable_twitter_share' );

		if ( ( ! in_array( $post->post_type, $post_types ) || $disable_twitter_share == 'yes' ) && ! $instant ) {
			return;
		}

		$account_template = $account['template'];
		if ( ! empty( $account_template ) ) {
			$template = $account_template;
		}

		$permalink = $this->get_permalink( $post->ID, 'twitter' );
		$data = array(
            'text' => $this->get_social_template( $template, $post->ID, $this->get_hashtags( $taxonomies, $post->ID ), $content_source, 280, $permalink ),
		);

		try {
		    // generate access token to fetch profile data
			$connection = $this->connection( $account['app_id'], $account['app_secret'], $account['auth_token'], $account['auth_token_secret'] );
			
			if ( has_post_thumbnail( $post->ID ) && $add_thumbnail == 'yes' ) {
				// upload media
				$media = $connection->upload( 'media/upload', array( 'media' => get_attached_file( get_post_thumbnail_id( $post->ID ) ) ) );

				if ( isset( $media->media_id_string ) && is_numeric( $media->media_id_string ) ) {
				    $data['media']['media_ids'] = array( $media->media_id_string );
		        }
			}

			$data = $this->do_filter( 'twitter_share_args', $data, $post );

			$connection->setApiVersion( '2' );
			$result = $connection->post( 'tweets', $data, true );
			$code = $connection->getLastHttpCode();

			if ( isset( $result->data ) && in_array( $code, array( 200, 201 ) ) ) {
				$this->set_post_metadata( $post_id, array(
					'share_id'     => $result->data->id,
					'publish_date' => $this->current_timestamp(),
					'account_id'   => $account_id,
				), 'twitter' );

				// insert log entry
		    	$this->do_action( 'insert_log', $post_id, 'twitter' );
		    } else {
				$message = __( 'Failed!', 'wp-auto-republish' );
				if ( isset( $result->errors ) && ! empty( $result->errors ) ) {
					$code = isset( $result->errors[0]->code ) ? $result->errors[0]->code : $code;
					$message = $result->errors[0]->message;
				} elseif ( isset( $result->error ) && ! empty( $result->error ) ) {
					$code = ( isset( $result->httpstatus ) ) ? $result->httpstatus : $code;
					$message = $result->error;
				}

				if ( $code == 89 ) {
					$this->expire_token( $account_id, 'twitter' );
				}
			
				// insert log entry
				$this->do_action( 'insert_log', $post_id, array(
					'status' => false,
					'info'   => sprintf( 
						/* translators: 1: Twitter Api error message, 2: Twitter Api error code. */
						__( 'Twitter API Connection Problem. Reason: %1$s Error Code: %2$s', 'wp-auto-republish' ), ucwords( $message ), $code 
					),
					'action' => 'twitter',
				) );
            }
		} catch ( Exception $e ) {
			// insert log entry
			$this->do_action( 'insert_log', $post_id, array(
				'status' => false,
				'info'   => $e->getMessage(),
				'action' => 'twitter',
			) );
		}
	}
}