<?php
/**
 * The file for Social REST Actions.
 *
 * @since      1.4.6
 * @package    RevivePress
 * @subpackage RevivePress\Core\Premium
 * @author     Sayan Datta <iamsayan@protonmail.com>
 */

// @codingStandardsIgnoreFile

namespace RevivePress\Core\Premium\Social;

use WP_REST_Server;
use RevivePress\Helpers\Hooker;
use RevivePress\Helpers\Scheduler;
use RevivePress\Helpers\Premium\SocialHelpers;

defined( 'ABSPATH' ) || exit;

/**
 * Social Ajax class.
 */
class RestApi
{
	use Hooker, Scheduler, SocialHelpers;

	/**
	 * REST API Namespace.
	 *
	 * @var string
	 */
	public $namespace = 'revivepress/v1';

	/**
	 * Register functions.
	 */
	public function register() {
		$this->action( 'rest_api_init', 'register_routes' );
	}

	/**
	 * Registers the routes for the objects of the controller.
	 */
	public function register_routes() {
		register_rest_route( $this->namespace, '/getShareButtons', [
			'methods' => WP_REST_Server::EDITABLE,
			'callback' => [ $this, 'get_share_template' ],
			'permission_callback' => function() { return current_user_can( 'edit_posts' ); },
		] );

		register_rest_route( $this->namespace, '/doSocialShare', [
			'methods' => WP_REST_Server::EDITABLE,
			'callback' => [ $this, 'social_share' ],
			'permission_callback' => function() { return current_user_can( 'edit_posts' ); },
		] );

		register_rest_route( $this->namespace, '/viewSocialTemplate', [
			'methods' => WP_REST_Server::EDITABLE,
			'callback' => [ $this, 'view_template' ],
			'permission_callback' => function() { return current_user_can( 'manage_options' ); },
		] );

		register_rest_route( $this->namespace, '/updateSocialTemplate', [
			'methods' => WP_REST_Server::EDITABLE,
			'callback' => [ $this, 'update_template' ],
			'permission_callback' => function() { return current_user_can( 'manage_options' ); },
		] );

		register_rest_route( $this->namespace, '/addSocialAccount', [
			'methods' => WP_REST_Server::EDITABLE,
			'callback' => [ $this, 'add_account' ],
			'permission_callback' => function() { return current_user_can( 'manage_options' ); },
		] );

		register_rest_route( $this->namespace, '/updateSocialAccount', [
			'methods' => WP_REST_Server::EDITABLE,
			'callback' => [ $this, 'update_account' ],
			'permission_callback' => function() { return current_user_can( 'manage_options' ); },
		] );

		register_rest_route( $this->namespace, '/removeSocialAccount', [
			'methods' => WP_REST_Server::EDITABLE,
			'callback' => [ $this, 'remove_account' ],
			'permission_callback' => function() { return current_user_can( 'manage_options' ); },
		] );

		register_rest_route( $this->namespace, '/removeSocialAccounts', [
			'methods' => WP_REST_Server::EDITABLE,
			'callback' => [ $this, 'remove_database' ],
			'permission_callback' => function() { return current_user_can( 'manage_options' ); },
		] );

		// Provider specific
		register_rest_route( $this->namespace, '/updatePinterestAccount', [
			'methods' => WP_REST_Server::EDITABLE,
			'callback' => [ $this, 'update_pinterest_account' ],
			'permission_callback' => function() { return current_user_can( 'manage_options' ); },
		] );
	}

	/**
	 * Get availble social providers list.
	 */
	public function get_share_template() {
		$template = [];
		foreach( $this->get_social_providers() as $provider ) {
			if ( $this->is_enabled( $provider . '_enable' ) ) {
				$template[] = sprintf( 
					'<div class="wpar-social-share"><button class="wpar-social-share-btn ' . esc_attr( $provider ) . '" data-provider="' . esc_attr( $provider ) . '"><i class="fab fa-' . esc_attr( $provider ) . '"></i>%s</button></div>', 
					sprintf( 
						/* translators: Social Provider name. */
						__( 'Share on %s', 'wp-auto-republish' ), ucfirst( $provider ) 
					) 
				);
			}
		}
		if ( ! empty( $template ) && count( $template ) > 1 ) {
			$template[] = sprintf( '<div class="wpar-social-share"><button class="wpar-social-share-btn share-all"><i class="fas fa-share"></i>%s</button></div>', __( 'Share on All', 'wp-auto-republish' ) );
		}

		return ! empty( $template ) ? join( '', $template ) : __( 'No Social Account is active!', 'wp-auto-republish' );
	}

	/**
	 * Initialize Social Share.
	 * 
	 * @param WP_REST_Request $request Full details about the request.
	 *
	 * @return WP_REST_Response|WP_Error Response object on success, or WP_Error object on failure.
	 */
	public function social_share( \WP_REST_Request $request ) {
		$provider = $request->get_param( 'provider' );
		$post_id = intval( $request->get_param( 'post_id' ) );

		if ( $this->is_enabled( $provider . '_enable' ) ) {
			return $this->schedule_single_action( time(), 'wpar/process_' . $provider . '_share', [ $post_id, true ] );
		}

		return true;
	}

	/**
	 * Activate or Deactivate Single Social account on request.
	 * 
	 * @param WP_REST_Request $request Full details about the request.
	 *
	 * @return WP_REST_Response|WP_Error Response object on success, or WP_Error object on failure.
	 */
	public function view_template( \WP_REST_Request $request ) {
		$account_id = $request->get_param( 'account_id' );
		$account_type = $request->get_param( 'account_type' );

		$data = maybe_unserialize( get_option( 'wpar_' . $account_type . '_accounts_db' ) );
		$template = false;
		
		if ( isset( $data[ $account_id ]['template'] ) ) {
			$template = $data[ $account_id ]['template'];
		}

		return $template;
	}

	/**
	 * Activate or Deactivate Single Social account on request.
	 * 
	 * @param WP_REST_Request $request Full details about the request.
	 *
	 * @return WP_REST_Response|WP_Error Response object on success, or WP_Error object on failure.
	 */
	public function update_template( \WP_REST_Request $request ) {
		$account_id = $request->get_param( 'account_id' );
		$account_type = $request->get_param( 'account_type' );
		$template = $request->get_param( 'template' );

		$option_name = 'wpar_' . $account_type . '_accounts_db';
		$data = maybe_unserialize( get_option( $option_name ) );
		
		if ( isset( $data[ $account_id ] ) ) {
			$data[ $account_id ]['template'] = $template;
			return update_option( $option_name, maybe_serialize( $data ), false );
		}

		return true;
	}

	/**
	 * Add New Social Account on request.
	 *
	 * @param WP_REST_Request $request Full details about the request.
	 *
	 * @return WP_REST_Response|WP_Error Response object on success, or WP_Error object on failure.
	 */
	public function add_account( \WP_REST_Request $request ) {
		$action = $request->get_param( 'action' );

		return apply_filters( 'wpar/social/add_account_' . $action, 'Something went wrong.', $request );
	}

	/**
	 * Activate or Deactivate Single Social account on request.
	 * 
	 * @param WP_REST_Request $request Full details about the request.
	 *
	 * @return WP_REST_Response|WP_Error Response object on success, or WP_Error object on failure.
	 */
	public function update_account( \WP_REST_Request $request ) {
		$account_id = $request->get_param( 'account_id' );
		$account_type = $request->get_param( 'account_type' );
		$action = $request->get_param( 'action' );

		$option_name = 'wpar_' . $account_type . '_accounts_db';
		$data = maybe_unserialize( get_option( $option_name ) );

		if ( isset( $data[ $account_id ] ) ) {
			if ( 'activate' === $action ) {
				$data[ $account_id ]['status'] = true;
			} elseif ( 'deactivate' === $action ) {
				$data[ $account_id ]['status'] = false;
			}
			return update_option( $option_name, maybe_serialize( $data ), false );
		}

		return true;
	}

	/**
	 * Single Social account delete on request.
	 * 
	 * @param WP_REST_Request $request Full details about the request.
	 *
	 * @return WP_REST_Response|WP_Error Response object on success, or WP_Error object on failure.
	 */
	public function remove_account( \WP_REST_Request $request ) {
		$account_id = $request->get_param( 'account_id' );
		$account_type = $request->get_param( 'account_type' );

		$option_name = 'wpar_' . $account_type . '_accounts_db';
		$data = maybe_unserialize( get_option( $option_name ) );

		if ( isset( $data[ $account_id ] ) ) {
			unset( $data[ $account_id ] );
			return update_option( $option_name, maybe_serialize( $data ), false );
		}

		return true;
	}

	/**
	 * Social profile database delete on request.
	 * 
	 * @param WP_REST_Request $request Full details about the request.
	 *
	 * @return WP_REST_Response|WP_Error Response object on success, or WP_Error object on failure.
	 */
	public function remove_database( \WP_REST_Request $request ) {
		$account_type = $request->get_param( 'account_type' );

		return delete_option( 'wpar_' . $account_type . '_accounts_db' );
	}

	/**
	 * Activate or Deactivate Single Social account on request.
	 * 
	 * @param WP_REST_Request $request Full details about the request.
	 *
	 * @return WP_REST_Response|WP_Error Response object on success, or WP_Error object on failure.
	 */
	public function update_pinterest_account( \WP_REST_Request $request ) {
		$account_id = $request->get_param( 'account_id' );
		$type = $request->get_param( 'type' );
		$value = $request->get_param( 'value' );

		$option_name = 'wpar_pinterest_accounts_db';
		$data = maybe_unserialize( get_option( $option_name ) );

		if ( isset( $data[ $account_id ] ) ) {
			if ( 'board' === $type ) {
				$board_id = intval( $value );
				$data[ $account_id ]['status'] = boolval( ! empty( $board_id ) );
				$data[ $account_id ]['default_board'] = $board_id;
				$data[ $account_id ]['default_section'] = '';
			} else {
				$data[ $account_id ]['default_section'] = intval( $value );
			}

			return update_option( $option_name, maybe_serialize( $data ), false );
		}

		return true;
	}
}