<?php
/**
 * The file for Single Republish.
 *
 * @since      1.1.0
 * @package    RevivePress
 * @subpackage RevivePress\Core\Premium
 * @author     Sayan Datta <iamsayan@protonmail.com>
 */

namespace RevivePress\Core\Premium;

use RevivePress\Helpers\Hooker;
use RevivePress\Helpers\HelperFunctions;

defined( 'ABSPATH' ) || exit;

/**
 * Single post republish class.
 */
class TriggerEvent
{
	use HelperFunctions;
    use Hooker;

	/**
	 * Register functions.
	 */
	public function register() {
		$this->action( 'wpar/old_post_republished', 'perform_actions', 15, 3 );
	}

	/**
	 * Update required post metas.
	 *
	 * @param int    $post_id  Post ID.
	 * @param object $post     WP Post object.
	 */
	public function perform_actions( $post_id, $republished_time, $action ) {
		$original_post_id = $post_id;
		
		$clone_post_id = $this->get_meta( $post_id, 'wpar_original_post_id' );
		if ( $action == 'clone' && $clone_post_id ) {
			$original_post_id = $clone_post_id;
			$this->delete_meta( $post_id, 'wpar_original_post_id' );
		}

		$reposted = maybe_unserialize( $this->get_meta( $original_post_id, '_wpar_repost_date' ) );
		if ( empty( $reposted ) || ! is_array( $reposted ) ) {
			$reposted = array();
		}
	
		$reposted[] = $republished_time;
		
		$this->update_meta( $original_post_id, '_wpar_repost_date', maybe_serialize( $reposted ) );

		// get post.
		$post = get_post( $post_id );

		if ( ! $this->is_enabled( 'disable_guid_reneration' ) ) {
			$this->update_guid( $post );
		}

		if ( ! $this->is_enabled( 'enable_silent_republishing', true ) ) {
			$this->trigger_publish( $post );
		}
	}

	/**
	 * Update GUID.
	 * 
	 * @since 1.4.2
	 * 
	 * @param object $post  WP_Post Object
	 */
	private function update_guid( $post ) {
		global $wpdb;

		$wp_guid = wp_generate_uuid4();
		$old_post_id = $post->ID;
		$old_guid = $this->get_meta( $old_post_id, 'rvp_previous_guid' );

		if ( $old_guid ) {
			$new_guid = str_replace( $old_guid, $wp_guid, $post->guid, $count );
			if ( ! $count ) {
				$new_guid = $post->guid . '_' . $wp_guid;
			}
			$this->update_meta( $old_post_id, 'rvp_previous_guid', $wp_guid, $old_guid );
		} else {
			$new_guid = $post->guid . '---' . $wp_guid;
			$this->update_meta( $old_post_id, 'rvp_previous_guid', $wp_guid );
		}

		$wpdb->update( $wpdb->posts, array( 'guid' => $new_guid ), array( 'ID' => $old_post_id ) );
	}

	/**
	 * Trigger publish event.
	 * 
	 * @since 1.1.7
	 * 
	 * @param object $post  WP_Post Object
	 */
	private function trigger_publish( $post ) {
		// trigger publish event
		$old_status = $post->post_status;
		$new_status = $this->do_filter( 'fake_post_status', 'pending' );

		if ( 'publish' === $old_status ) {
			// set post status to draft forcefully
			$post->post_status = $new_status;
		}
		
		// clear cache
		$clear_cache = $this->do_filter( 'enable_single_post_cache', true, $post->ID );
		if ( $clear_cache ) {
			$this->do_action( 'single_post_cache', $post->ID, $post );
		}

		if ( 'publish' !== $old_status ) {
			return;
		}

		// Jetpack publicize.
		$do_publicize = $this->do_filter( 'enable_jetpack_publicize', true, $post->ID );
		if ( $do_publicize ) {
			// update jetpack meta to available it for publicize
			$this->update_meta( $post->ID, '_publicize_pending', true );
			$this->delete_meta( $post->ID, '_wpas_done_all' );
		}

		// Handle OneSignal.
		$do_onesignal = $this->do_filter( 'do_onesignal', true, $post->ID, $post );
		if ( $do_onesignal ) {
			// enable onesignal
			\add_filter( 'onesignal_include_post', '__return_true' );
		}

		\wp_transition_post_status( 'publish', $new_status, $post );

		// Check if originally has filter or not.
		if ( $do_onesignal ) {
			// disable onesignal
			\remove_filter( 'onesignal_include_post', '__return_true' );
		}

		/** This action is documented in wp-includes/post.php */
		\do_action( "edit_post_{$post->post_type}", $post->ID, $post );

		/** This action is documented in wp-includes/post.php */
		\do_action( 'edit_post', $post->ID, $post );

		/** This action is documented in wp-includes/post.php */
		\do_action( "save_post_{$post->post_type}", $post->ID, $post, true );

		/** This action is documented in wp-includes/post.php */
		\do_action( 'save_post', $post->ID, $post, true );

		/** This action is documented in wp-includes/post.php */
		\do_action( 'wp_insert_post', $post->ID, $post, true );
	}
}