<?php
/**
 * Copyright 2017 Facebook, Inc.
 *
 * You are hereby granted a non-exclusive, worldwide, royalty-free license to
 * use, copy, modify, and distribute this software in source code or binary
 * form for use in connection with the web services and APIs provided by
 * Facebook.
 *
 * As with any software that integrates with the RevivePress\Dependencies\Facebook platform, your use
 * of this software is subject to the RevivePress\Dependencies\Facebook Developer Principles and
 * Policies [http://developers.facebook.com/policy/]. This copyright notice
 * shall be included in all copies or substantial portions of the software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 *
 */

namespace RevivePress\Dependencies\Facebook\HttpClients;

use RevivePress\Dependencies\Facebook\Http\GraphRawResponse;
use RevivePress\Dependencies\Facebook\Exceptions\FacebookSDKException;

use GuzzleHttp\Client;
use GuzzleHttp\Exception\GuzzleException;
use Psr\Http\Message\ResponseInterface;

class FacebookGuzzleHttpClient implements FacebookHttpClientInterface
{
    /**
     * @var Client The Guzzle client.
     */
    protected $guzzleClient;

    /**
     * @param Client|null $guzzleClient The Guzzle client.
     */
    public function __construct(Client $guzzleClient = null)
    {
        $this->guzzleClient = $guzzleClient ?: new Client();
    }

    /**
     * @inheritdoc
     */
    public function send($url, $method, $body, array $headers, $timeOut)
    {
        $options = [
            'headers'         => $headers,
            'body'            => $body,
            'timeout'         => $timeOut,
            'http_errors'     => false,
            'connect_timeout' => 10,
            'verify'          => __DIR__ . '/certs/DigiCertHighAssuranceEVRootCA.pem',
        ];

        try {

            $rawResponse = $this->guzzleClient->request($method, $url, $options);

            return new GraphRawResponse(
                $this->getHeadersAsString($rawResponse),
                $rawResponse->getBody(),
                $rawResponse->getStatusCode()
            );

        } catch (GuzzleException $e) {
            throw new FacebookSDKException($e->getMessage(), $e->getCode());
        }

    }

    /**
     * Returns the Guzzle array of headers as a string.
     *
     * @param ResponseInterface $response The Guzzle response.
     *
     * @return string
     */
    public function getHeadersAsString(ResponseInterface $response)
    {
        $headers = $response->getHeaders();
        $rawHeaders = [];
        foreach ($headers as $name => $values) {
            $rawHeaders[] = $name . ": " . implode(", ", $values);
        }

        return implode("\r\n", $rawHeaders);
    }
}
