<?php
/**
 * Social Helper functions.
 *
 * @since      1.2.2
 * @package    RevivePress
 * @subpackage RevivePress\Helpers
 * @author     Sayan Datta <iamsayan@protonmail.com>
 */

namespace RevivePress\Helpers\Premium;

use RevivePress\Helpers\HelperFunctions;

defined( 'ABSPATH' ) || exit;

/**
 * Meta & Option class.
 */
trait SocialHelpers
{
	use HelperFunctions;

	/**
	 * Save the Social Profile Data to db.
	 *
	 * @param  array   $args  Social Profile Data.
	 * @param  string  $type  Social account Type.
	 */
	protected function update_db( $args, $type ) {
		$data = maybe_unserialize( get_option( 'wpar_' . $type . '_accounts_db' ) );
		if ( empty( $data ) ) $data = array();

		$data[ $args['id'] ] = $args;

		update_option( 'wpar_' . $type . '_accounts_db', maybe_serialize( $data ), false );
	}

	/**
	 * Expire social account if access token is expired.
	 *
	 * @param string  $account_id  Social Profile ID.
	 * @param string  $type        Social account Type.
	 */
	protected function expire_token( $account_id, $type ) {
        $data = maybe_unserialize( get_option( 'wpar_' . $type . '_accounts_db' ) );
		if ( isset( $data[ $account_id ] ) ) {
			$data[ $account_id ]['status'] = false;
			$data[ $account_id ]['valid'] = false;

			update_option( 'wpar_' . $type . '_accounts_db', maybe_serialize( $data ), false );
		}
	}

	/**
	 * Save Twitter share logs as post meta.
	 * 
	 * @param int     $post_id WP Post ID.
	 * @param array   $args    Share Log data
	 * @param string  $type    Social account Type.
	 */
	protected function set_post_metadata( $post_id, $args, $type ) {
		$share_logs = maybe_unserialize( $this->get_meta( $post_id, 'wpar_' . $type . '_share_logs' ) );
		if ( empty( $share_logs ) ) $share_logs = array();
	
		$share_logs[] = $args;

		$this->update_meta( $post_id, 'wpar_' . $type . '_share_logs', maybe_serialize( $share_logs ) );
	}

	/**
	 * Generate Hashtags for social share.
	 * 
	 * @param array $taxonomies  Taxonomies Array
	 * @param int   $post_id     WP Post ID.
	 * @return array
	 */
	protected function get_hashtags( $taxonomies, $post_id ) {
		$terms = array();
        if ( ! empty( $taxonomies ) ) {
	    	foreach ( $taxonomies as $taxonomy ) {
				$taxonomy_data = $this->process_taxonomy( $taxonomy );
				$taxonomy_name = $taxonomy_data[0];
    			$term_id = $taxonomy_data[1];

				// Check if actually the term is associated with the post or not.
	    		if ( has_term( $term_id, $taxonomy_name, $post_id ) ) {
					$terms[] = str_replace( array( ' ', '-' ), '_', get_term( $term_id, $taxonomy_name )->name );
	    		}
	    	}
		}
		
		return $terms;
	}

	/**
	 * Generate Social Share Template.
	 *
	 * @param string $template   Template Structure from DB. 
	 * @param int    $post_id    WP Post ID.
	 * @param array  $hashtags   Post Hashtags
	 * @param string $source     Post Content Source.
	 * @param int    $length     Maximum Allowed Content Length.
	 * @param string $post_link  WordPress Post Link.
	 * 
	 * @return array $template
	 */
    protected function get_social_template( $template, $post_id, $hashtags, $source, $length, $post_link ) {
		$post = \get_post( $post_id );
		$post_title = wp_strip_all_tags( $post->post_title );
		$post_link = esc_url( $post_link );
        $excerpt = ( has_excerpt( $post->ID ) ) ? wp_strip_all_tags( strip_shortcodes( $post->post_excerpt ) ) : '';
		$post_content = ( $source == 'post_excerpt' ) ? $excerpt : wp_strip_all_tags( strip_shortcodes( $post->post_content ) );
		
		$hashtag = '';
        if ( ! empty( $hashtags ) && is_array( $hashtags ) && count( $hashtags ) > 0 ) {
			shuffle( $hashtags );
			$hashtag = '#' . implode( ' #', $hashtags );
        }

		$userdata = \get_userdata( $post->post_author );
    	$first_name = $userdata->first_name;
        $last_name = $userdata->last_name;
		$full_author = $first_name . ' ' . $last_name;
		$nickname_author = \get_user_meta( $post->post_author, 'nickname', true );

		$post_type = ucwords( str_replace( array( '-', '_' ), ' ', $post->post_type ) );

		$search_arr = $this->get_available_tags();
        $replace_arr = array( $post_title, $post_link, $post_content, $hashtag, $full_author, $nickname_author, $post_type, $first_name, $last_name, get_option( 'blogname' ), get_site_url(), $excerpt, uniqid(), $post->ID, wp_get_shortlink( $post->ID ) );

		// replace post content with length
		$code_matches = array();
        if ( preg_match_all( '/\%(post_content)(_)(\d*)\%/', $template, $code_matches ) ) {
            $trim_tag = $code_matches[0][0];
            $trim_length = $code_matches[3][0];
            $post_content = substr( $post_content, 0, $trim_length );
			$search_arr[] = $trim_tag;
            $replace_arr[] = $post_content;
        }

		// replace custom fields
		$matches = array();
		if ( preg_match_all( '/\%(cf)(_)(\S*)\%/', $template, $matches ) ) {
			foreach ( $matches[0] as $key => $value ) {
				$search_arr[] = $value;
			}

			foreach ( $matches[3] as $key => $value ) {
				$field_name = $value;
				$tag_value = '';

				if ( $field_name ) {
					$tag_value = $this->get_meta( $post_id, $field_name );
					if ( ! $tag_value || is_array( $tag_value ) ) {
						$tag_value = '';
					}
				}

				$replace_arr[] = $tag_value;
			}
		}

		// replace template tags
		$template = str_replace( $search_arr, $replace_arr, $template );

		return $this->short_content( html_entity_decode( $template ), $length );
    }

	/**
	 * Short Content.
	 * 
	 * @since 1.3.4
	 */
	protected function short_content( $content, $length ) {
        $short_content = $content;
        ++$length;

        if ( strlen( $content ) > $length ) {
            $subex = substr( $content, 0, $length - 5 );
            $exwords = explode( ' ', $subex );
            $excut = - ( strlen( $exwords[ count( $exwords ) - 1 ] ) );
            if ( $excut < 0 ) {
                $short_content = substr( $subex, 0, $excut );
            } else {
                $short_content = $subex;
            }
        }

        return $short_content;
    }

	/**
	 * Get Post Permalinks
	 * 
	 * @since 1.3.2
	 * @return string
	 */
	protected function get_permalink( $post_id, $provider ) {
		$permalink = get_permalink( $post_id );
		$shortner = $this->get_data( 'link_shortner' );

		if ( $shortner === 'WordPress' ) {
			$permalink = wp_get_shortlink( $post_id );
		}
		
		$permalink = $this->get_parameters( $permalink, $provider );
		if ( $this->get_data( 'unique_posting' ) ) {
			$permalink = add_query_arg( 'rvp', uniqid(), $permalink );
		}

		$permalink = $this->do_filter( "post_permalink_$shortner", $permalink, $provider, $post_id );

		return $permalink;
	}

	/**
	 * Get Auth success URL.
	 * 
	 * @since 1.4.6
	 * @param string $provider Social account Type.
	 */
	protected function get_parameters( $permalink, $provider ) {
		$parameters = $this->get_data( 'url_patameters' );
		$provider_parameters = $this->get_data( $provider . '_url_patameters' );
		if ( ! empty( $provider_parameters ) ) {
			$parameters = $provider_parameters;
		}
		if ( ! empty( $parameters ) ) {
			$parameters = str_replace( '%provider%', $provider, $parameters );
			$data = array();
			parse_str( $parameters, $data );
			$permalink = add_query_arg( $data, $permalink );
		}
	
		return $permalink;
	}

	/**
	 * Save the Social Profile Data to db.
	 *
	 * @param  array   $args  Social Profile Data.
	 * @param  string  $type  Social account Type.
	 */
	protected function get_credentials( $type, $field = '' ) {
		$data = get_option( 'wpar_social_credentials' );
		if ( empty( $data ) ) $data = array();

		if ( isset( $data[ $type ] ) ) {
			if ( ! empty( $field ) ) {
				return isset( $data[ $type ][ $field ] ) ? $data[ $type ][ $field ] : false;
			}
			return $data[ $type ];
		}
		return false;
	}

	/**
	 * Save the Social Profile Data to db.
	 *
	 * @param  array   $args  Social Profile Data.
	 * @param  string  $type  Social account Type.
	 */
	protected function save_credentials( $args, $type ) {
		$data = get_option( 'wpar_social_credentials' );
		if ( empty( $data ) ) $data = array();

		$data[ $type ] = $args;

		update_option( 'wpar_social_credentials', $data, false );
	}

	/**
	 * Get Auth URL.
	 * 
	 * @since 1.3.0
	 * @param string $provider Social account Type.
	 */
	protected function get_auth_url( $provider ) {
		return 'https://api.wprevivepress.com/auth/' . $provider;
	}

	/**
	 * Get Auth success URL.
	 * 
	 * @since 1.3.0
	 * @param string $provider Social account Type.
	 */
	protected function get_auth_success_url( $provider ) {
		return 'https://api.wprevivepress.com/auth/success?provider=' . $provider;
	}

	/**
	 * Get avaialable template tags.
	 * 
	 * @since 1.3.2
	 * @return string
	 */
	protected function get_available_tags() {
		$tags = array( '%post_title%', '%post_url%', '%post_content%', '%hashtags%', '%post_author_fullname%', '%post_author_nickname%', '%post_type%', '%author_first_name%', '%author_last_name%', '%site_name%', '%site_url%', '%post_excerpt%', '%unique_id%', '%post_id%', '%short_link%', '%post_content_LENGTH%' );
		
		return $tags;
	}

	/**
	 * Check is localhost.
	 * 
	 * @since 1.4.0
	 * @return bool
	 */
	protected function is_localhost() {
        $server_ip = null;

		if ( defined( 'INPUT_SERVER' ) && filter_has_var( INPUT_SERVER, 'REMOTE_ADDR' ) ) {
			$server_ip = filter_input( INPUT_SERVER, 'REMOTE_ADDR', FILTER_VALIDATE_IP );
		} elseif ( defined( 'INPUT_ENV' ) && filter_has_var( INPUT_ENV, 'REMOTE_ADDR' ) ) {
			$server_ip = filter_input( INPUT_ENV, 'REMOTE_ADDR', FILTER_VALIDATE_IP );
		} elseif ( isset( $_SERVER['REMOTE_ADDR'] ) ) {
			$server_ip = filter_var( $_SERVER['REMOTE_ADDR'], FILTER_VALIDATE_IP ); // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.MissingUnslash
		}

		if ( empty( $server_ip ) ) {
			$server_ip = '127.0.0.1';
		}

		return empty( filter_var( $server_ip, FILTER_VALIDATE_IP, FILTER_FLAG_NO_RES_RANGE | FILTER_FLAG_NO_PRIV_RANGE ));
    }

	/**
	 * Get Social providers.
	 * 
	 * @since 1.4.0
	 * @return array
	 */
	protected function get_social_providers() {
		return array( 'facebook', 'twitter', 'linkedin', 'pinterest', 'tumblr' );
	}

	/**
	 * Filter text.
	 * 
	 * @since 1.5.4
	 * @return string
	 */
	protected function filter_little_text( $text ) {
        $filtered_text = preg_replace_callback( '/([\(\)\{\}\[\]])|([@*<>|\\\\\_~])/m', function ( $matches ) {
            return '\\'.$matches[0];
        }, $text );

        return substr( $filtered_text, 0, 4086 );
    }
}